@@ -11,11 +11,16 @@
    GNU General Public License for more details.
 ***********************************************************************/
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include <assert.h>
 
 #include "government.h"
 #include "packets.h"
 #include "spaceship.h"
+
 #include "spacerace.h"
 
 #include "advspace.h"
@@ -25,59 +30,53 @@
  *  AJS, 19990610
  */
 
-int ai_spaceship_autoplace(struct player *pplayer, struct player_spaceship *ship)
+bool ai_spaceship_autoplace(struct player *pplayer, struct player_spaceship *ship)
 {
-  struct government *g = get_gov_pplayer(pplayer);
-  struct packet_spaceship_action packet;
-  int i,retval=0;
+  enum spaceship_place_type type;
+  int num, i;
+  bool retval = FALSE;
   
   while (ship->modules > (ship->habitation + ship->life_support
 		       + ship->solar_panels)) {
     
-    int nice = government_has_hint(g, G_IS_NICE);
-    /* "nice" governments prefer to keep success 100%;
-     * others build habitation first (for score?)  (Thanks Massimo.)
-     */
-
-    packet.action =
-      (ship->habitation==0)   ? SSHIP_ACT_PLACE_HABITATION :
-      (ship->life_support==0) ? SSHIP_ACT_PLACE_LIFE_SUPPORT :
-      (ship->solar_panels==0) ? SSHIP_ACT_PLACE_SOLAR_PANELS :
+    type =
+      (ship->habitation==0)   ? SSHIP_PLACE_HABITATION :
+      (ship->life_support==0) ? SSHIP_PLACE_LIFE_SUPPORT :
+      (ship->solar_panels==0) ? SSHIP_PLACE_SOLAR_PANELS :
       ((ship->habitation < ship->life_support)
        && (ship->solar_panels*2 >= ship->habitation + ship->life_support + 1))
-                              ? SSHIP_ACT_PLACE_HABITATION :
+                              ? SSHIP_PLACE_HABITATION :
       (ship->solar_panels*2 < ship->habitation + ship->life_support)
-                              ? SSHIP_ACT_PLACE_SOLAR_PANELS :
+                              ? SSHIP_PLACE_SOLAR_PANELS :
       (ship->life_support<ship->habitation)
-                              ? SSHIP_ACT_PLACE_LIFE_SUPPORT :
-      (nice && (ship->life_support <= ship->habitation)
+                              ? SSHIP_PLACE_LIFE_SUPPORT :
+      ((ship->life_support <= ship->habitation)
        && (ship->solar_panels*2 >= ship->habitation + ship->life_support + 1))
-                              ? SSHIP_ACT_PLACE_LIFE_SUPPORT :
-      (nice)                  ? SSHIP_ACT_PLACE_SOLAR_PANELS :
-                                SSHIP_ACT_PLACE_HABITATION;
+                              ? SSHIP_PLACE_LIFE_SUPPORT :
+                                SSHIP_PLACE_SOLAR_PANELS;
 
-    if (packet.action == SSHIP_ACT_PLACE_HABITATION) {
-      packet.num = ship->habitation + 1;
-    } else if(packet.action == SSHIP_ACT_PLACE_LIFE_SUPPORT) {
-      packet.num = ship->life_support + 1;
+    if (type == SSHIP_PLACE_HABITATION) {
+      num = ship->habitation + 1;
+    } else if(type == SSHIP_PLACE_LIFE_SUPPORT) {
+      num = ship->life_support + 1;
     } else {
-      packet.num = ship->solar_panels + 1;
+      num = ship->solar_panels + 1;
     }
-    assert(packet.num<=NUM_SS_MODULES/3);
+    assert(num <= NUM_SS_MODULES / 3);
 
-    handle_spaceship_action(pplayer, &packet);
-    retval=1;
+    handle_spaceship_place(pplayer, type, num);
+    retval = TRUE;
   }
   while (ship->components > ship->fuel + ship->propulsion) {
     if (ship->fuel <= ship->propulsion) {
-      packet.action = SSHIP_ACT_PLACE_FUEL;
-      packet.num = ship->fuel + 1;
+      type = SSHIP_PLACE_FUEL;
+      num = ship->fuel + 1;
     } else {
-      packet.action = SSHIP_ACT_PLACE_PROPULSION;
-      packet.num = ship->propulsion + 1;
+      type = SSHIP_PLACE_PROPULSION;
+      num = ship->propulsion + 1;
     }
-    handle_spaceship_action(pplayer, &packet);
-    retval=1;
+    handle_spaceship_place(pplayer, type, num);
+    retval = TRUE;
   }
   while (ship->structurals > num_spaceship_structurals_placed(ship)) {
     /* Want to choose which structurals are most important.
@@ -89,9 +88,9 @@ int ai_spaceship_autoplace(struct player *pplayer, struct player_spaceship *ship
     
     if (!ship->structure[0]) {
       /* if we don't have the first structural, place that! */
-      packet.action = SSHIP_ACT_PLACE_STRUCTURAL;
-      packet.num = 0;
-      handle_spaceship_action(pplayer, &packet);
+      type = SSHIP_PLACE_STRUCTURAL;
+      num = 0;
+      handle_spaceship_place(pplayer, type, num);
     }
     
     if (ship->habitation >= 1
@@ -146,10 +145,10 @@ int ai_spaceship_autoplace(struct player *pplayer, struct player_spaceship *ship
     while(!ship->structure[structurals_info[req].required]) {
       req = structurals_info[req].required;
     }
-    packet.action = SSHIP_ACT_PLACE_STRUCTURAL;
-    packet.num = req;
-    handle_spaceship_action(pplayer, &packet);
-    retval=1;
+    type = SSHIP_PLACE_STRUCTURAL;
+    num = req;
+    handle_spaceship_place(pplayer, type, num);
+    retval = TRUE;
   }
   return retval;
 }
