@@ -72,7 +72,7 @@ static struct {
       int wait_at_network, wait_at_network_requests;
     } stats;
   } entries[MAX_AGENTS];
-  struct call_list calls;
+  struct call_list *calls;
 } agents;
 
 static bool initialized = FALSE;
@@ -152,7 +152,7 @@ static void enqueue_call(struct my_agent *agent,
     }
   } call_list_iterate_end;
 
-  call_list_insert(&agents.calls, pcall2);
+  call_list_prepend(agents.calls, pcall2);
 
   if (DEBUG_TODO_LISTS) {
     freelog(LOG_NORMAL, "A: adding call");
@@ -180,15 +180,15 @@ static struct call *remove_and_return_a_call(void)
 {
   struct call *result;
 
-  if (call_list_size(&agents.calls) == 0) {
+  if (call_list_size(agents.calls) == 0) {
     return NULL;
   }
 
   /* get calls to agents with low levels first */
-  call_list_sort(&agents.calls, my_call_sort);
+  call_list_sort(agents.calls, my_call_sort);
 
-  result = call_list_get(&agents.calls, 0);
-  call_list_unlink(&agents.calls, result);
+  result = call_list_get(agents.calls, 0);
+  call_list_unlink(agents.calls, result);
 
   if (DEBUG_TODO_LISTS) {
     freelog(LOG_NORMAL, "A: removed call");
@@ -277,7 +277,7 @@ static void thaw(void)
   }
   frozen_level--;
   assert(frozen_level >= 0);
-  if (frozen_level == 0 && get_client_state() == CLIENT_GAME_RUNNING_STATE) {
+  if (0 == frozen_level && C_S_RUNNING == client_state()) {
     call_handle_methods();
   }
 }
@@ -341,7 +341,7 @@ static void print_stats(struct my_agent *agent)
 void agents_init(void)
 {
   agents.entries_used = 0;
-  call_list_init(&agents.calls);
+  agents.calls = call_list_new();
 
   /* Add init calls of agents here */
   cma_init();
@@ -362,6 +362,8 @@ void agents_free(void)
    * let the OS free the memory on exit instead of doing it ourselves. */
   /* cmafec_free(); */
 
+  /*simple_historian_done();*/
+
   for (;;) {
     struct call *pcall = remove_and_return_a_call();
     if (!pcall) {
@@ -376,6 +378,7 @@ void agents_free(void)
 
     free_timer(agent->stats.network_wall_timer);
   }
+  call_list_free(agents.calls);
 }
 
 /***********************************************************************
@@ -516,8 +519,10 @@ void agents_unit_changed(struct unit *punit)
 
   freelog(LOG_DEBUG,
 	  "A: agents_unit_changed(unit=%d) type=%s pos=(%d,%d) owner=%s",
-	  punit->id, unit_types[punit->type].name, TILE_XY(punit->tile),
-	  unit_owner(punit)->name);
+	  punit->id,
+	  unit_rule_name(punit),
+	  TILE_XY(punit->tile),
+	  player_name(unit_owner(punit)));
 
   for (i = 0; i < agents.entries_used; i++) {
     struct my_agent *agent = &agents.entries[i];
@@ -542,8 +547,10 @@ void agents_unit_new(struct unit *punit)
 
   freelog(LOG_DEBUG,
 	  "A: agents_new_unit(unit=%d) type=%s pos=(%d,%d) owner=%s",
-	  punit->id, unit_types[punit->type].name, TILE_XY(punit->tile),
-	  unit_owner(punit)->name);
+	  punit->id,
+	  unit_rule_name(punit),
+	  TILE_XY(punit->tile),
+	  player_name(unit_owner(punit)));
 
   for (i = 0; i < agents.entries_used; i++) {
     struct my_agent *agent = &agents.entries[i];
@@ -569,8 +576,10 @@ void agents_unit_remove(struct unit *punit)
 
   freelog(LOG_DEBUG,
 	  "A: agents_remove_unit(unit=%d) type=%s pos=(%d,%d) owner=%s",
-	  punit->id, unit_types[punit->type].name, TILE_XY(punit->tile),
-	  unit_owner(punit)->name);
+	  punit->id,
+	  unit_rule_name(punit),
+	  TILE_XY(punit->tile),
+	  player_name(unit_owner(punit)));
 
   for (i = 0; i < agents.entries_used; i++) {
     struct my_agent *agent = &agents.entries[i];
@@ -595,7 +604,9 @@ void agents_city_changed(struct city *pcity)
   int i;
 
   freelog(LOG_DEBUG, "A: agents_city_changed(city='%s'(%d)) owner=%s",
-	  pcity->name, pcity->id, city_owner(pcity)->name);
+	  city_name(pcity),
+	  pcity->id,
+	  player_name(city_owner(pcity)));
 
   for (i = 0; i < agents.entries_used; i++) {
     struct my_agent *agent = &agents.entries[i];
@@ -621,8 +632,8 @@ void agents_city_new(struct city *pcity)
 
   freelog(LOG_DEBUG,
 	  "A: agents_city_new(city='%s'(%d)) pos=(%d,%d) owner=%s",
-	  pcity->name, pcity->id, TILE_XY(pcity->tile),
-	  city_owner(pcity)->name);
+	  city_name(pcity), pcity->id, TILE_XY(pcity->tile),
+	  player_name(city_owner(pcity)));
 
   for (i = 0; i < agents.entries_used; i++) {
     struct my_agent *agent = &agents.entries[i];
@@ -648,8 +659,8 @@ void agents_city_remove(struct city *pcity)
 
   freelog(LOG_DEBUG,
 	  "A: agents_city_remove(city='%s'(%d)) pos=(%d,%d) owner=%s",
-	  pcity->name, pcity->id, TILE_XY(pcity->tile),
-	  city_owner(pcity)->name);
+	  city_name(pcity), pcity->id, TILE_XY(pcity->tile),
+	  player_name(city_owner(pcity)));
 
   for (i = 0; i < agents.entries_used; i++) {
     struct my_agent *agent = &agents.entries[i];
@@ -809,7 +820,7 @@ bool agents_busy(void)
 {
   int i;
 
-  if (!initialized || call_list_size(&agents.calls) > 0 || frozen_level > 0
+  if (!initialized || call_list_size(agents.calls) > 0 || frozen_level > 0
       || currently_running) {
     return TRUE;
   }
