@@ -25,12 +25,14 @@
 #include "dataio.h"
 #include "events.h"
 #include "fcintl.h"
+#include "game.h"
 #include "government.h"
 #include "hash.h"
 #include "log.h"
 #include "mem.h"
 #include "packets.h"
 #include "shared.h"		/* for MIN() */
+#include "specialist.h"
 #include "support.h"
 #include "timing.h"
 
@@ -78,13 +80,13 @@ static struct {
   int apply_result_ignored, apply_result_applied, refresh_forced;
 } stats;
 
-#define my_city_map_iterate(pcity, cx, cy) {                           \
-  city_map_checked_iterate(pcity->tile, cx, cy, _ptile) { \
-    if(!is_city_center(cx, cy)) {
+#define my_city_map_iterate(pcity, cx, cy) {				\
+  city_map_checked_iterate(pcity->tile, cx, cy, cx##cy##_ptile) {	\
+    if (!is_free_worked_tile(cx, cy)) {
 
-#define my_city_map_iterate_end \
-    }                                \
-  } city_map_checked_iterate_end;    \
+#define my_city_map_iterate_end						\
+    }									\
+  } city_map_checked_iterate_end;					\
 }
 
 
@@ -100,17 +102,16 @@ static bool results_are_equal(struct city *pcity,
 			     const struct cm_result *const result1,
 			     const struct cm_result *const result2)
 {
-  enum cm_stat stat;
-
   T(disorder);
   T(happy);
-  T(specialists[SP_ELVIS]);
-  T(specialists[SP_SCIENTIST]);
-  T(specialists[SP_TAXMAN]);
 
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  specialist_type_iterate(sp) {
+    T(specialists[sp]);
+  } specialist_type_iterate_end;
+
+  output_type_iterate(stat) {
     T(surplus[stat]);
-  }
+  } output_type_iterate_end;
 
   my_city_map_iterate(pcity, x, y) {
     if (result1->worker_positions_used[x][y] !=
@@ -164,7 +165,7 @@ static void get_current_as_result(struct city *pcity,
 *****************************************************************************/
 static bool check_city(int city_id, struct cm_parameter *parameter)
 {
-  struct city *pcity = find_city_by_id(city_id);
+  struct city *pcity = game_find_city_by_number(city_id);
   struct cm_parameter dummy;
 
   if (!parameter) {
@@ -178,9 +179,6 @@ static bool check_city(int city_id, struct cm_parameter *parameter)
 
   if (city_owner(pcity) != game.player_ptr) {
     cma_release_city(pcity);
-    create_event(pcity->tile, E_CITY_CMA_RELEASE,
-		 _("CMA: You lost control of %s. Detaching from city."),
-		 pcity->name);
     return FALSE;
   }
 
@@ -194,7 +192,7 @@ static bool check_city(int city_id, struct cm_parameter *parameter)
 static bool apply_result_on_server(struct city *pcity,
 				   const struct cm_result *const result)
 {
-  int first_request_id = 0, last_request_id = 0, i, sp;
+  int first_request_id = 0, last_request_id = 0, i;
   struct cm_result current_state;
   bool success;
 
@@ -210,7 +208,7 @@ static bool apply_result_on_server(struct city *pcity,
   stats.apply_result_applied++;
 
   freelog(APPLY_RESULT_LOG_LEVEL, "apply_result(city='%s'(%d))",
-	  pcity->name, pcity->id);
+	  city_name(pcity), pcity->id);
 
   connection_do_buffer(&aconnection);
 
@@ -234,24 +232,27 @@ static bool apply_result_on_server(struct city *pcity,
     }
   } my_city_map_iterate_end;
 
-  /* Change the excess non-elvis specialists to elvises. */
-  assert(SP_ELVIS == 0);
-  for (sp = 1; sp < SP_COUNT; sp++) {
+  /* Change the excess non-default specialists to default. */
+  specialist_type_iterate(sp) {
+    if (sp == DEFAULT_SPECIALIST) {
+      continue;
+    }
     for (i = 0; i < pcity->specialists[sp] - result->specialists[sp]; i++) {
       freelog(APPLY_RESULT_LOG_LEVEL, "Change specialist from %d to %d.",
-	      sp, SP_ELVIS);
-      last_request_id = city_change_specialist(pcity, sp, SP_ELVIS);
+	      sp, DEFAULT_SPECIALIST);
+      last_request_id = city_change_specialist(pcity,
+					       sp, DEFAULT_SPECIALIST);
       if (first_request_id == 0) {
 	first_request_id = last_request_id;
       }
     }
-  }
+  } specialist_type_iterate_end;
 
   /* now all surplus people are enterainers */
 
   /* Set workers */
-  /* FIXME: This code assumes that any toggled worker will turn into an
-   * elvis! */
+  /* FIXME: This code assumes that any toggled worker will turn into a
+   * DEFAULT_SPECIALIST! */
   my_city_map_iterate(pcity, x, y) {
     if (result->worker_positions_used[x][y] &&
 	pcity->city_map[x][y] != C_TILE_WORKER) {
@@ -264,19 +265,22 @@ static bool apply_result_on_server(struct city *pcity,
     }
   } my_city_map_iterate_end;
 
-  /* Set all specialists except SP_ELVIS (all the unchanged ones remain
-   * as elvises). */
-  assert(SP_ELVIS == 0);
-  for (sp = 1; sp < SP_COUNT; sp++) {
+  /* Set all specialists except DEFAULT_SPECIALIST (all the unchanged
+   * ones remain as DEFAULT_SPECIALIST). */
+  specialist_type_iterate(sp) {
+    if (sp == DEFAULT_SPECIALIST) {
+      continue;
+    }
     for (i = 0; i < result->specialists[sp] - pcity->specialists[sp]; i++) {
       freelog(APPLY_RESULT_LOG_LEVEL, "Changing specialist from %d to %d.",
-	      SP_ELVIS, sp);
-      last_request_id = city_change_specialist(pcity, SP_ELVIS, sp);
+	      DEFAULT_SPECIALIST, sp);
+      last_request_id = city_change_specialist(pcity,
+					       DEFAULT_SPECIALIST, sp);
       if (first_request_id == 0) {
 	first_request_id = last_request_id;
       }
     }
-  }
+  } specialist_type_iterate_end;
 
   if (last_request_id == 0 || ALWAYS_APPLY_AT_SERVER) {
       /*
@@ -331,10 +335,6 @@ static void report_stats(void)
 #if SHOW_TIME_STATS
   int total, per_mill;
 
-  freelog(LOG_NORMAL, "CMA: overall=%fs queries=%d %fms / query",
-	  read_timer_seconds(stats.wall_timer), stats.queries,
-	  (1000.0 * read_timer_seconds(stats.wall_timer)) /
-	  ((double) stats.queries));
   total = stats.apply_result_ignored + stats.apply_result_applied;
   per_mill = (stats.apply_result_ignored * 1000) / (total ? total : 1);
 
@@ -371,11 +371,11 @@ static void handle_city(struct city *pcity)
   int i, city_id = pcity->id;
 
   freelog(HANDLE_CITY_LOG_LEVEL,
-	  "handle_city(city='%s'(%d) pos=(%d,%d) owner=%s)", pcity->name,
-	  pcity->id, TILE_XY(pcity->tile), city_owner(pcity)->name);
+	  "handle_city(city='%s'(%d) pos=(%d,%d) owner=%s)", city_name(pcity),
+	  pcity->id, TILE_XY(pcity->tile), player_name(city_owner(pcity)));
 
   freelog(HANDLE_CITY_LOG_LEVEL2, "START handle city='%s'(%d)",
-	  pcity->name, pcity->id);
+	  city_name(pcity), pcity->id);
 
   handled = FALSE;
   for (i = 0; i < 5; i++) {
@@ -388,7 +388,7 @@ static void handle_city(struct city *pcity)
       break;
     }
 
-    pcity = find_city_by_id(city_id);
+    pcity = game_find_city_by_number(city_id);
 
     cm_query_result(pcity, &parameter, &result);
     if (!result.found_a_valid) {
@@ -397,18 +397,18 @@ static void handle_city(struct city *pcity)
       cma_release_city(pcity);
 
       create_event(pcity->tile, E_CITY_CMA_RELEASE,
-		   _("CMA: The agent can't fulfill the requirements "
-		     "for %s. Passing back control."), pcity->name);
+		   _("The citizen governor can't fulfill the requirements "
+		     "for %s. Passing back control."), city_name(pcity));
       handled = TRUE;
       break;
     } else {
       if (!apply_result_on_server(pcity, &result)) {
 	freelog(HANDLE_CITY_LOG_LEVEL2, "  doesn't cleanly apply");
 	if (check_city(city_id, NULL) && i == 0) {
-	  create_event(pcity->tile, E_NOEVENT,
-		       _("CMA: %s has changed and the calculated "
-			 "result can't be applied. Will retry."),
-		       pcity->name);
+	  create_event(pcity->tile, E_CITY_CMA_RELEASE,
+		       _("The citizen governor has gotten confused dealing "
+			 "with %s.  You may want to have a look."),
+		       city_name(pcity));
 	}
       } else {
 	freelog(HANDLE_CITY_LOG_LEVEL2, "  ok");
@@ -419,22 +419,22 @@ static void handle_city(struct city *pcity)
     }
   }
 
-  pcity = find_city_by_id(city_id);
+  pcity = game_find_city_by_number(city_id);
 
   if (!handled) {
     assert(pcity != NULL);
     freelog(HANDLE_CITY_LOG_LEVEL2, "  not handled");
 
     create_event(pcity->tile, E_CITY_CMA_RELEASE,
-		 _("CMA: %s has changed multiple times. This may be "
-		   "an error in Freeciv or bad luck. The CMA will detach "
-		   "itself from the city now."), pcity->name);
+		 _("The citizen governor has gotten confused dealing "
+		   "with %s.  You may want to have a look."),
+		 city_name(pcity));
 
     cma_release_city(pcity);
 
     freelog(LOG_ERROR, "CMA: %s has changed multiple times due to "
             "an error in Freeciv. Please send a savegame that can reproduce "
-            "this bug to <bugs@freeciv.org>. Thank you.", pcity->name);
+            "this bug at %s. Thank you.", city_name(pcity), BUG_URL);
   }
 
   freelog(HANDLE_CITY_LOG_LEVEL2, "END handle city=(%d)", city_id);
@@ -445,7 +445,7 @@ static void handle_city(struct city *pcity)
 *****************************************************************************/
 static void city_changed(int city_id)
 {
-  struct city *pcity = find_city_by_id(city_id);
+  struct city *pcity = game_find_city_by_number(city_id);
 
   if (pcity) {
     cm_clear_cache(pcity);
@@ -485,6 +485,10 @@ void cma_init(void)
 
   /* reset cache counters */
   memset(&stats, 0, sizeof(stats));
+
+  /* We used to just use new_timer here, but apparently cma_init can be
+   * called multiple times per client invocation so that lead to memory
+   * leaks. */
   stats.wall_timer = renew_timer(timer, TIMER_USER, TIMER_ACTIVE);
 
   memset(&self, 0, sizeof(self));
@@ -517,7 +521,7 @@ void cma_put_city_under_agent(struct city *pcity,
 			      const struct cm_parameter *const parameter)
 {
   freelog(LOG_DEBUG, "cma_put_city_under_agent(city='%s'(%d))",
-	  pcity->name, pcity->id);
+	  city_name(pcity), pcity->id);
 
   assert(city_owner(pcity) == game.player_ptr);
 
@@ -569,7 +573,7 @@ bool cma_get_parameter(enum attr_city attr, int city_id,
   size_t len;
   char buffer[SAVED_PARAMETER_SIZE];
   struct data_in din;
-  int i, version, dummy;
+  int version, dummy;
 
   /* Changing this function is likely to break compatability with old
    * savegames that store these values. */
@@ -585,17 +589,18 @@ bool cma_get_parameter(enum attr_city attr, int city_id,
   dio_get_uint8(&din, &version);
   assert(version == 2);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  /* Initialize the parameter (includes some AI-only fields that aren't
+   * touched below). */
+  cm_init_parameter(parameter);
+
+  output_type_iterate(i) {
     dio_get_sint16(&din, &parameter->minimal_surplus[i]);
     dio_get_sint16(&din, &parameter->factor[i]);
-  }
+  } output_type_iterate_end;
 
   dio_get_sint16(&din, &parameter->happy_factor);
   dio_get_uint8(&din, &dummy); /* Dummy value; used to be factor_target. */
   dio_get_bool8(&din, &parameter->require_happy);
-  /* These options are only for server-AI use. */
-  parameter->allow_disorder = FALSE;
-  parameter->allow_specialists = TRUE;
 
   return TRUE;
 }
@@ -608,7 +613,6 @@ void cma_set_parameter(enum attr_city attr, int city_id,
 {
   char buffer[SAVED_PARAMETER_SIZE];
   struct data_out dout;
-  int i;
 
   /* Changing this function is likely to break compatability with old
    * savegames that store these values. */
@@ -617,10 +621,10 @@ void cma_set_parameter(enum attr_city attr, int city_id,
 
   dio_put_uint8(&dout, 2);
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     dio_put_sint16(&dout, parameter->minimal_surplus[i]);
     dio_put_sint16(&dout, parameter->factor[i]);
-  }
+  } output_type_iterate_end;
 
   dio_put_sint16(&dout, parameter->happy_factor);
   dio_put_uint8(&dout, 0); /* Dummy value; used to be factor_target. */
