@@ -23,165 +23,58 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-#include <stdlib.h>
-#include <string.h>
+#include "SDL.h"
 
-#include <SDL/SDL.h>
-
-#include "hash.h"
-#include "support.h"
+/* utility */
+#include "fcintl.h"
 #include "log.h"
 
-#include "tilespec.h"
+/* common */
+#include "game.h"
 
-#include "gui_mem.h"
-#include "gui_main.h"
+/* gui-sdl */
+#include "colors.h"
 #include "graphics.h"
-#include "gui_zoom.h"
+#include "gui_main.h"
+#include "gui_string.h"
+#include "sprite.h"
+#include "themespec.h"
 
 #include "gui_tilespec.h"
 
-extern SDL_Surface *pDitherMask;
+struct Theme *pTheme;
+struct City_Icon *pIcons;
 
-static struct {
-    /* Each citizen type has up to MAX_NUM_CITIZEN_SPRITES different
-     * sprites, as defined by the tileset. */
-    int count;
-    SDL_Surface *surface[MAX_NUM_CITIZEN_SPRITES];
-  } citizen[NUM_TILES_CITIZEN];
+static SDL_Surface *pCity_Surf;
 
+static SDL_Surface *pNeutral_Tech_Icon;
+static SDL_Surface *pNone_Tech_Icon;
+static SDL_Surface *pFuture_Tech_Icon;
 
-#define load_GUI_surface(pSpr, pStruct, pSurf, tag)		\
-do {								\
-  pSpr = load_sprite(tag);					\
-  pStruct->pSurf = (pSpr ? GET_SURF(pSpr) : NULL);		\
-  assert(pStruct->pSurf != NULL);				\
-  pSpr->psurface = NULL;					\
-  unload_sprite(tag);						\
+#define load_GUI_surface(pSpr, pStruct, pSurf, tag)		  \
+do {								  \
+  pSpr = theme_lookup_sprite_tag_alt(theme, tag, "", TRUE, "", ""); \
+  assert(pSpr != NULL);				                  \
+  pStruct->pSurf = adj_surf(GET_SURF(pSpr));            	  \
+  FREESURFACE(GET_SURF(pSpr));                                    \
 } while(0)
 
-
 #define load_theme_surface(pSpr, pSurf, tag)		\
 	load_GUI_surface(pSpr, pTheme, pSurf, tag)
 
-#define load_city_icon_surface(pSpr, pSurf, tag)	\
-	load_GUI_surface(pSpr, pIcons, pSurf, tag)
-
+#define load_city_icon_surface(pSpr, pSurf, tag)        \
+        load_GUI_surface(pSpr, pIcons, pSurf, tag)
 
 #define load_order_theme_surface(pSpr, pSurf, tag)	\
 do {							\
   load_GUI_surface(pSpr, pTheme, pSurf, tag);		\
-  if(pTheme->pSurf->format->Amask) {			\
-    SDL_SetAlpha(pTheme->pSurf, 0x0 , 0x0);		\
-  }							\
 } while(0)
 
-/**********************************************************************
-  Returns a text name for the citizen, as used in the tileset.
-***********************************************************************/
-static const char *get_citizen_name(struct citizen_type citizen)
-{
-  /* These strings are used in reading the tileset.  Do not
-   * translate. */
-  switch (citizen) {
-  case CITIZEN_ELVIS:
-    return "entertainer";
-  case CITIZEN_SCIENTIST:
-    return "scientist";
-  case CITIZEN_TAXMAN:
-    return "tax_collector";
-  case CITIZEN_HAPPY:
-    return "happy";
-  case CITIZEN_CONTENT:
-    return "content";
-  case CITIZEN_UNHAPPY:
-    return "unhappy";
-  case CITIZEN_ANGRY:
-    return "angry";
-  default:
-    die("unknown citizen type %d", (int) citizen);
-  }
-  return NULL;
-}
-
-/*******************************************************************************
- * reload citizens "style" icons.
- *******************************************************************************/
-static void real_reload_citizens_icons(int style)
-{
-  char tag[64];
-  char alt_buf[32] = ".";
-  int i , j;
-  struct Sprite *pSpr = NULL;
-    
-  if (strcmp("generic" , city_styles[style].citizens_graphic_alt))
-  {
-    my_snprintf(alt_buf , sizeof(alt_buf) , ".%s_",
-                       city_styles[style].citizens_graphic_alt ); 
-  }
-	
-  /* Load the citizen sprite graphics. */
-  for (i = 0; i < NUM_TILES_CITIZEN; i++) {
-	  
-    my_snprintf(tag, sizeof(tag), "citizen.%s_%s",
-	  city_styles[style].citizens_graphic , get_citizen_name(i));  
-    
-    
-    pSpr = load_sprite(tag);
-    if(!pSpr) {
-      freelog(LOG_DEBUG,"Can't find %s", tag);
-      my_snprintf(tag, sizeof(tag), "citizen%s%s", alt_buf ,get_citizen_name(i));  
-      freelog(LOG_DEBUG,"Trying load alternative %s", tag);
-      pSpr = load_sprite(tag);
-    }
-    
-    FREESURFACE(citizen[i].surface[0]);
-    citizen[i].surface[0] = (pSpr ? GET_SURF(pSpr) : NULL);
-    
-    if (citizen[i].surface[0]) {
-      /* If this form exists, use it as the only sprite.  This allows
-       * backwards compatability with tilesets that use e.g.,
-       * citizen.entertainer. */
-      citizen[i].count = 1;
-      pSpr->psurface = NULL;
-      unload_sprite(tag);
-      continue;
-    }
-    for (j = 0; j < MAX_NUM_CITIZEN_SPRITES; j++) {
-      my_snprintf(tag, sizeof(tag), "citizen.%s_%s_%d",
-	  city_styles[style].citizens_graphic ,get_citizen_name(i) , j );
-      
-      pSpr = load_sprite(tag);
-      if(!pSpr) {
-        freelog(LOG_DEBUG,"Can't find %s", tag);
-        my_snprintf(tag, sizeof(tag), "citizen%s%s_%d", alt_buf,
-	    				get_citizen_name(i), j);
-        freelog(LOG_DEBUG,"Trying load alternative %s", tag);
-        pSpr = load_sprite(tag);
-      }
-      
-      FREESURFACE(citizen[i].surface[j]);
-      citizen[i].surface[j] = (pSpr ? GET_SURF(pSpr) : NULL);
-      if (!citizen[i].surface[j]) {
-	break;
-      }
-      pSpr->psurface = NULL;
-      unload_sprite(tag);
-    }
-    citizen[i].count = j;
-    assert(j > 0);
-  }	
-
-}
-
-
 /*******************************************************************************
  * reload small citizens "style" icons.
  *******************************************************************************/
 static void reload_small_citizens_icons(int style)
 {
-
   /* free info icons */
   FREESURFACE(pIcons->pMale_Content);
   FREESURFACE(pIcons->pFemale_Content);
@@ -191,67 +84,23 @@ static void reload_small_citizens_icons(int style)
   FREESURFACE(pIcons->pFemale_Unhappy);
   FREESURFACE(pIcons->pMale_Angry);
   FREESURFACE(pIcons->pFemale_Angry);
-	
+  
   FREESURFACE(pIcons->pSpec_Lux); /* Elvis */
   FREESURFACE(pIcons->pSpec_Tax); /* TaxMan */
   FREESURFACE(pIcons->pSpec_Sci); /* Scientist */
-
+  
   /* allocate icons */
-  pIcons->pMale_Happy =
-    ResizeSurface(get_citizen_surface(CITIZEN_HAPPY, 0), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pMale_Happy , SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pMale_Happy));
-    
-  pIcons->pFemale_Happy =
-    ResizeSurface(get_citizen_surface(CITIZEN_HAPPY, 1), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pFemale_Happy , SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pFemale_Happy));
-    
-  pIcons->pMale_Content =
-    ResizeSurface(get_citizen_surface(CITIZEN_CONTENT, 0), 15, 26, 1);
-  SDL_SetColorKey( pIcons->pMale_Content , SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pMale_Content ));
-    
-  pIcons->pFemale_Content =
-    ResizeSurface(get_citizen_surface(CITIZEN_CONTENT, 1), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pFemale_Content, SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pFemale_Content));
-    
-  pIcons->pMale_Unhappy =
-    ResizeSurface(get_citizen_surface(CITIZEN_UNHAPPY, 0), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pMale_Unhappy , SDL_SRCCOLORKEY|SDL_RLEACCEL , 
-			    get_first_pixel(pIcons->pMale_Unhappy));
-    
-  pIcons->pFemale_Unhappy =
-    ResizeSurface(get_citizen_surface(CITIZEN_UNHAPPY, 1), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pFemale_Unhappy , SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pFemale_Unhappy));
-    
-  pIcons->pMale_Angry =
-    ResizeSurface(get_citizen_surface(CITIZEN_ANGRY, 0), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pMale_Angry , SDL_SRCCOLORKEY|SDL_RLEACCEL , 
-			    get_first_pixel(pIcons->pMale_Angry));
-    
-  pIcons->pFemale_Angry =
-    ResizeSurface(get_citizen_surface(CITIZEN_ANGRY, 1), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pFemale_Angry , SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pFemale_Angry));
-    
-  pIcons->pSpec_Lux =
-    ResizeSurface(get_citizen_surface(CITIZEN_ELVIS, 0), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pSpec_Lux , SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pSpec_Lux));
-    
-  pIcons->pSpec_Tax =
-    ResizeSurface(get_citizen_surface(CITIZEN_TAXMAN, 0), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pSpec_Tax, SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pSpec_Tax));
-    
-  pIcons->pSpec_Sci =
-    ResizeSurface(get_citizen_surface(CITIZEN_SCIENTIST, 0), 15, 26, 1);
-  SDL_SetColorKey(pIcons->pSpec_Sci, SDL_SRCCOLORKEY|SDL_RLEACCEL, 
-			    get_first_pixel(pIcons->pSpec_Sci));
-    
+  pIcons->pMale_Happy = adj_surf(get_citizen_surface(CITIZEN_HAPPY, 0));
+  pIcons->pFemale_Happy = adj_surf(get_citizen_surface(CITIZEN_HAPPY, 1));
+  pIcons->pMale_Content = adj_surf(get_citizen_surface(CITIZEN_CONTENT, 0));
+  pIcons->pFemale_Content = adj_surf(get_citizen_surface(CITIZEN_CONTENT, 1));
+  pIcons->pMale_Unhappy = adj_surf(get_citizen_surface(CITIZEN_UNHAPPY, 0));
+  pIcons->pFemale_Unhappy = adj_surf(get_citizen_surface(CITIZEN_UNHAPPY, 1));
+  pIcons->pMale_Angry = adj_surf(get_citizen_surface(CITIZEN_ANGRY, 0));
+  pIcons->pFemale_Angry = adj_surf(get_citizen_surface(CITIZEN_ANGRY, 1));
+  pIcons->pSpec_Lux = get_tax_surface(O_LUXURY);
+  pIcons->pSpec_Tax = get_tax_surface(O_GOLD);
+  pIcons->pSpec_Sci = get_tax_surface(O_SCIENCE);
 }
 
 /* ================================================================================= */
@@ -264,21 +113,32 @@ static void reload_small_citizens_icons(int style)
 ***********************************************************************/
 void reload_citizens_icons(int style)
 {
-  real_reload_citizens_icons(style);
   reload_small_citizens_icons(style);
   pIcons->style = style;
 }
 
+/**************************************************************************
+  ...
+**************************************************************************/
+void tilespec_setup_city_gfx(void) {
+  struct sprite *pSpr = theme_lookup_sprite_tag_alt(
+                                    theme, "theme.city", "", TRUE, "", "");    
+
+  pCity_Surf = (pSpr ? adj_surf(GET_SURF(pSpr)) : NULL);
+  
+  assert(pCity_Surf != NULL);
+}
+
 /**********************************************************************
   Set city icons sprite value; should only happen after
-  tilespec_load_tiles().
+  tileset_load_tiles(tileset).
 ***********************************************************************/
 void tilespec_setup_city_icons(void)
 {
-  int i, j;
-  struct Sprite *pSpr = NULL;
+
+  struct sprite *pSpr = NULL;
   
-  pIcons = ( struct City_Icon *)MALLOC( sizeof( struct City_Icon ));
+  pIcons = ( struct City_Icon *)fc_calloc(1,  sizeof( struct City_Icon ));
   
   load_city_icon_surface(pSpr, pBIG_Food_Corr, "city.food_waste");
   load_city_icon_surface(pSpr, pBIG_Shield_Corr, "city.shield_waste");
@@ -315,21 +175,19 @@ void tilespec_setup_city_icons(void)
   
   load_city_icon_surface(pSpr, pPolice, "city.police");
   /* ================================================================= */
-  pIcons->pWorklist = create_surf(9,9, SDL_SWSURFACE);
+  pIcons->pWorklist = create_surf_alpha(9,9, SDL_SWSURFACE);
   SDL_FillRect(pIcons->pWorklist, NULL,
 		  SDL_MapRGB(pIcons->pWorklist->format, 255, 255,255));
-  putframe(pIcons->pWorklist, 0,0,
-	pIcons->pWorklist->w - 1, pIcons->pWorklist->h - 1, 0xFF000000);
-  putline(pIcons->pWorklist, 3, 2, 5, 2, 0xFF000000);
-  putline(pIcons->pWorklist, 3, 4, 7, 4, 0xFF000000);
-  putline(pIcons->pWorklist, 3, 6, 6, 6, 0xFF000000);
+  putframe(pIcons->pWorklist, 0,0, pIcons->pWorklist->w - 1, pIcons->pWorklist->h - 1,
+    map_rgba(pIcons->pWorklist->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putline(pIcons->pWorklist, 3, 2, 5, 2,
+    map_rgba(pIcons->pWorklist->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putline(pIcons->pWorklist, 3, 4, 7, 4, 
+    map_rgba(pIcons->pWorklist->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  putline(pIcons->pWorklist, 3, 6, 6, 6,
+    map_rgba(pIcons->pWorklist->format, *get_game_colorRGB(COLOR_THEME_CITYREP_FRAME)));
+  
   /* ================================================================= */
-  /* clear all citizens icons pointer */
-  for (i = 0; i < NUM_TILES_CITIZEN; i++) {
-    for (j = 0; j < MAX_NUM_CITIZEN_SPRITES; j++) {
-      citizen[i].surface[j] = NULL;
-    }
-  }
   
   /* force reload citizens icons */
   pIcons->style = 999;
@@ -337,33 +195,37 @@ void tilespec_setup_city_icons(void)
 
 void tilespec_free_city_icons(void)
 {
-  FREE(pIcons->pBIG_Food_Corr);
-  FREE(pIcons->pBIG_Shield_Corr);
-  FREE(pIcons->pBIG_Trade_Corr);
-  FREE(pIcons->pBIG_Food);
-  FREE(pIcons->pBIG_Food_Surplus);
-  FREE(pIcons->pBIG_Shield);
-  FREE(pIcons->pBIG_Shield_Surplus);
-  FREE(pIcons->pBIG_Trade);
-  FREE(pIcons->pBIG_Luxury);
-  FREE(pIcons->pBIG_Coin);
-  FREE(pIcons->pBIG_Colb);
-  FREE(pIcons->pBIG_Face);
-  FREE(pIcons->pBIG_Coin_Corr);
-  FREE(pIcons->pBIG_Coin_UpKeep);
-  
-  FREE(pIcons->pFood);
-  FREE(pIcons->pShield);
-  FREE(pIcons->pTrade);
-  FREE(pIcons->pFace);
-  FREE(pIcons->pLuxury);
-  FREE(pIcons->pCoin);		  
-  FREE(pIcons->pColb);		  
-    
-  FREE(pIcons->pPollution);
-  FREE(pIcons->pPolice);
-  FREE(pIcons->pWorklist);
+  if (!pIcons) {
+    return;
+  }
   
+  FREESURFACE(pIcons->pBIG_Food_Corr);
+  FREESURFACE(pIcons->pBIG_Shield_Corr);
+  FREESURFACE(pIcons->pBIG_Trade_Corr);
+  FREESURFACE(pIcons->pBIG_Food);
+  FREESURFACE(pIcons->pBIG_Food_Surplus);
+  FREESURFACE(pIcons->pBIG_Shield);
+  FREESURFACE(pIcons->pBIG_Shield_Surplus);
+  FREESURFACE(pIcons->pBIG_Trade);
+  FREESURFACE(pIcons->pBIG_Luxury);
+  FREESURFACE(pIcons->pBIG_Coin);
+  FREESURFACE(pIcons->pBIG_Colb);
+  FREESURFACE(pIcons->pBIG_Face);
+  FREESURFACE(pIcons->pBIG_Coin_Corr);
+  FREESURFACE(pIcons->pBIG_Coin_UpKeep);
+
+  FREESURFACE(pIcons->pFood);
+  FREESURFACE(pIcons->pShield);
+  FREESURFACE(pIcons->pTrade);
+  FREESURFACE(pIcons->pFace);
+  FREESURFACE(pIcons->pLuxury);
+  FREESURFACE(pIcons->pCoin);		  
+  FREESURFACE(pIcons->pColb);		  
+
+  FREESURFACE(pIcons->pPollution);
+  FREESURFACE(pIcons->pPolice);
+  FREESURFACE(pIcons->pWorklist);
+
   /* small citizens */
   FREESURFACE(pIcons->pMale_Content);
   FREESURFACE(pIcons->pFemale_Content);
@@ -373,12 +235,12 @@ void tilespec_free_city_icons(void)
   FREESURFACE(pIcons->pFemale_Unhappy);
   FREESURFACE(pIcons->pMale_Angry);
   FREESURFACE(pIcons->pFemale_Angry);
-	
+
   FREESURFACE(pIcons->pSpec_Lux); /* Elvis */
   FREESURFACE(pIcons->pSpec_Tax); /* TaxMan */
   FREESURFACE(pIcons->pSpec_Sci); /* Scientist */
-  
-  FREE(pIcons);
+
+  FC_FREE(pIcons);
   
 }
 
@@ -392,27 +254,27 @@ void tilespec_free_city_icons(void)
  */
 void tilespec_setup_theme(void)
 {
-  struct Sprite *pBuf = NULL;
-  
-  pTheme = MALLOC(sizeof(struct Theme));
-  
-  if(!sprite_exists("theme.tech_tree")) {
-    freelog(LOG_FATAL, "Your current tileset don't contains ""all"" GUI theme graphic\n"
-    "Please use other tileset with ""full"" GUI graphic pack (use -t tileset options)\n"
-    "If you don't have any tileset with SDLClient GUI theme then go to freeciv\n"
-    "(ftp.freeciv.org/pub/freeciv/incoming) ftp site and download current DELUXE"
-    "tileset theme");
-  }
+  struct sprite *pBuf = NULL;
   
+  pTheme = fc_calloc(1, sizeof(struct Theme));
+
+  load_theme_surface(pBuf, FR_Left, "theme.left_frame");
+  load_theme_surface(pBuf, FR_Right, "theme.right_frame");
+  load_theme_surface(pBuf, FR_Top, "theme.top_frame");
+  load_theme_surface(pBuf, FR_Bottom, "theme.bottom_frame");
   load_theme_surface(pBuf, Button, "theme.button");
   load_theme_surface(pBuf, Edit, "theme.edit");
-  load_theme_surface(pBuf, Vertic, "theme.vertic_scrollbar");
-  load_theme_surface(pBuf, Horiz, "theme.horiz_scrollbar");
   load_theme_surface(pBuf, CBOX_Sell_Icon, "theme.sbox");
   load_theme_surface(pBuf, CBOX_Unsell_Icon, "theme.ubox");
-  load_theme_surface(pBuf, Block, "theme.block");
-  load_theme_surface(pBuf, FR_Vert, "theme.vertic_frame");
-  load_theme_surface(pBuf, FR_Hor, "theme.horiz_frame");
+  load_theme_surface(pBuf, UP_Icon, "theme.UP_scroll");
+  load_theme_surface(pBuf, DOWN_Icon, "theme.DOWN_scroll");
+#if 0
+  load_theme_surface(pBuf, LEFT_Icon, "theme.LEFT_scroll");
+  load_theme_surface(pBuf, RIGHT_Icon, "theme.RIGHT_scroll");
+#endif
+  load_theme_surface(pBuf, Vertic, "theme.vertic_scrollbar");
+  load_theme_surface(pBuf, Horiz, "theme.horiz_scrollbar");
+  
   /* ------------------- */
   load_theme_surface(pBuf, OK_PACT_Icon, "theme.pact_ok");
   load_theme_surface(pBuf, CANCEL_PACT_Icon, "theme.pact_cancel");
@@ -426,13 +288,13 @@ void tilespec_setup_theme(void)
   load_theme_surface(pBuf, BACK_Icon, "theme.BACK_button");
   load_theme_surface(pBuf, L_ARROW_Icon, "theme.LEFT_ARROW_button");
   load_theme_surface(pBuf, R_ARROW_Icon, "theme.RIGHT_ARROW_button");
-  load_theme_surface(pBuf, META_Icon, "theme.META_button");
   load_theme_surface(pBuf, MAP_Icon, "theme.MAP_button");
   load_theme_surface(pBuf, FindCity_Icon, "theme.FIND_CITY_button");
   load_theme_surface(pBuf, NEW_TURN_Icon, "theme.NEW_TURN_button");
   load_theme_surface(pBuf, LOG_Icon, "theme.LOG_button");
   load_theme_surface(pBuf, UNITS_Icon, "theme.UNITS_INFO_button");
   load_theme_surface(pBuf, Options_Icon, "theme.OPTIONS_button");
+  load_theme_surface(pBuf, Block, "theme.block");
   load_theme_surface(pBuf, INFO_Icon, "theme.INFO_button");
   load_theme_surface(pBuf, Army_Icon, "theme.ARMY_button");
   load_theme_surface(pBuf, Happy_Icon, "theme.HAPPY_button");
@@ -452,13 +314,6 @@ void tilespec_setup_theme(void)
   /* ------------------------------ */
   load_theme_surface(pBuf, Tech_Tree_Icon, "theme.tech_tree");
   /* ------------------------------ */
-  load_theme_surface(pBuf, UP_Icon, "theme.UP_scroll");
-  load_theme_surface(pBuf, DOWN_Icon, "theme.DOWN_scroll");
-#if 0
-  load_theme_surface(pBuf, LEFT_Icon, "theme.LEFT_scroll");
-  load_theme_surface(pBuf, RIGHT_Icon, "theme.RIGHT_button");
-#endif
-  /* ------------------------------ */
 
   load_order_theme_surface(pBuf, Order_Icon, "theme.order_empty");  
   load_order_theme_surface(pBuf, OAutoAtt_Icon, "theme.order_auto_attack");
@@ -497,185 +352,43 @@ void tilespec_setup_theme(void)
   load_order_theme_surface(pBuf, OWakeUp_Icon, "theme.order_wakeup");
   load_order_theme_surface(pBuf, OReturn_Icon, "theme.order_return");
   load_order_theme_surface(pBuf, OAirLift_Icon, "theme.order_airlift");
+  load_order_theme_surface(pBuf, OLoad_Icon, "theme.order_load");
   
   /* ------------------------------ */
     
-  /* Map Dithering */
-  
-    if (is_isometric)
-    {
-      pBuf = sprites.dither_tile;
-      pDitherMask = GET_SURF(pBuf);
-      pBuf->psurface = NULL;
-      unload_sprite("t.dither_tile");
-      assert(pDitherMask != NULL);	  
-  /* ------------------------------ */
-  /* Map Borders */
-      load_theme_surface(pBuf, NWEST_BORDER_Icon, "theme.normal_border_iso_west");
-      load_theme_surface(pBuf, NNORTH_BORDER_Icon, "theme.normal_border_iso_north");
-      load_theme_surface(pBuf, NSOUTH_BORDER_Icon, "theme.normal_border_iso_south");
-      load_theme_surface(pBuf, NEAST_BORDER_Icon, "theme.normal_border_iso_east");
-    }
   return;
 }
 
-/* Code come from SDL-dev list */
-static SDL_Cursor *SurfaceToCursor(SDL_Surface *image, int hx, int hy) {
-        int             w, x, y;
-        Uint8           *data, *mask, *d, *m, r, g, b;
-        Uint32          color;
-        SDL_Cursor      *cursor;
-
-        w = (image->w + 7) / 8;
-        data = (Uint8 *)MALLOC(w * image->h * 2);
-        if (data == NULL)
-                return NULL;
-        /*memset(data, 0, w * image->h * 2);*/
-        mask = data + w * image->h;
-	lock_surf(image);
-        for (y = 0; y < image->h; y++) {
-                d = data + y * w;
-                m = mask + y * w;
-                for (x = 0; x < image->w; x++) {
-                        color = getpixel(image, x, y);
-                        if ((image->flags & SDL_SRCCOLORKEY) == 0
-			  || color != image->format->colorkey) {
-                                SDL_GetRGB(color, image->format, &r, &g, &b);
-                                color = (r + g + b) / 3;
-                                m[x / 8] |= 128 >> (x & 7);
-                                if (color < 128)
-                                        d[x / 8] |= 128 >> (x & 7);
-                        }
-                }
-        }
-	unlock_surf(image);
-        
-        cursor = SDL_CreateCursor(data, mask, w * 8, image->h, hx, hy);
-	
-	FREE(data);
-        return cursor;
-}
-
-#define load_cursor(iter, num, pSpr, image, cBuf, Type, Tag, x, y, center) \
-do { \
-  iter = 0;	\
-  my_snprintf(cBuf , sizeof(cBuf), "%s_%d", Tag, iter);	\
-  while(sprite_exists(cBuf)) {	\
-    iter++;	\
-    my_snprintf(cBuf , sizeof(cBuf), "%s_%d", Tag, iter);	\
-  }	\
-  num = iter;	\
-  if (num) {	\
-    pAnim->Cursors.Type = CALLOC(num + 1, sizeof(SDL_Cursor *));	\
-    for( iter=0; iter<num; iter++) {	\
-      my_snprintf(cBuf,sizeof(cBuf), "%s_%d", Tag, iter);	\
-      pSpr = load_sprite(cBuf);	\
-      image = (pSpr ? GET_SURF(pSpr) : NULL);	\
-      assert(image != NULL);	\
-      if (center) {	\
-        pAnim->Cursors.Type[iter] = SurfaceToCursor(image, image->w/2, image->h/2);	\
-      } else {	\
-	pAnim->Cursors.Type[iter] = SurfaceToCursor(image, x, y);	\
-      }	\
-      unload_sprite(cBuf);	\
-    }	\
-  }	\
-} while(0)
-
-/*
- *	Alloc and fill Animation struct
- */
-void tilespec_setup_anim(void)
-{
-  char buf[32];	/* I hope this is enought :) */
-  struct Sprite *pSpr = NULL;
-  SDL_Surface *image = NULL;
-  int i, num;
-  pAnim = MALLOC(sizeof(struct Animation));
-    
-  i = 0;
-  my_snprintf(buf , sizeof(buf), "explode.iso_nuke_%d", i);
-  while(sprite_exists(buf)) {
-    i++;
-    my_snprintf(buf , sizeof(buf), "explode.iso_nuke_%d", i);
-  }
-  pAnim->num_tiles_explode_nuke = i;
-  
-  /* focus unit animation */
-  i = 0;
-  my_snprintf(buf , sizeof(buf), "anim.focus_%d", i);
-  while(sprite_exists(buf)) {
-    i++;
-    my_snprintf(buf , sizeof(buf), "anim.focus_%d", i);
-  }
-  pAnim->num_tiles_focused_unit = i;
-  
-  pAnim->Focus = CALLOC(pAnim->num_tiles_focused_unit, sizeof(SDL_Surface *));
-  for( i=0; i<pAnim->num_tiles_focused_unit; i++) {
-    my_snprintf(buf,sizeof(buf), "anim.focus_%d", i);
-    load_GUI_surface(pSpr, pAnim, Focus[i], buf);
-  }
-  
-  /* load cursors */
-  load_cursor(i, num, pSpr, image, buf, Patrol, "anim.patrol_cursor", 0, 0, TRUE);
-  load_cursor(i, num, pSpr, image, buf, Goto, "anim.goto_cursor", 0, 0, TRUE);
-  load_cursor(i, num, pSpr, image, buf, Connect, "anim.connect_cursor", 0, 0, TRUE);
-  load_cursor(i, num, pSpr, image, buf, Nuke, "anim.nuke_cursor", 0, 0, TRUE);
-  load_cursor(i, num, pSpr, image, buf, Paradrop, "anim.paradrop_cursor", 0, 0, TRUE);
-  
-  load_cursor(i, num, pSpr, image, buf, MapScroll[SCROLL_NORTH], "anim.scroll_north_cursor", 20, 3, FALSE);
-  load_cursor(i, num, pSpr, image, buf, MapScroll[SCROLL_SOUTH], "anim.scroll_south_cursor", 20, 37, FALSE);
-  load_cursor(i, num, pSpr, image, buf, MapScroll[SCROLL_EAST], "anim.scroll_east_cursor", 37, 20, FALSE);
-  load_cursor(i, num, pSpr, image, buf, MapScroll[SCROLL_WEST], "anim.scroll_west_cursor", 3, 20, FALSE);
-  
-}
-
-void tilespec_free_anim(void)
-{
-  int i,j;
-  for(i=0; i<pAnim->num_tiles_focused_unit; i++) {
-    FREESURFACE(pAnim->Focus[i]);
-  }
-  FREE(pAnim->Focus);
-  
-  for(i=0; pAnim->Cursors.Patrol[i]; i++) {
-    SDL_FreeCursor(pAnim->Cursors.Patrol[i]);
-  }
-  for(i=0; pAnim->Cursors.Goto[i]; i++) {
-    SDL_FreeCursor(pAnim->Cursors.Goto[i]);
-  }
-  for(i=0; pAnim->Cursors.Connect[i]; i++) {
-    SDL_FreeCursor(pAnim->Cursors.Connect[i]);
-  }
-  for(i=0; pAnim->Cursors.Nuke[i]; i++) {
-    SDL_FreeCursor(pAnim->Cursors.Nuke[i]);
-  }
-  for(i=0; pAnim->Cursors.Paradrop[i]; i++) {
-    SDL_FreeCursor(pAnim->Cursors.Paradrop[i]);
-  }
-  for (i = 0; i < SCROLL_LAST; i++) {
-    for (j = 0; pAnim->Cursors.MapScroll[i][j]; j++) {
-      SDL_FreeCursor(pAnim->Cursors.MapScroll[i][j]);
-    }
-  }
-  
-  FREE(pAnim);
-}
-
 /*
  *	Free memmory
  */
-void tilespec_unload_theme(void)
+void tilespec_free_theme(void)
 {
+  if (!pTheme) {
+    return;
+  }
+
+  FREESURFACE(pTheme->FR_Left);
+  FREESURFACE(pTheme->FR_Right);
+  FREESURFACE(pTheme->FR_Top);
+  FREESURFACE(pTheme->FR_Bottom);
+  
   FREESURFACE(pTheme->Button);
+  
   FREESURFACE(pTheme->Edit);
-  FREESURFACE(pTheme->Vertic);
-  FREESURFACE(pTheme->Horiz);
+
   FREESURFACE(pTheme->CBOX_Sell_Icon);
   FREESURFACE(pTheme->CBOX_Unsell_Icon);
-  FREESURFACE(pTheme->Block);
-  FREESURFACE(pTheme->FR_Vert);
-  FREESURFACE(pTheme->FR_Hor);
+
+  FREESURFACE(pTheme->UP_Icon);
+  FREESURFACE(pTheme->DOWN_Icon);
+#if 0
+  FREESURFACE(pTheme->LEFT_Icon);
+  FREESURFACE(pTheme->RIGHT_Icon);
+#endif
+  FREESURFACE(pTheme->Vertic);
+  FREESURFACE(pTheme->Horiz);
+  
   /* ------------------- */
   
   FREESURFACE(pTheme->OK_Icon);
@@ -686,7 +399,6 @@ void tilespec_unload_theme(void)
   FREESURFACE(pTheme->BACK_Icon);
   FREESURFACE(pTheme->L_ARROW_Icon);
   FREESURFACE(pTheme->R_ARROW_Icon);
-  FREESURFACE(pTheme->META_Icon);
   FREESURFACE(pTheme->MAP_Icon);
   FREESURFACE(pTheme->FindCity_Icon);
   FREESURFACE(pTheme->NEW_TURN_Icon);
@@ -695,6 +407,7 @@ void tilespec_unload_theme(void)
   FREESURFACE(pTheme->UNITS2_Icon);
   FREESURFACE(pTheme->PLAYERS_Icon);
   FREESURFACE(pTheme->Options_Icon);
+  FREESURFACE(pTheme->Block);
   FREESURFACE(pTheme->INFO_Icon);
   FREESURFACE(pTheme->Army_Icon);
   FREESURFACE(pTheme->Happy_Icon);
@@ -714,13 +427,6 @@ void tilespec_unload_theme(void)
   /* ------------------------------ */
   FREESURFACE(pTheme->Tech_Tree_Icon);
   /* ------------------------------ */
-  FREESURFACE(pTheme->UP_Icon);
-  FREESURFACE(pTheme->DOWN_Icon);
-#if 0
-  FREESURFACE(pTheme->LEFT_Icon);
-  FREESURFACE(pTheme->RIGHT_Icon);
-#endif
-  /* ------------------------------ */
 
   FREESURFACE(pTheme->Order_Icon);
   FREESURFACE(pTheme->OAutoAtt_Icon);
@@ -759,68 +465,135 @@ void tilespec_unload_theme(void)
   FREESURFACE(pTheme->OWakeUp_Icon);
   FREESURFACE(pTheme->OReturn_Icon);
   FREESURFACE(pTheme->OAirLift_Icon);
+  FREESURFACE(pTheme->OLoad_Icon);
 
-  /* Map Dithering */
-   
-  FREESURFACE(pDitherMask);
-  	
-  /* Map Borders */
-  FREESURFACE(pTheme->NWEST_BORDER_Icon);
-  FREESURFACE(pTheme->NNORTH_BORDER_Icon);
-  FREESURFACE(pTheme->NSOUTH_BORDER_Icon);
-  FREESURFACE(pTheme->NEAST_BORDER_Icon);
-	
-  FREE(pTheme);
+  FC_FREE(pTheme);
   return;
 }
 
 /**************************************************************************
-  Return a surface for the given citizen.  The citizen's type is given,
-  as well as their index (in the range [0..pcity->size)).
+  ...
 **************************************************************************/
-SDL_Surface * get_citizen_surface(struct citizen_type type,
-				  int citizen_index)
+void setup_auxiliary_tech_icons(void)
 {
-  assert(type >= 0 && type < NUM_TILES_CITIZEN);
-  citizen_index %= sprites.citizen[type].count;
-  return citizen[type].surface[citizen_index];
+  SDL_Color bg_color = {255, 255, 255, 136};
+
+  SDL_Surface *pSurf;
+  SDL_String16 *pStr = create_str16_from_char(_("None"), adj_font(10));
+  
+  pStr->style |= (TTF_STYLE_BOLD | SF_CENTER);
+    
+  /* create icons */
+  pSurf = create_surf_alpha(adj_size(50), adj_size(50), SDL_SWSURFACE);
+  SDL_FillRect(pSurf, NULL, map_rgba(pSurf->format, bg_color));
+  putframe(pSurf, 0 , 0, pSurf->w - 1, pSurf->h - 1,
+         map_rgba(pSurf->format, *get_game_colorRGB(COLOR_THEME_SCIENCEDLG_FRAME)));
+
+  pNeutral_Tech_Icon = SDL_DisplayFormatAlpha(pSurf);
+  pNone_Tech_Icon = SDL_DisplayFormatAlpha(pSurf);    
+  pFuture_Tech_Icon = SDL_DisplayFormatAlpha(pSurf);
+  
+  FREESURFACE(pSurf);
+    
+  /* None */
+  pSurf = create_text_surf_from_str16(pStr);
+  blit_entire_src(pSurf, pNone_Tech_Icon ,
+	  (adj_size(50) - pSurf->w) / 2 , (adj_size(50) - pSurf->h) / 2);
+  
+  FREESURFACE(pSurf);
+  
+  /* TRANS: Future Technology */ 
+  copy_chars_to_string16(pStr, _("FT"));
+  pSurf = create_text_surf_from_str16(pStr);
+  blit_entire_src(pSurf, pFuture_Tech_Icon,
+	  (adj_size(50) - pSurf->w) / 2 , (adj_size(50) - pSurf->h) / 2);
+  
+  FREESURFACE(pSurf);
+  
+  FREESTRING16(pStr);
+    
 }
 
-void unload_unused_graphics(void)
+/**************************************************************************
+  ...
+**************************************************************************/
+void free_auxiliary_tech_icons(void)
 {
-  unload_sprite("treaty.disagree_thumb_down");
-  unload_sprite("treaty.agree_thumb_up");
-  unload_sprite("spaceship.solar_panels");
-  unload_sprite("spaceship.life_support");
-  unload_sprite("spaceship.habitation");
-  unload_sprite("spaceship.structural");
-  unload_sprite("spaceship.fuel");
-  unload_sprite("spaceship.propulsion");
-  unload_sprite("citizen.entertainer");
-  unload_sprite("citizen.scientist");
-  unload_sprite("citizen.tax_collector");
-  unload_sprite("citizen.content_0");
-  unload_sprite("citizen.content_1");
-  unload_sprite("citizen.happy_0");
-  unload_sprite("citizen.happy_1");
-  unload_sprite("citizen.unhappy_0");
-  unload_sprite("citizen.unhappy_1");
-  unload_sprite("citizen.angry_0");
-  unload_sprite("citizen.angry_1");
-  unload_sprite("s.right_arrow");
-  if (sprite_exists("t.coast_color"))
+  FREESURFACE(pNeutral_Tech_Icon);
+  FREESURFACE(pNone_Tech_Icon);
+  FREESURFACE(pFuture_Tech_Icon);
+}
+
+/**************************************************************************
+  ...
+**************************************************************************/
+SDL_Surface * get_tech_icon(Tech_type_id tech)
+{
+  switch(tech)
   {
-    unload_sprite("t.coast_color");
+    case A_NONE:
+    case A_UNSET:
+    case A_UNKNOWN:
+    case A_LAST:
+      return SDL_DisplayFormatAlpha(pNone_Tech_Icon);
+    case A_FUTURE:
+      return SDL_DisplayFormatAlpha(pFuture_Tech_Icon);
+    default:
+      if (get_tech_sprite(tileset, tech)) {
+        return adj_surf(GET_SURF(get_tech_sprite(tileset, tech)));
+      } else {
+        return SDL_DisplayFormatAlpha(pNeutral_Tech_Icon);
+      }
   }
-  unload_sprite("upkeep.gold");
-  unload_sprite("upkeep.gold2");
-  unload_sprite("upkeep.food");
-  unload_sprite("upkeep.food2");
-  unload_sprite("upkeep.unhappy");
-  unload_sprite("upkeep.unhappy2");
-  unload_sprite("upkeep.shield");
-  if (is_isometric && sprite_exists("explode.iso_nuke"))
+  return NULL;
+}
+
+/**************************************************************************
+  ...
+**************************************************************************/
+SDL_Color * get_tech_color(Tech_type_id tech_id)
+{
+  if (tech_is_available(game.player_ptr, tech_id))
   {
-    unload_sprite("explode.iso_nuke");
+    switch (get_invention(game.player_ptr, tech_id))
+    {
+      case TECH_UNKNOWN:
+        return get_game_colorRGB(COLOR_REQTREE_UNREACHABLE);	  
+      case TECH_KNOWN:
+        return get_game_colorRGB(COLOR_REQTREE_KNOWN);
+      case TECH_REACHABLE:
+        return get_game_colorRGB(COLOR_REQTREE_REACHABLE);
+      default:
+        return get_game_colorRGB(COLOR_REQTREE_BACKGROUND);
+    }
   }
+  return get_game_colorRGB(COLOR_REQTREE_UNREACHABLE);
+}
+
+/**************************************************************************
+  ...
+**************************************************************************/
+SDL_Surface * get_city_gfx(void)
+{
+  return pCity_Surf;
+}
+
+/**************************************************************************
+  ...
+**************************************************************************/
+void draw_intro_gfx(void)
+{
+  SDL_Surface *pIntro = theme_get_background(theme, BACKGROUND_MAINPAGE);
+
+  if(pIntro->w != Main.screen->w)
+  {
+    SDL_Surface *pTmp = ResizeSurface(pIntro, Main.screen->w, Main.screen->h,1);
+    FREESURFACE(pIntro);
+    pIntro = pTmp;
+  }
+  
+  /* draw intro gfx center in screen */
+  alphablit(pIntro, NULL, Main.map, NULL);
+  
+  FREESURFACE(pIntro);
 }
