@@ -15,16 +15,9 @@
 #include <config.h>
 #endif
 
-#include <SDL/SDL.h>
-
-#include "events.h"
-#include "fcintl.h"
+/* utility */
 #include "log.h"
 
-#include "graphics.h"
-#include "gui_main.h"
-#include "options.h"
-
 #include "messagedlg.h"
 
 /**************************************************************************
