@@ -10,15 +10,14 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/  
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
-#endif        
-#include <stdlib.h>
- 
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <stdarg.h>
                          
 #include <windows.h>
 #include <windowsx.h>
@@ -42,6 +41,7 @@
 #include "options.h"
 #include "repodlgs.h"
 #include "climisc.h"
+#include "text.h"
                            
 #include "cityrep.h"
 extern HINSTANCE freecivhinst;
@@ -55,9 +55,8 @@ int max_improvement_id;
 static int city_sort_id;
 static int city_sort_order;
 static HMENU menu_shown; 
-static HWND sort_buttons[NUM_CREPORT_COLS];
+static HWND *sort_buttons;
 HWND cityrep_list;
-typedef bool TestCityFunc(struct city *, int);     
 
 #define ID_CITYREP_SORTBASE 6400
 #define ID_CHANGE_POPUP_BASE 7000
@@ -126,7 +125,7 @@ static void create_cityrep_config_dlg(HWND hWnd)
   struct city_report_spec *spec;   
   box=fcwin_vbox_new(hWnd,FALSE);
   fcwin_box_add_static_default(box,_("Set columns shown"),-1,SS_CENTER);
-  for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {     
+  for(i=0, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) {     
     fcwin_box_add_checkbox(box, spec->explanation,
 			   ID_CITYREP_CONFIG_BASE+i,0,FALSE,FALSE,5);
   }
@@ -158,7 +157,7 @@ LONG APIENTRY ConfigCityRepProc(HWND hWnd,
 	{
 	  struct city_report_spec *spec;   
 	  int i;
-	  for(i=1, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) 
+	  for(i=0, spec=city_report_specs+i; i<NUM_CREPORT_COLS; i++, spec++) 
 	    {   
 	      spec->show=IsDlgButtonChecked(hWnd,ID_CITYREP_CONFIG_BASE+i);  
 	    }
@@ -186,9 +185,9 @@ append_impr_or_unit_to_menu_sub(HMENU menu,
 				int *selitems,
 				int selcount, int *idcount)
 {
-  cid cids[U_LAST + B_LAST];
-  struct item items[U_LAST + B_LAST];
-  int item, cids_used, num_selected_cities = 0;
+  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct item items[MAX_NUM_PRODUCTION_TARGETS];
+  int item, targets_used, num_selected_cities = 0;
   struct city **selected_cities = NULL;
 
   if (change_prod) {
@@ -205,26 +204,26 @@ append_impr_or_unit_to_menu_sub(HMENU menu,
     }
   }
 
-  cids_used = collect_cids1(cids, selected_cities,
+  targets_used = collect_production_targets(targets, selected_cities,
 			    num_selected_cities, append_units,
 			    append_wonders, change_prod, test_func);
   if (selected_cities) {
     free(selected_cities);
   }
-  name_and_sort_items(cids, cids_used, items, change_prod, NULL);
+  name_and_sort_items(targets, targets_used, items, change_prod, NULL);
 
-  for (item = 0; item < cids_used; item++) {
+  for (item = 0; item < targets_used; item++) {
     MENUITEMINFO iteminfo;
 
     AppendMenu(menu, MF_STRING, (*idcount), items[item].descr);
-    iteminfo.dwItemData = items[item].cid;
+    iteminfo.dwItemData = cid_encode(items[item].item);
     iteminfo.fMask = MIIM_DATA;
     iteminfo.cbSize = sizeof(MENUITEMINFO);
     SetMenuItemInfo(menu, (*idcount), FALSE, &iteminfo);
     (*idcount)++;
   }
 
-  if (cids_used == 0) {
+  if (targets_used == 0) {
     AppendMenu(menu, MF_STRING, -1, nothing_appended_text);
   }
 }
@@ -245,7 +244,6 @@ static void append_impr_or_unit_to_menu(HMENU menu,
     /* Add all buildings */
     append_impr_or_unit_to_menu_sub(menu, _("No Buildings Available"),
 				    FALSE, FALSE, change_prod,
-				    (bool (*)(struct city *, int))
 				    test_func,
 				    selitems, selcount, idcount);
     /* Add a separator */
@@ -269,7 +267,6 @@ static void append_impr_or_unit_to_menu(HMENU menu,
     /* Add all wonders */
     append_impr_or_unit_to_menu_sub(menu, _("No Wonders Available"),
 				    FALSE, TRUE, change_prod,
-				    (bool (*)(struct city *, int))
 				    test_func,
 				    selitems, selcount, idcount);
   }
@@ -300,7 +297,8 @@ static void get_city_text(struct city *pcity, char *buf[], int n)
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {
     buf[i][0]='\0';
     if(!spec->show) continue;
-    my_snprintf(buf[i], n, "%*s", NEG_VAL(spec->width)-2, (spec->func)(pcity)); 
+    my_snprintf(buf[i], n, "%*s", NEG_VAL(spec->width)-2,
+		(spec->func)(pcity, spec->data)); 
   }
 }
 
@@ -319,7 +317,7 @@ static void cityrep_center(HWND hWnd)
     return;
   pcity=(struct city *)ListBox_GetItemData(GetDlgItem(hWnd,ID_CITYREP_LIST),
 					   id);
-  center_tile_mapcanvas(pcity->x,pcity->y);
+  center_tile_mapcanvas(pcity->tile);
 }
 
 /**************************************************************************
@@ -340,59 +338,37 @@ static void cityrep_popup(HWND hWnd)
     {
       pcity=(struct city *)ListBox_GetItemData(GetDlgItem(hWnd,ID_CITYREP_LIST),
 				cityids[i]);
-      popup_city_dialog(pcity,0);
+      popup_city_dialog(pcity);
     }
 }
 
 /**************************************************************************
 
 **************************************************************************/
-static void cityrep_buy(HWND hWnd)
+static void cityrep_do_buy(HWND hWnd)
 {
   int cityids[256];
-  int selcount;
   int i;
-  struct city *pcity;
-   selcount=ListBox_GetSelCount(GetDlgItem(hWnd,ID_CITYREP_LIST));
-  if (selcount==LB_ERR) return;
-  selcount=MIN(256,selcount);
-  selcount=ListBox_GetSelItems(GetDlgItem(hWnd,ID_CITYREP_LIST),
-			       selcount,&cityids[0]);
-  for(i=0;i<selcount;i++)
-    {
-      int value;
-      char *name;
-      char buf[512];            
-      pcity=(struct city *)ListBox_GetItemData(GetDlgItem(hWnd,
-							  ID_CITYREP_LIST),
-					       cityids[i]);
-      value=city_buy_cost(pcity);
-      if(pcity->is_building_unit)
-        name=get_unit_type(pcity->currently_building)->name;
-      else
-        name=get_impr_name_ex(pcity, pcity->currently_building);
- 
-      if (game.player_ptr->economic.gold >= value)
-        {
-          struct packet_city_request packet;
-          packet.city_id=pcity->id;
-          send_packet_city_request(&aconnection, &packet, PACKET_CITY_BUY);
-        }
-      else
-        {
-          my_snprintf(buf, sizeof(buf),
-                      _("Game: %s costs %d gold and you only have %d gold."),
-                      name,value,game.player_ptr->economic.gold);
-          append_output_window(buf);
-        }             
-    }
+  int selcount = ListBox_GetSelCount(GetDlgItem(hWnd, ID_CITYREP_LIST));
+
+  if (selcount == LB_ERR) {
+    return;
+  }
+  selcount = MIN(256, selcount);
+  selcount = ListBox_GetSelItems(GetDlgItem(hWnd, ID_CITYREP_LIST),
+				 selcount, &cityids[0]);
+  for (i = 0; i < selcount; i++) {
+    cityrep_buy((struct city *) ListBox_GetItemData(GetDlgItem(hWnd,
+							       ID_CITYREP_LIST),
+						    cityids[i]));
+  }
 }
   
-
+#if 0
 /**************************************************************************
 
 **************************************************************************/
-void cityrep_refresh(HWND hWnd)
+static void cityrep_refresh(HWND hWnd)
 {
     int cityids[256];
   int selcount;
@@ -419,6 +395,8 @@ void cityrep_refresh(HWND hWnd)
 				  &packet);      
     }
 }
+#endif
+
 /**************************************************************************
 
 **************************************************************************/
@@ -491,7 +469,7 @@ static void cityrep_select(HWND hWnd)
   AppendMenu(popup,MF_POPUP,(UINT)submenu,_("Improvements in City"));
   max_improvement_id=ID_IMPROVEMENTS_POPUP_BASE;
   append_impr_or_unit_to_menu(submenu, FALSE, TRUE, FALSE,
-			      (TestCityFunc*)city_got_building,
+			      city_building_present,
 			      &cityids,selcount,
 			      &max_improvement_id);
   
@@ -507,9 +485,8 @@ static void cityrep_change_menu(HWND hWnd, cid cid)
 {  
   int cityids[256];
   int selcount, i, last_request_id = 0;
-  struct city *pcity; 
-  bool is_unit = cid_is_unit(cid);
-  int number = cid_id(cid);
+  struct city *pcity;
+  struct city_production target = cid_production(cid);
   
   selcount=ListBox_GetSelCount(GetDlgItem(hWnd,ID_CITYREP_LIST));
   if (selcount==LB_ERR) return;
@@ -519,15 +496,11 @@ static void cityrep_change_menu(HWND hWnd, cid cid)
 
   connection_do_buffer(&aconnection);
   for (i = 0; i < selcount; i++) {
-    struct packet_city_request packet;
     pcity = (struct city *) ListBox_GetItemData(GetDlgItem(hWnd,
 							   ID_CITYREP_LIST),
 						cityids[i]);
-    packet.city_id = pcity->id;
-    packet.build_id = number;
-    packet.is_build_id_unit_id = is_unit;
     last_request_id =
-	send_packet_city_request(&aconnection, &packet, PACKET_CITY_CHANGE);
+      city_change_production(pcity, target);
     ListBox_SetSel(GetDlgItem(hWnd, ID_CITYREP_LIST), FALSE, cityids[i]);
   }
 
@@ -575,8 +548,9 @@ static void list_coastal_select(HWND hLst)
   for (i=0;i<num;i++)
     {
       pcity=(struct city *)ListBox_GetItemData(hLst,i);
-      if (is_terrain_near_tile(pcity->x, pcity->y, T_OCEAN)) 
+      if (is_ocean_near_tile(pcity->tile)) {
 	ListBox_SetSel(hLst,TRUE,i);
+      }
     }
 }
 
@@ -603,8 +577,8 @@ static void list_sameisland_select(HWND hLst)
 	{
 	  struct city *selectedcity;
 	  selectedcity=(struct city *)ListBox_GetItemData(hLst,cityids[j]);
-          if (map_get_continent(pcity->x, pcity->y)
-              == map_get_continent(selectedcity->x, selectedcity->y))
+          if (tile_get_continent(pcity->tile)
+              == tile_get_continent(selectedcity->tile))
 	    {    
 	      ListBox_SetSel(hLst,TRUE,i);
 	      break;
@@ -616,7 +590,9 @@ static void list_sameisland_select(HWND hLst)
 /**************************************************************************
 
 **************************************************************************/
-void list_impr_or_unit_select(HWND hLst,int num, TestCityFunc *test_func)
+static void list_impr_or_unit_select(HWND hLst,
+				     struct city_production target,
+				     TestCityFunc test_func)
 {
   int i,rows;
   list_all_select(hLst,FALSE);
@@ -624,7 +600,7 @@ void list_impr_or_unit_select(HWND hLst,int num, TestCityFunc *test_func)
   for (i=0;i<rows;i++)
     {
       struct city *pcity=(struct city *)ListBox_GetItemData(hLst,i);
-      if (test_func(pcity,num))
+      if (test_func(pcity, target))
 	ListBox_SetSel(hLst,TRUE,i);
     }
 }
@@ -635,6 +611,7 @@ void list_impr_or_unit_select(HWND hLst,int num, TestCityFunc *test_func)
 static void menu_proc(HWND hWnd,int cmd, DWORD num)
 {
   HWND hLst;
+  struct city_production target = cid_decode(num);
   hLst=GetDlgItem(hWnd,ID_CITYREP_LIST);
   if ((cmd>=ID_CHANGE_POPUP_BASE)&&
       (cmd<max_changemenu_id))
@@ -645,25 +622,25 @@ static void menu_proc(HWND hWnd,int cmd, DWORD num)
   if ((cmd>=ID_SUPPORTED_POPUP_BASE)&&
       (cmd<max_supportmenu_id))
     {
-      list_impr_or_unit_select(hLst,num,city_unit_supported);
+      list_impr_or_unit_select(hLst, target, city_unit_supported);
       max_supportmenu_id=0;
     }
   if ((cmd>=ID_PRESENT_POPUP_BASE)&&
       (cmd<max_presentmenu_id))
     {
-      list_impr_or_unit_select(hLst,num,city_unit_present);
+      list_impr_or_unit_select(hLst, target, city_unit_present);
       max_presentmenu_id=0;
     }
   if ((cmd>=ID_AVAILABLE_POPUP_BASE)&&
       (cmd<max_availablemenu_id))
     {
-      list_impr_or_unit_select(hLst,num,city_can_build_impr_or_unit);
+      list_impr_or_unit_select(hLst, target, city_can_build_impr_or_unit);
       max_availablemenu_id=0;
     }
   if ((cmd>=ID_IMPROVEMENTS_POPUP_BASE)&&
       (cmd<max_improvement_id))
     {
-      list_impr_or_unit_select(hLst,num,city_got_building);
+      list_impr_or_unit_select(hLst, target, city_building_present);
       max_improvement_id=0;
     }
 }
@@ -698,12 +675,12 @@ static LONG CALLBACK cityrep_changeall_proc(HWND hWnd,
 	case ID_PRODCHANGE_CHANGE:
 	  {
 	    int id;
-	    cid from, to;
+	    int from, to;
 
 	    id=ListBox_GetCurSel(GetDlgItem(hWnd,ID_PRODCHANGE_FROM));
 	    if (id==LB_ERR)
 	      {
-		append_output_window(_("Game: Select a unit or improvement"
+		append_output_window(_("Select a unit or improvement"
 				       " to change production from."));
 		break;        
 	      }
@@ -712,16 +689,16 @@ static LONG CALLBACK cityrep_changeall_proc(HWND hWnd,
 	    id=ListBox_GetCurSel(GetDlgItem(hWnd,ID_PRODCHANGE_TO));
 	    if (id==LB_ERR)
 	      {
-		append_output_window(_("Game: Select a unit or improvement"
+		append_output_window(_("Select a unit or improvement"
 				       " to change production to."));
 		break;          
 	      }
 	    to=ListBox_GetItemData(GetDlgItem(hWnd,ID_PRODCHANGE_TO),id);
 	    if (from==to) {
-	      append_output_window(_("Game: That's the same thing!"));
+	      append_output_window(_("That's the same thing!"));
 	      break;
 	    }
-	    client_change_all(from,to);
+	    client_change_all(cid_decode(from), cid_decode(to));
 	    DestroyWindow(hWnd);
 	    hChangeAll=NULL;
 	  }
@@ -743,10 +720,10 @@ static void cityrep_changeall(HWND hWnd)
   struct fcwin_box *vbox;
   struct fcwin_box *hbox;
   int selid;
-  cid cids[B_LAST + U_LAST];
-  int cids_used;
+  struct city_production targets[MAX_NUM_PRODUCTION_TARGETS];
+  struct item items[MAX_NUM_PRODUCTION_TARGETS];
+  int targets_used;
   cid selected_cid;
-  struct item items[U_LAST + B_LAST];
   int id,i;
   HWND hDlg;
   HWND hLst;
@@ -787,24 +764,24 @@ static void cityrep_changeall(HWND hWnd)
 						 selid));
   }
 
-  cids_used = collect_cids2(cids);
-  name_and_sort_items(cids, cids_used, items, FALSE, NULL);
+  targets_used = collect_currently_building_targets(targets);
+  name_and_sort_items(targets, targets_used, items, FALSE, NULL);
 
   hLst = GetDlgItem(hDlg, ID_PRODCHANGE_FROM);
-  for (i = 0; i < cids_used; i++) {
+  for (i = 0; i < targets_used; i++) {
     id = ListBox_AddString(hLst, items[i].descr);
-    ListBox_SetItemData(hLst, id, items[i].cid);
-    if (items[i].cid == selected_cid)
+    ListBox_SetItemData(hLst, id, cid_encode(items[i].item));
+    if (cid_encode(items[i].item) == selected_cid)
       ListBox_SetCurSel(hLst, id);
   }
 
-  cids_used = collect_cids3(cids);
-  name_and_sort_items(cids, cids_used, items, TRUE, NULL);
+  targets_used = collect_buildable_targets(targets);
+  name_and_sort_items(targets, targets_used, items, TRUE, NULL);
 
   hLst = GetDlgItem(hDlg, ID_PRODCHANGE_TO);
-  for (i = 0; i < cids_used; i++) {
+  for (i = 0; i < targets_used; i++) {
     id = ListBox_AddString(hLst, items[i].descr);
-    ListBox_SetItemData(hLst, id, items[i].cid);
+    ListBox_SetItemData(hLst, id, cid_encode(items[i].item));
   }
 
   fcwin_set_box(hDlg,hbox);
@@ -888,8 +865,8 @@ static void list_del(void *data)
 **************************************************************************/
 static void city_report_create(HWND hWnd)
 {
-  static char *titles   [NUM_CREPORT_COLS];
-  static char  buf      [NUM_CREPORT_COLS][64];   
+  static char **titles;
+  static char (*buf)[64];   
   struct city_report_spec *spec;        
   struct fcwin_box *vbox;
   struct fcwin_box *hbox;
@@ -911,24 +888,35 @@ static void city_report_create(HWND hWnd)
 			TRUE,TRUE,5);
   fcwin_box_add_box(vbox,hbox,FALSE,FALSE,5);
   
- 
+  if (titles) {
+    free(titles);
+  }
+  if (buf) {
+    free(buf);
+  }
+  if (sort_buttons) {
+    free(sort_buttons);
+  }
+  
+  titles = fc_malloc(sizeof(*titles) * NUM_CREPORT_COLS);
+  buf = fc_malloc(sizeof(*buf) * NUM_CREPORT_COLS);
+  sort_buttons = fc_malloc(sizeof(*sort_buttons) * NUM_CREPORT_COLS);
+
   for (i=0;i<NUM_CREPORT_COLS;i++)
     titles[i]=buf[i];
   
   get_city_table_header(titles, sizeof(buf[0]));  
   for(i=0, spec=city_report_specs; i<NUM_CREPORT_COLS; i++, spec++) {         
-    if (spec->show)
-      {
-	sort_buttons[i]=CreateWindow("BUTTON",titles[i],
-				     WS_CHILD | WS_VISIBLE | BS_MULTILINE,
-				     0,0,0,0,
-				     hWnd,
-				     (HMENU)(ID_CITYREP_SORTBASE+i),
-				     freecivhinst,
-				     NULL);
-	SendMessage(sort_buttons[i],
-		    WM_SETFONT,(WPARAM) font_12courier,MAKELPARAM(TRUE,0));
- 				           }
+    if (spec->show) {
+      sort_buttons[i]=CreateWindow("BUTTON", titles[i], WS_CHILD | WS_VISIBLE
+				   | BS_MULTILINE, 0, 0, 0, 0, hWnd,
+				   (HMENU)(ID_CITYREP_SORTBASE+i),
+				   freecivhinst, NULL);
+      SendMessage(sort_buttons[i], WM_SETFONT, (WPARAM)font_12courier,
+		  MAKELPARAM(TRUE,0));
+    } else {
+      sort_buttons[i] = NULL;
+    }
   }
 
   cityrep_list=CreateWindow("LISTBOX",NULL,WS_CHILD | WS_VISIBLE | 
@@ -1017,7 +1005,7 @@ static LONG APIENTRY city_report_proc(HWND hWnd,
 	  cityrep_popup(hWnd);
 	  break;
 	case ID_CITYREP_BUY:
-	  cityrep_buy(hWnd);
+	  cityrep_do_buy(hWnd);
 	  break;
 	case ID_CITYREP_CHANGE:
 	  cityrep_change(hWnd);
@@ -1082,21 +1070,24 @@ static LONG APIENTRY city_report_proc(HWND hWnd,
 
 **************************************************************************/
 void
-popup_city_report_dialog(bool make_modal)
+popup_city_report_dialog(bool raise)
 {
-  if (hCityRep) return;
-  hCityRep=
-    fcwin_create_layouted_window(city_report_proc,_("City Report"),
-				 WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX | 
-				 WS_MAXIMIZEBOX | WS_THICKFRAME,
-				 CW_USEDEFAULT,CW_USEDEFAULT,
-				 root_window,NULL,
-				 JUST_CLEANUP,
-				 NULL);
+  if (!hCityRep) {
+    hCityRep =
+      fcwin_create_layouted_window(city_report_proc,_("City Report"),
+				   WS_OVERLAPPED | WS_SYSMENU | WS_MINIMIZEBOX
+				    | WS_MAXIMIZEBOX | WS_THICKFRAME,
+				   CW_USEDEFAULT, CW_USEDEFAULT,
+				   root_window, NULL, JUST_CLEANUP, NULL);
   
-  hChangeAll=NULL;
-  city_report_dialog_update();
-  ShowWindow(hCityRep,SW_SHOWNORMAL);
+    hChangeAll = NULL;
+    city_report_dialog_update();
+  }
+
+  ShowWindow(hCityRep, SW_SHOWNORMAL);
+  if (raise) {
+    SetFocus(hCityRep);
+  }
 }
 
 /**************************************************************************
@@ -1119,7 +1110,7 @@ static void my_cityrep_add_city(HWND hLst,struct city *pcity_new,char *full_row,
   int i,n,listcount;  
   struct city_report_spec *spec; 
   char *rowold   [NUM_CREPORT_COLS];
-  char  buf   [NUM_CREPORT_COLS][64];
+  char  buf   [NUM_CREPORT_COLS][128];
   for (i=0, spec=city_report_specs;i<NUM_CREPORT_COLS;i++, spec++)
     {
       rowold[i] = buf[i];    
@@ -1149,16 +1140,15 @@ void
 city_report_dialog_update(void)
 { 
   char *row   [NUM_CREPORT_COLS];
-  char  buf   [NUM_CREPORT_COLS][64];
+  char  buf   [NUM_CREPORT_COLS][128];
   char full_row[1024];
   int   i;
-  char *report_title;
   struct city_report_spec *spec; 
   if(is_report_dialogs_frozen()) return;    
   if (!hCityRep)
     return;
-  report_title=get_report_title(_("City Advisor"));      
-  SetWindowText(GetDlgItem(hCityRep,ID_CITYREP_TOP),report_title);
+  SetWindowText(GetDlgItem(hCityRep, ID_CITYREP_TOP),
+		get_report_title(_("City Advisor")));
   for (i=0, spec=city_report_specs;i<NUM_CREPORT_COLS;i++, spec++)
     {
       row[i] = buf[i];    
@@ -1187,7 +1177,7 @@ void
 city_report_dialog_update_city(struct city *pcity)
 {
   char *row [NUM_CREPORT_COLS];
-  char  buf [NUM_CREPORT_COLS][64];
+  char  buf [NUM_CREPORT_COLS][128];
   char full_row[512];
   int   i,nCount;     
   struct city_report_spec *spec; 
@@ -1216,3 +1206,21 @@ city_report_dialog_update_city(struct city *pcity)
 	}
     }
 }
+
+/****************************************************************
+ After a selection rectangle is defined, make the cities that
+ are hilited on the canvas exclusively hilited in the
+ City List window.
+*****************************************************************/
+void hilite_cities_from_canvas(void)
+{
+  /* PORTME */
+}
+
+/****************************************************************
+ Toggle a city's hilited status.
+*****************************************************************/
+void toggle_city_hilite(struct city *pcity, bool on_off)
+{
+  /* PORTME */
+}
