@@ -1,4 +1,4 @@
-/********************************************************************** 
+/**********************************************************************
  Freeciv - Copyright (C) 1996 - A Kjeldberg, L Gregersen, P Unold
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -28,11 +28,13 @@
 #include "player.h"
 #include "support.h"
 #include "unit.h"
+#include "unitlist.h"
 
 #include "clinet.h"
 #include "civclient.h"
 #include "control.h"
 #include "dialogs.h"
+#include "goto.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -78,8 +80,8 @@ static LONG CALLBACK goto_dialog_proc(HWND dlg,UINT message,
       switch(LOWORD(wParam))
 	{
 	case ID_LIST:
-	  if((pdestcity=get_selected_city())) {
-	    struct unit *punit=get_unit_in_focus();
+	  if (NULL != (pdestcity=get_selected_city())) {
+	    struct unit *punit = head_of_units_in_focus();
 	    center_tile_mapcanvas(pdestcity->tile);
 	    if(punit && unit_can_airlift_to(punit, pdestcity)) {
 	      EnableWindow(GetDlgItem(dlg,ID_AIRLIFT),TRUE);
@@ -97,11 +99,10 @@ static LONG CALLBACK goto_dialog_proc(HWND dlg,UINT message,
 	  {
 	    pdestcity=get_selected_city();
 	    if (pdestcity) {
-	      struct unit *punit=get_unit_in_focus();
-	      if (punit) {
-		send_goto_unit(punit, pdestcity->tile);
-		DestroyWindow(dlg);
-	      }
+	      unit_list_iterate(get_units_in_focus(), punit) {
+      		send_goto_tile(punit, pdestcity->tile);
+      	} unit_list_iterate_end;
+		    DestroyWindow(dlg);
 	    }
 	  }
 	  break;
@@ -109,13 +110,12 @@ static LONG CALLBACK goto_dialog_proc(HWND dlg,UINT message,
 	  {
 	    pdestcity=get_selected_city();
 	    if (pdestcity) {
-	      struct unit *punit=get_unit_in_focus();
-	      if (punit) {
-		request_unit_airlift(punit, pdestcity);
-		DestroyWindow(dlg);
-	      }
+	      unit_list_iterate(get_units_in_focus(), punit) {
+          request_unit_airlift(punit, pdestcity);
+      	} unit_list_iterate_end;
+        DestroyWindow(dlg);
 	    }
-	 
+
 	  }
 	  break;
 	case IDCANCEL:
@@ -143,11 +143,12 @@ popup_goto_dialog(void)
   if (!can_client_change_view()) {
     return;
   }
-  if (get_unit_in_focus()==0)
+  if (get_num_units_in_focus()==0) {
     return;
+  }
 
   original_tile = get_center_tile_mapcanvas();
-  
+
   goto_dialog=fcwin_create_layouted_window(goto_dialog_proc,
 					   _("Goto/Airlift Unit"),
 					   WS_OVERLAPPEDWINDOW,
@@ -185,10 +186,12 @@ static void update_goto_dialog(HWND list)
   ListBox_ResetContent(list);
   Button_SetState(GetDlgItem(goto_dialog,ID_ALL),show_all_cities);
 
-  for(i=0; i<game.nplayers; i++) {
-    if(!show_all_cities && i!=game.player_idx) continue;
+  for(i=0; i<game.info.nplayers; i++) {
+    if (!show_all_cities && i != game.info.player_idx) {
+      continue;
+    }
     city_list_iterate(game.players[i].cities, pcity) {
-      sz_strlcpy(name, pcity->name);
+      sz_strlcpy(name, city_name(pcity));
       /* FIXME: should use unit_can_airlift_to(). */
       if (pcity->airlift) {
         sz_strlcat(name, "(A)");
@@ -208,6 +211,6 @@ static struct city *get_selected_city(void)
   int selection;  
   if ((selection=ListBox_GetCurSel(goto_list))==LB_ERR)
     return 0;
-  return find_city_by_id(ListBox_GetItemData(goto_list,selection));
+  return game_find_city_by_number(ListBox_GetItemData(goto_list,selection));
 
 }
