@@ -58,7 +58,7 @@ static int sort_column=2;
 #define ID_PLAYERS_VISION 105
 #define ID_PLAYERS_SSHIP 106
 
-#define NUM_COLUMNS 10
+#define NUM_COLUMNS 9
 
 /******************************************************************
 
@@ -69,7 +69,7 @@ static void players_meet(int player_index)
     dsend_packet_diplomacy_init_meeting_req(&aconnection, player_index);
 
   } else {
-    append_output_window(_("Game: You need an embassy to "
+    append_output_window(_("You need an embassy to "
 			   "establish a diplomatic meeting."));
   }
 }
@@ -117,7 +117,7 @@ static void players_sship(int player_index)
 static void build_row(const char **row, int i, int update)
 {
   static char namebuf[MAX_LEN_NAME],  aibuf[2], dsbuf[32],
-      repbuf[32], statebuf[32], idlebuf[32];
+      statebuf[32], idlebuf[32];
   const struct player_diplstate *pds;
 
   /* we cassume that neither name nor the nation of a player changes */
@@ -128,7 +128,7 @@ static void build_row(const char **row, int i, int update)
 
 
     /* the nation */
-    row[1] = get_nation_name(game.players[i].nation);
+    row[1] = nation_adjective_for_player(&game.players[i]);
   }
 
   /* text for name, plus AI marker */
@@ -136,7 +136,7 @@ static void build_row(const char **row, int i, int update)
   aibuf[1] = '\0';
 
   /* text for diplstate type and turns -- not applicable if this is me */
-  if (i == game.player_idx) {
+  if (i == game.info.player_idx) {
     strcpy(dsbuf, "-");
   } else {
     pds = pplayer_get_diplstate(game.player_ptr, get_player(i));
@@ -151,7 +151,7 @@ static void build_row(const char **row, int i, int update)
   /* text for state */
   if (game.players[i].is_alive) {
     if (game.players[i].is_connected) {
-      if (game.players[i].turn_done) {
+      if (game.players[i].phase_done) {
 	sz_strlcpy(statebuf, _("done"));
       } else {
 	sz_strlcpy(statebuf, _("moving"));
@@ -173,19 +173,14 @@ static void build_row(const char **row, int i, int update)
     idlebuf[0] = '\0';
   }
 
-  /* text for reputation */
-  my_snprintf(repbuf, sizeof(repbuf),
-	      reputation_text(game.players[i].reputation));
-
   /* assemble the whole lot */
   row[2] = aibuf;
   row[3] = get_embassy_status(game.player_ptr, &game.players[i]);
   row[4] = dsbuf;
   row[5] = get_vision_status(game.player_ptr, &game.players[i]);
-  row[6] = repbuf;
-  row[7] = statebuf;
-  row[8] = (char *) player_addr_hack(&game.players[i]);	/* Fixme */
-  row[9] = idlebuf;
+  row[6] = statebuf;
+  row[7] = (char *) player_addr_hack(&game.players[i]);	/* Fixme */
+  row[8] = idlebuf;
 }
 
 
@@ -338,12 +333,15 @@ static void create_players_dialog(void)
   int i;
   static char *titles_[NUM_COLUMNS] =
     { N_("Name"), N_("Nation"), N_("AI"),
-      N_("Embassy"), N_("Dipl.State"), N_("Vision"), N_("Reputation"),
+      N_("Embassy"), N_("Dipl.State"), N_("Vision"),
       N_("State"), N_("Host"), N_("Idle")
     };
   struct fcwin_box *vbox;
   struct fcwin_box *hbox;
-  players_dialog=fcwin_create_layouted_window(players_proc,_("Players"),
+
+  players_dialog=fcwin_create_layouted_window(players_proc,
+					      /* TRANS: Nations report title */
+					      _("Nations"),
 					      WS_OVERLAPPEDWINDOW,
 					      CW_USEDEFAULT,CW_USEDEFAULT,
 					      root_window,NULL,
@@ -381,11 +379,14 @@ static void create_players_dialog(void)
 
 *******************************************************************/      
 void
-popup_players_dialog(void)
+popup_players_dialog(bool raise)
 {
   if (!players_dialog)
     create_players_dialog();
   ShowWindow(players_dialog,SW_SHOWNORMAL);
+  if (raise) {
+    SetFocus(players_dialog);
+  }
 }
 
 /**************************************************************************
@@ -401,7 +402,7 @@ update_players_dialog(void)
     int i,row;
     lv=GetDlgItem(players_dialog,ID_PLAYERS_LIST);
     ListView_DeleteAllItems(lv);
-    for (i = 0; i < game.nplayers; i++) {
+    for (i = 0; i < game.info.nplayers; i++) {
       build_row(row_texts, i, 0);
       row=fcwin_listview_add_row(lv,i,NUM_COLUMNS, (char **)row_texts);
       lvi.iItem=row;
