@@ -82,15 +82,12 @@ static Widget help_unit_tile;
 
 static Widget help_terrain_movement_defense, help_terrain_movement_defense_data;
 static Widget help_terrain_food_shield_trade, help_terrain_food_shield_trade_data;
-static Widget help_terrain_special_1, help_terrain_special_1_data;
-static Widget help_terrain_special_2, help_terrain_special_2_data;
+static Widget help_terrain_resources;
 static Widget help_terrain_road_result_time, help_terrain_road_result_time_data;
 static Widget help_terrain_irrigation_result_time, help_terrain_irrigation_result_time_data;
 static Widget help_terrain_mining_result_time, help_terrain_mining_result_time_data;
 static Widget help_terrain_transform_result_time, help_terrain_transform_result_time_data;
 
-/* HACK: we use a static string for convenience. */
-static char long_buffer[64000];
 
 static void create_help_page(enum help_page_type type);
 
@@ -281,7 +278,7 @@ static void create_tech_tree(Widget tree, Widget parent, int tech, int levels)
   }
   
   my_snprintf(label, sizeof(label),
-	      "%s:%d", advances[tech].name,
+	      "%s:%d", advance_name_translation(tech),
 	      num_unknown_techs_for_goal(game.player_ptr, tech));
 
   if(parent) {
@@ -490,8 +487,8 @@ static void create_help_page(enum help_page_type type)
       XtVaCreateManagedWidget("helpunittile",
 			      labelWidgetClass,
 			      help_right_form,
-			      XtNwidth, UNIT_TILE_WIDTH,
-			      XtNheight, UNIT_TILE_HEIGHT,
+			      XtNwidth, tileset_full_tile_width(tileset),
+			      XtNheight, tileset_full_tile_height(tileset),
 			      NULL);  
     XtAddCallback(help_unit_tile,
                   XtNdestroyCallback,free_bitmap_destroy_callback,
@@ -645,32 +642,10 @@ static void create_help_page(enum help_page_type type)
        help_right_form,
        NULL);
 
-    help_terrain_special_1 =
+    help_terrain_resources =
       XtVaCreateManagedWidget
       (
-       "help_terrain_special_1",
-       labelWidgetClass,
-       help_right_form,
-       NULL);
-    help_terrain_special_1_data =
-      XtVaCreateManagedWidget
-      (
-       "help_terrain_special_1_data",
-       labelWidgetClass,
-       help_right_form,
-       NULL);
-
-    help_terrain_special_2 =
-      XtVaCreateManagedWidget
-      (
-       "help_terrain_special_2",
-       labelWidgetClass,
-       help_right_form,
-       NULL);
-    help_terrain_special_2_data =
-      XtVaCreateManagedWidget
-      (
-       "help_terrain_special_2_data",
+       "help_terrain_resources",
        labelWidgetClass,
        help_right_form,
        NULL);
@@ -747,21 +722,38 @@ static void help_update_improvement(const struct help_item *pitem,
   
   create_help_page(HELP_IMPROVEMENT);
   
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
+  if (which<game.control.num_impr_types) {
+    struct impr_type *imp = improvement_by_number(which);
+    int i;
+    char req_buf[512];
+
     sprintf(buf, "%d ", impr_build_shield_cost(which));
     xaw_set_label(help_improvement_cost_data, buf);
     sprintf(buf, "%d ", imp->upkeep);
     xaw_set_label(help_improvement_upkeep_data, buf);
-    if (imp->tech_req == A_LAST) {
+    /*if (imp->tech_req == A_LAST) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advances[imp->tech_req].name);
-    }
-    create_tech_tree(help_tech_tree, 0, imp->tech_req, 3);
-  }
-  else {
+		    advance_name_translation(imp->tech_req));
+    }*/
+    
+    /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+     * Currently it's limited to 1 req but this code is partially prepared
+     * to be extended.  Remember MAX_NUM_REQS is a compile-time
+     * definition. */
+    i = 0;
+    requirement_vector_iterate(&imp->reqs, preq) {
+      xaw_set_label(help_improvement_req_data,
+		    get_req_source_text(&preq->source,
+					req_buf, sizeof(req_buf)));
+      i++;
+      break;
+    } requirement_vector_iterate_end;
+/*    create_tech_tree(help_tech_tree, 0,
+                     imp->req[0].source.value.tech, 3);
+*/
+  } else {
     xaw_set_label(help_improvement_cost_data, "0 ");
     xaw_set_label(help_improvement_upkeep_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
@@ -782,30 +774,48 @@ static void help_update_wonder(const struct help_item *pitem,
   
   create_help_page(HELP_WONDER);
 
-  if (which<game.num_impr_types) {
-    struct impr_type *imp = &improvement_types[which];
+  if (which<game.control.num_impr_types) {
+    struct impr_type *imp = improvement_by_number(which);
+    int i;
+    char req_buf[512];
+
     sprintf(buf, "%d ", impr_build_shield_cost(which));
     xaw_set_label(help_improvement_cost_data, buf);
-    if (imp->tech_req == A_LAST) {
+    /*if (imp->tech_req == A_LAST) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advances[imp->tech_req].name);
-    }
+		    advance_name_translation(imp->tech_req));
+    }*/
+
+     /* FIXME: this should show ranges and all the MAX_NUM_REQS reqs. 
+      * Currently it's limited to 1 req but this code is partially prepared
+      * to be extended.  Remember MAX_NUM_REQS is a compile-time
+      * definition. */
+    i = 0;
+    requirement_vector_iterate(&imp->reqs, preq) {
+      xaw_set_label(help_improvement_req_data,
+                   get_req_source_text(&preq->source,
+                                       req_buf, sizeof(req_buf)));
+      i++;
+      break;
+    } requirement_vector_iterate_end;
+
     if (tech_exists(imp->obsolete_by)) {
       xaw_set_label(help_wonder_obsolete_data,
-		    advances[imp->obsolete_by].name);
+		    advance_name_translation(imp->obsolete_by));
     } else {
       xaw_set_label(help_wonder_obsolete_data, _("(Never)"));
     }
-    create_tech_tree(help_tech_tree, 0, imp->tech_req, 3);
-  }
-  else {
+/*    create_tech_tree(help_tech_tree, 0, 
+                     imp->req[0].source.value.tech, 3);
+*/
+  } else {
     /* can't find wonder */
     xaw_set_label(help_improvement_cost_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
     xaw_set_label(help_wonder_obsolete_data, _("None"));
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3); 
+    create_tech_tree(help_tech_tree, 0, game.control.num_tech_types, 3); 
   }
   set_title_topic(pitem);
   helptext_building(buf, sizeof(buf), which, pitem->text);
@@ -816,43 +826,45 @@ static void help_update_wonder(const struct help_item *pitem,
 ...
 **************************************************************************/
 static void help_update_unit_type(const struct help_item *pitem,
-				  char *title, int i)
+				  char *title, struct unit_type *punittype)
 {
-  char *buf = &long_buffer[0];
+  char buf[64000];
   
   create_help_page(HELP_UNIT);
-  if (i<game.num_unit_types) {
-    struct unit_type *utype = get_unit_type(i);
-    sprintf(buf, "%d ", unit_build_shield_cost(i));
+  if (punittype) {
+/*    struct unit_type *punittype = utype_by_number(i);*/
+    sprintf(buf, "%d ", unit_build_shield_cost(punittype));
     xaw_set_label(help_unit_cost_data, buf);
-    sprintf(buf, "%d ", utype->attack_strength);
+    sprintf(buf, "%d ", punittype->attack_strength);
     xaw_set_label(help_unit_attack_data, buf);
-    sprintf(buf, "%d ", utype->defense_strength);
+    sprintf(buf, "%d ", punittype->defense_strength);
     xaw_set_label(help_unit_def_data, buf);
-    sprintf(buf, "%d ", utype->move_rate/3);
+    sprintf(buf, "%d ", punittype->move_rate/3);
     xaw_set_label(help_unit_move_data, buf);
-    sprintf(buf, "%d ", utype->firepower);
+    sprintf(buf, "%d ", punittype->firepower);
     xaw_set_label(help_unit_fp_data, buf);
-    sprintf(buf, "%d ", utype->hp);
+    sprintf(buf, "%d ", punittype->hp);
     xaw_set_label(help_unit_hp_data, buf);
-    sprintf(buf, "%d ", utype->vision_range);
+    sprintf(buf, "%d ", (int)sqrt((double)punittype->vision_radius_sq));
     xaw_set_label(help_unit_visrange_data, buf);
-    xaw_set_label(help_unit_upkeep_data, helptext_unit_upkeep_str(i));
-    if(utype->tech_requirement==A_LAST) {
+    xaw_set_label(help_unit_upkeep_data,
+		  helptext_unit_upkeep_str(punittype));
+    if (punittype->tech_requirement == A_LAST) {
       xaw_set_label(help_improvement_req_data, _("(Never)"));
     } else {
       xaw_set_label(help_improvement_req_data,
-		    advances[utype->tech_requirement].name);
+		    advance_name_for_player(game.player_ptr,
+				  punittype->tech_requirement));
     }
-    create_tech_tree(help_tech_tree, 0, utype->tech_requirement, 3);
-    if(utype->obsoleted_by==-1) {
+    create_tech_tree(help_tech_tree, 0, punittype->tech_requirement, 3);
+    if (punittype->obsoleted_by == U_NOT_OBSOLETED) {
       xaw_set_label(help_wonder_obsolete_data, _("None"));
     } else {
       xaw_set_label(help_wonder_obsolete_data,
-		    get_unit_type(utype->obsoleted_by)->name);
+		    utype_name_translation(punittype->obsoleted_by));
     }
     /* add text for transport_capacity, fuel, and flags: */
-    helptext_unit(buf, i, pitem->text);
+    helptext_unit(buf, sizeof(buf), punittype, pitem->text);
     XtVaSetValues(help_text, XtNstring, buf, NULL);
   }
   else {
@@ -864,11 +876,11 @@ static void help_update_unit_type(const struct help_item *pitem,
     xaw_set_label(help_unit_hp_data, "0 ");
     xaw_set_label(help_unit_visrange_data, "0 ");
     xaw_set_label(help_improvement_req_data, _("(Never)"));
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, game.control.num_tech_types, 3);
     xaw_set_label(help_wonder_obsolete_data, _("None"));
     XtVaSetValues(help_text, XtNstring, pitem->text, NULL);
   }
-  xaw_set_bitmap(help_unit_tile, create_overlay_unit(i));
+  xaw_set_bitmap(help_unit_tile, create_overlay_unit(punittype));
   set_title_topic(pitem);
 }
 
@@ -877,7 +889,7 @@ static void help_update_unit_type(const struct help_item *pitem,
 **************************************************************************/
 static void help_update_tech(const struct help_item *pitem, char *title, int i)
 {
-  char *buf = &long_buffer[0];
+  char buf[4096];
   int j;
 
   create_help_page(HELP_TECH);
@@ -885,45 +897,54 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 
   if (!is_future_tech(i)) {
     create_tech_tree(help_tech_tree, 0, i, 3);
-    helptext_tech(buf, i, pitem->text);
+    helptext_tech(buf, sizeof(buf), i, pitem->text);
 
-    impr_type_iterate(j) {
-      if(i==improvement_types[j].tech_req) 
+    impr_type_iterate(impr_t) {
+      /*if(i==improvement_types[j].tech_req) 
 	sprintf(buf+strlen(buf), _("Allows %s.\n"),
 		improvement_types[j].name);
-      if(i==improvement_types[j].obsolete_by)
+      */
+
+       /* FIXME: need a more general mechanism for this, since this
+        * helptext needs to be shown in all possible req source types. */
+      requirement_vector_iterate(&improvement_by_number(impr_t)->reqs, preq) {
+	if (preq->source.type == REQ_BUILDING
+	    && preq->source.value.building == i) {
+	  sprintf(buf+strlen(buf), _("Allows %s.\n"),
+		  improvement_name_translation(impr_t));
+        }
+      } requirement_vector_iterate_end;
+      if (i == improvement_by_number(impr_t)->obsolete_by)
 	sprintf(buf+strlen(buf), _("Obsoletes %s.\n"),
-		improvement_types[j].name);
+		improvement_name_translation(impr_t));
     } impr_type_iterate_end;
 
-    unit_type_iterate(j) {
-      if(i==get_unit_type(j)->tech_requirement) 
-	sprintf(buf+strlen(buf), _("Allows %s.\n"), 
-		get_unit_type(j)->name);
+    unit_type_iterate(punittype) {
+      if (i == punittype->tech_requirement) 
+	sprintf(buf + strlen(buf), _("Allows %s.\n"),
+	        utype_name_translation(punittype));
     } unit_type_iterate_end;
 
-    for (j = 0; j < game.num_tech_types; j++) {
+    for (j = 0; j < game.control.num_tech_types; j++) {
       if(i==advances[j].req[0]) {
 	if(advances[j].req[1]==A_NONE)
 	  sprintf(buf+strlen(buf), _("Allows %s.\n"), 
-		  advances[j].name);
+		  advance_name_translation(j));
 	else
 	  sprintf(buf+strlen(buf), _("Allows %s (with %s).\n"), 
-		  advances[j].name, advances[advances[j].req[1]].name);
+		  advance_name_translation(j),
+		  advance_name_translation(advances[j].req[1]));
       }
       if(i==advances[j].req[1]) {
 	sprintf(buf+strlen(buf), _("Allows %s (with %s).\n"), 
-		advances[j].name, advances[advances[j].req[0]].name);
+		advance_name_translation(j),
+		advance_name_translation(advances[j].req[0]));
       }
     }
     if (strlen(buf)) strcat(buf, "\n");
-    if (advances[i].helptext) {
-      sprintf(buf+strlen(buf), "%s\n", _(advances[i].helptext));
-    }
-  }
-  else {
+  } else {
     create_help_page(HELP_TECH);
-    create_tech_tree(help_tech_tree, 0, game.num_tech_types, 3);
+    create_tech_tree(help_tech_tree, 0, game.control.num_tech_types, 3);
     strcpy(buf, pitem->text);
   }
   wordwrap_string(buf, 68);
@@ -934,136 +955,106 @@ static void help_update_tech(const struct help_item *pitem, char *title, int i)
 ...
 **************************************************************************/
 static void help_update_terrain(const struct help_item *pitem,
-				char *title, int i)
+				char *title, struct terrain *pterrain)
 {
-  char *buf = &long_buffer[0];
+  char buf[4096];
 
   create_help_page(HELP_TERRAIN);
   set_title_topic(pitem);
 
-  helptext_terrain(buf, i, pitem->text);
+  helptext_terrain(buf, sizeof(buf), pterrain, pitem->text);
   XtVaSetValues(help_text, XtNstring, buf, NULL);
 
-  if (i < T_COUNT)
-    {
-      sprintf (buf, "%d/%d.%d",
-	       tile_types[i].movement_cost,
-	       (int)(tile_types[i].defense_bonus/10), tile_types[i].defense_bonus%10);
-      xaw_set_label (help_terrain_movement_defense_data, buf);
+  if (pterrain) {
+    sprintf(buf, "%d/%d.%d",
+	    pterrain->movement_cost,
+	    (int)((pterrain->defense_bonus + 100) / 100),
+	    (pterrain->defense_bonus + 100) % 100 / 10);
+    xaw_set_label(help_terrain_movement_defense_data, buf);
 
-      sprintf (buf, "%d/%d/%d",
-	       tile_types[i].food,
-	       tile_types[i].shield,
-	       tile_types[i].trade);
-      xaw_set_label (help_terrain_food_shield_trade_data, buf);
+    sprintf(buf, "%d/%d/%d",
+	    pterrain->output[O_FOOD],
+	    pterrain->output[O_SHIELD],
+	    pterrain->output[O_TRADE]);
+    xaw_set_label(help_terrain_food_shield_trade_data, buf);
 
-      if (*(tile_types[i].special_1_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_1_name);
-	  xaw_set_label (help_terrain_special_1, buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_1,
-		   tile_types[i].shield_special_1,
-		   tile_types[i].trade_special_1);
-	  xaw_set_label (help_terrain_special_1_data, buf);
-	} else {
-	  xaw_set_label (help_terrain_special_1, "");
-	  xaw_set_label (help_terrain_special_1_data, "");
-	}
+    buf[0] = '\0';
+    if (*(pterrain->resources)) {
+      struct resource **r;
 
-      if (*(tile_types[i].special_2_name))
-	{
-	  sprintf (buf, _("%s F/R/T:"),
-		   tile_types[i].special_2_name);
-	  xaw_set_label (help_terrain_special_2, buf);
-	  sprintf (buf, "%d/%d/%d",
-		   tile_types[i].food_special_2,
-		   tile_types[i].shield_special_2,
-		   tile_types[i].trade_special_2);
-	  xaw_set_label (help_terrain_special_2_data, buf);
-	} else {
-	  xaw_set_label (help_terrain_special_2, "");
-	  xaw_set_label (help_terrain_special_2_data, "");
-	}
-
-      if (tile_types[i].road_trade_incr > 0)
-	{
-	  sprintf (buf, _("+%d Trade / %d"),
-		   tile_types[i].road_trade_incr,
-		   tile_types[i].road_time);
-	}
-      else if (tile_types[i].road_time > 0)
-	{
-	  sprintf (buf, _("no extra / %d"),
-		   tile_types[i].road_time);
-	}
-      else
-	{
-	  strcpy (buf, _("n/a"));
-	}
-      xaw_set_label (help_terrain_road_result_time_data, buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].irrigation_result == i)
-	{
-	  if (tile_types[i].irrigation_food_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Food / %d"),
-		       tile_types[i].irrigation_food_incr,
-		       tile_types[i].irrigation_time);
-	    }
-	}
-      else if (tile_types[i].irrigation_result != T_NONE)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].irrigation_result].terrain_name,
-		   tile_types[i].irrigation_time);
-	}
-      xaw_set_label (help_terrain_irrigation_result_time_data, buf);
-
-      strcpy (buf, _("n/a"));
-      if (tile_types[i].mining_result == i)
-	{
-	  if (tile_types[i].mining_shield_incr > 0)
-	    {
-	      sprintf (buf, _("+%d Res. / %d"),
-		       tile_types[i].mining_shield_incr,
-		       tile_types[i].mining_time);
-	    }
-	}
-      else if (tile_types[i].mining_result != T_NONE)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].mining_result].terrain_name,
-		   tile_types[i].mining_time);
-	}
-      xaw_set_label (help_terrain_mining_result_time_data, buf);
-
-      if (tile_types[i].transform_result != T_NONE)
-	{
-	  sprintf (buf, "%s / %d",
-		   tile_types[tile_types[i].transform_result].terrain_name,
-		   tile_types[i].transform_time);
-	} else {
-	  strcpy (buf, _("n/a"));
-	}
-      xaw_set_label (help_terrain_transform_result_time_data, buf);
+      for (r = pterrain->resources; *r; r++) {
+	sprintf (buf + strlen (buf), " %s,", resource_name_translation(*r));
+      }
+      buf[strlen (buf) - 1] = '.';
+    } else {
+      /* TRANS: "Resources: (none)". */
+      sprintf (buf + strlen (buf), _("(none)"));
     }
+    xaw_set_label(help_terrain_resources, buf);
+
+    if (pterrain->road_trade_incr > 0) {
+      sprintf(buf, _("+%d Trade / %d"),
+	      pterrain->road_trade_incr,
+	      pterrain->road_time);
+    } else if (pterrain->road_time > 0) {
+      sprintf(buf, _("no extra / %d"), pterrain->road_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    xaw_set_label(help_terrain_road_result_time_data, buf);
+
+    strcpy(buf, _("n/a"));
+    if (pterrain->irrigation_result == pterrain) {
+      if (pterrain->irrigation_food_incr > 0) {
+	sprintf(buf, _("+%d Food / %d"),
+		pterrain->irrigation_food_incr,
+		pterrain->irrigation_time);
+      }
+    } else if (pterrain->irrigation_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->irrigation_result),
+	      pterrain->irrigation_time);
+    }
+    xaw_set_label(help_terrain_irrigation_result_time_data, buf);
+
+    strcpy(buf, _("n/a"));
+    if (pterrain->mining_result == pterrain) {
+      if (pterrain->mining_shield_incr > 0) {
+	sprintf(buf, _("+%d Res. / %d"),
+		pterrain->mining_shield_incr,
+		pterrain->mining_time);
+      }
+    } else if (pterrain->mining_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->mining_result),
+	      pterrain->mining_time);
+    }
+    xaw_set_label (help_terrain_mining_result_time_data, buf);
+
+    if (pterrain->transform_result != T_NONE) {
+      sprintf(buf, "%s / %d",
+	      terrain_name_translation(pterrain->transform_result),
+	      pterrain->transform_time);
+    } else {
+      strcpy(buf, _("n/a"));
+    }
+    xaw_set_label (help_terrain_transform_result_time_data, buf);
+  }
 }
 
 /**************************************************************************
   This is currently just a text page, with special text:
 **************************************************************************/
 static void help_update_government(const struct help_item *pitem,
-				   char *title, struct government *gov)
+				   char *title,
+				   struct government *pgovernment)
 {
-  char *buf = &long_buffer[0];
+  char buf[4096];
 
-  if (gov==NULL) {
+  if (!pgovernment) {
     strcat(buf, pitem->text);
   } else {
-    helptext_government(buf, gov-governments, pitem->text);
+    helptext_government(buf, sizeof(buf), pgovernment, pitem->text);
   }
   create_help_page(HELP_TEXT);
   set_title_topic(pitem);
@@ -1086,26 +1077,26 @@ static void help_update_dialog(const struct help_item *pitem)
 
   switch(pitem->type) {
   case HELP_IMPROVEMENT:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && is_wonder(i)) i = B_LAST;
+    i = find_improvement_by_translated_name(top);
+    if(i!=B_LAST && is_great_wonder(i)) i = B_LAST;
     help_update_improvement(pitem, top, i);
     break;
   case HELP_WONDER:
-    i = find_improvement_by_name(top);
-    if(i!=B_LAST && !is_wonder(i)) i = B_LAST;
+    i = find_improvement_by_translated_name(top);
+    if(i!=B_LAST && !is_great_wonder(i)) i = B_LAST;
     help_update_wonder(pitem, top, i);
     break;
   case HELP_UNIT:
-    help_update_unit_type(pitem, top, find_unit_type_by_name(top));
+    help_update_unit_type(pitem, top, find_unit_type_by_translated_name(top));
     break;
   case HELP_TECH:
-    help_update_tech(pitem, top, find_tech_by_name(top));
+    help_update_tech(pitem, top, find_advance_by_translated_name(top));
     break;
   case HELP_TERRAIN:
-    help_update_terrain(pitem, top, get_terrain_by_name(top));
+    help_update_terrain(pitem, top, find_terrain_by_translated_name(top));
     break;
   case HELP_GOVERNMENT:
-    help_update_government(pitem, top, find_government_by_name(top));
+    help_update_government(pitem, top, find_government_by_translated_name(top));
     break;
   case HELP_TEXT:
   default:
