@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -51,8 +52,9 @@ static Widget meswin_viewport;
 static Widget meswin_close_command;
 static Widget meswin_goto_command;
 static Widget meswin_popcity_command;
+static bool meswin_dialog_shell_is_raised;
 
-static void create_meswin_dialog(void);
+static void create_meswin_dialog(bool raise);
 static void meswin_scroll_down(void);
 static void meswin_close_callback(Widget w, XtPointer client_data,
 				  XtPointer call_data);
@@ -71,15 +73,21 @@ static char *dummy_message_list[] = {
 /****************************************************************
 popup the dialog 10% inside the main-window 
 *****************************************************************/
-void popup_meswin_dialog(void)
+void popup_meswin_dialog(bool raise)
 {
   int updated = 0;
   
+  meswin_dialog_shell_is_raised = raise;
+
   if(!meswin_dialog_shell) {
-    create_meswin_dialog();
+    create_meswin_dialog(raise);
     updated = 1;		/* create_ calls update_ */
   }
 
+  if (raise) {
+    XtSetSensitive(main_form, FALSE);
+  }
+
   xaw_set_relative_position(toplevel, meswin_dialog_shell, 25, 25);
   XtPopup(meswin_dialog_shell, XtGrabNone);
   if(!updated) 
@@ -95,6 +103,20 @@ void popup_meswin_dialog(void)
   meswin_scroll_down();
 }
 
+/****************************************************************
+  Closes the message window dialog.
+*****************************************************************/
+void popdown_meswin_dialog(void)
+{
+  if (meswin_dialog_shell) {
+    if (meswin_dialog_shell_is_raised) {
+      XtSetSensitive(main_form, TRUE);
+    }
+    XtDestroyWidget(meswin_dialog_shell);
+    meswin_dialog_shell = 0;
+  }
+}
+
 /****************************************************************
 ...
 *****************************************************************/
@@ -110,12 +132,14 @@ static bool creating = FALSE;
 /****************************************************************
 ...
 *****************************************************************/
-static void create_meswin_dialog(void)
+static void create_meswin_dialog(bool raise)
 {
   creating = TRUE;
 
   meswin_dialog_shell =
-    I_IN(I_T(XtCreatePopupShell("meswinpopup", topLevelShellWidgetClass,
+    I_IN(I_T(XtCreatePopupShell("meswinpopup",
+				raise ? transientShellWidgetClass
+				: topLevelShellWidgetClass,
 				toplevel, NULL, 0)));
 
   meswin_form = XtVaCreateManagedWidget("meswinform", 
@@ -209,17 +233,18 @@ static void meswin_scroll_down(void)
 **************************************************************************/
 void real_update_meswin_dialog(void)
 {
-  Dimension height, iheight, width;
-  int i;
+  Dimension height, iheight, width, oldheight, newheight;
+  int i, num = get_num_messages();
 
   XawFormDoLayout(meswin_form, False);
 
-  if (get_num_messages() == 0) {
+  XtVaGetValues(meswin_viewport, XtNheight, &oldheight, NULL);
+
+  if (num == 0) {
     XawListChange(meswin_list, dummy_message_list, 1, 0, True);
   } else {
     /* strings will not be freed */
     static char **strings = NULL;
-    int i, num = get_num_messages();
 
     strings = fc_realloc(strings, num * sizeof(char *));
 
@@ -227,7 +252,7 @@ void real_update_meswin_dialog(void)
       strings[i] = get_message(i)->descr;
     }
 
-    XawListChange(meswin_list, strings, get_num_messages(), 0, True);
+    XawListChange(meswin_list, strings, num, 0, True);
   }
 
   /* Much of the following copied from city_report_dialog_update() */
@@ -241,19 +266,33 @@ void real_update_meswin_dialog(void)
   /* Seems have to do this here so we get the correct height below. */
   XawFormDoLayout(meswin_form, True);
 
-  if (get_num_messages() <= N_MSG_VIEW) {
+  if (num <= N_MSG_VIEW) {
     XtVaGetValues(meswin_list, XtNheight, &height, NULL);
-    XtVaSetValues(meswin_viewport, XtNheight, height, NULL);
+    if ((oldheight == 0) || (num == 0)) {
+      XtVaSetValues(meswin_viewport, XtNheight, height, NULL);
+    } else {
+      XtVaGetValues(meswin_form, XtNheight, &newheight, NULL);
+      newheight = newheight + height - oldheight;
+      XtVaSetValues(meswin_form, XtNheight, newheight, NULL);
+    }
   } else {
     XtVaGetValues(meswin_list, XtNheight, &height, NULL);
     XtVaGetValues(meswin_list, XtNinternalHeight, &iheight, NULL);
     height -= (iheight * 2);
-    height /= get_num_messages();
+    height /= num;
     height *= N_MSG_VIEW;
     height += (iheight * 2);
-    XtVaSetValues(meswin_viewport, XtNheight, height, NULL);
+    if (height != oldheight) {
+      if (oldheight == 0) {
+	XtVaSetValues(meswin_viewport, XtNheight, height, NULL);
+      } else {
+	XtVaGetValues(meswin_form, XtNheight, &newheight, NULL);
+	newheight = newheight + height - oldheight;
+	XtVaSetValues(meswin_form, XtNheight, newheight, NULL);
+      }
+    }
+    meswin_scroll_down();
   }
-  meswin_scroll_down();
 
   XtSetSensitive(meswin_goto_command, FALSE);
   XtSetSensitive(meswin_popcity_command, FALSE);
@@ -267,7 +306,7 @@ static void meswin_list_callback(Widget w, XtPointer client_data,
 {
   XawListReturnStruct *ret = XawListShowCurrent(meswin_list);
 
-  if (ret->list_index != XAW_LIST_NONE) {
+  if ((ret->list_index != XAW_LIST_NONE) && (get_num_messages() != 0)) {
     struct message *message = get_message(ret->list_index);
 
     XtSetSensitive(meswin_goto_command, message->location_ok ? True : False);
@@ -284,8 +323,7 @@ static void meswin_list_callback(Widget w, XtPointer client_data,
 static void meswin_close_callback(Widget w, XtPointer client_data,
 				  XtPointer call_data)
 {
-  XtDestroyWidget(meswin_dialog_shell);
-  meswin_dialog_shell=0;
+  popdown_meswin_dialog();
 }
 
 /****************************************************************
