@@ -10,14 +10,13 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <ctype.h>
-#include <stdarg.h>
 
 #include <X11/Intrinsic.h>
 #include <X11/StringDefs.h>
@@ -36,7 +35,9 @@
 #include "shared.h"
 #include "support.h"
 
+#include "civclient.h"
 #include "clinet.h"
+
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "mapview.h"
@@ -80,6 +81,10 @@ void popup_rates_dialog(void)
   Dimension width, height;
   char buf[64];
 
+  if (!can_client_issue_orders()) {
+    return;
+  }
+
   XtSetSensitive(main_form, FALSE);
 
   create_rates_dialog();
@@ -91,8 +96,8 @@ void popup_rates_dialog(void)
   XtVaSetValues(rates_dialog_shell, XtNx, x, XtNy, y, NULL);
 
   my_snprintf(buf, sizeof(buf), _("%s max rate: %d%%"),
-	  get_government_name(game.player_ptr->government),
-	  get_government_max_rate(game.player_ptr->government));
+	      government_name_for_player(game.player_ptr),
+	      get_player_bonus(game.player_ptr, EFT_MAX_RATES));
   xaw_set_label(rates_gov_label, buf);
   
   XtPopup(rates_dialog_shell, XtGrabNone);
@@ -107,6 +112,9 @@ void create_rates_dialog(void)
   Widget rates_form;
   Widget rates_ok_command, rates_cancel_command;
   
+  if (!can_client_issue_orders()) {
+    return;
+  }
 
   rates_dialog_shell =
     I_T(XtCreatePopupShell("ratespopup", transientShellWidgetClass,
@@ -219,15 +227,11 @@ void create_rates_dialog(void)
 void rates_ok_command_callback(Widget w, XtPointer client_data, 
 			       XtPointer call_data)
 {
-  struct packet_player_request packet;
-  
   XtSetSensitive(main_form, TRUE);
   XtDestroyWidget(rates_dialog_shell);
 
-  packet.tax=rates_tax_value;
-  packet.science=rates_sci_value;
-  packet.luxury=rates_lux_value;
-  send_packet_player_request(&aconnection, &packet, PACKET_PLAYER_RATES);
+  dsend_packet_player_rates(&aconnection, rates_tax_value, rates_lux_value,
+			    rates_sci_value);
 }
 
 /**************************************************************************
@@ -255,11 +259,15 @@ void rates_set_values(int tax, int no_tax_scroll,
   Boolean tax_lock, lux_lock, sci_lock;
   int maxrate;
   
+  if (!can_client_issue_orders()) {
+    return;
+  }
+
   XtVaGetValues(rates_tax_toggle, XtNstate, &tax_lock, NULL);
   XtVaGetValues(rates_lux_toggle, XtNstate, &lux_lock, NULL);
   XtVaGetValues(rates_sci_toggle, XtNstate, &sci_lock, NULL);
   
-  maxrate=get_government_max_rate(game.player_ptr->government);
+  maxrate = get_player_bonus(game.player_ptr, EFT_MAX_RATES);
   /* This's quite a simple-minded "double check".. */
   tax=MIN(tax, maxrate);
   lux=MIN(lux, maxrate);
@@ -362,8 +370,7 @@ void rates_scroll_jump_callback(Widget w, XtPointer client_data,
 void rates_scroll_scroll_callback(Widget w, XtPointer client_data,
 				  XtPointer position_val)
 {
-  int pos=(int)position_val;
-  int val;
+  int val, pos = XTPOINTER_TO_INT(position_val);
   
   if(w==rates_tax_scroll) {
     if(pos<0)
@@ -387,4 +394,3 @@ void rates_scroll_scroll_callback(Widget w, XtPointer client_data,
     rates_set_values(rates_tax_value,0, rates_lux_value,0, val,0);
   }
 }
-
