@@ -10,6 +10,7 @@
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 ***********************************************************************/
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -19,33 +20,35 @@
 #include <string.h>
 
 #include <X11/Intrinsic.h>
+#include <X11/IntrinsicP.h>
 #include <X11/StringDefs.h>
+#include <X11/Xaw/AsciiText.h>  
+#include <X11/Xaw/Command.h>
 #include <X11/Xaw/Form.h>
 #include <X11/Xaw/Label.h>
-#include <X11/Xaw/Command.h>
+#include <X11/Xaw/List.h>
 #include <X11/Xaw/MenuButton.h>
 #include <X11/Xaw/SimpleMenu.h>
 #include <X11/Xaw/SmeBSB.h>
-#include <X11/Xaw/Command.h>
-#include <X11/Xaw/List.h>
-#include <X11/Xaw/Viewport.h>
-#include <X11/Xaw/AsciiText.h>  
 #include <X11/Xaw/Toggle.h>     
-#include <X11/IntrinsicP.h>
+#include <X11/Xaw/Viewport.h>
 
+#include "fcintl.h"
+#include "mem.h"
 
 #include "city.h"
-#include "fcintl.h"
 #include "game.h"
+#include "packets.h"
+#include "worklist.h"
+
+#include "civclient.h"
+#include "climisc.h"
+#include "clinet.h"
+
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "helpdlg.h"
 #include "inputdlg.h"
-#include "mem.h"
-#include "packets.h"
-#include "worklist.h"
-#include "climisc.h"
-#include "clinet.h"
 
 #include "wldlg.h"
 
@@ -103,7 +106,8 @@ static struct worklist_report_dialog *report_dialog;
 
 static int uni_id(struct worklist *pwl, int wlinx);
 
-static void worklist_id_to_name(char buf[], int id, bool is_unit, 
+static void worklist_id_to_name(char buf[],
+				struct city_production production,
 				struct city *pcity);
 
 static void rename_worklist_callback(Widget w, XtPointer client_data, 
@@ -153,7 +157,7 @@ static void worklist_avail_help_callback(Widget w, XtPointer client_data,
 					  XtPointer call_data);
 static void worklist_show_advanced_callback(Widget w, XtPointer client_data, 
 					  XtPointer call_data);
-static void worklist_help(int id, bool is_unit);
+static void worklist_help(struct city_production production);
 static void worklist_populate_worklist(struct worklist_dialog *pdialog);
 static void worklist_populate_targets(struct worklist_dialog *pdialog);
 
@@ -561,36 +565,38 @@ void update_worklist_report_dialog(void)
 }
 
 /****************************************************************
-
+  Returns an unique id for element in units + buildings sequence
 *****************************************************************/
 int uni_id(struct worklist *pwl, int inx)
 {
-  if ((inx < 0) || (inx >= MAX_LEN_WORKLIST)) {
+  if ((inx < 0) || (inx >= worklist_length(pwl))) {
     return WORKLIST_END;
-  } else if (pwl->wlefs[inx] == WEF_UNIT) {
-    return pwl->wlids[inx] + B_LAST;
-  } else if (pwl->wlefs[inx] == WEF_IMPR) {
-    return pwl->wlids[inx];
+  } else if (pwl->entries[inx].is_unit) {
+    return pwl->entries[inx].value + B_LAST;
   } else {
-    return WORKLIST_END;
+    return pwl->entries[inx].value;
   }
 }
 
 /****************************************************************
 
 *****************************************************************/
-void worklist_id_to_name(char buf[], int id, bool is_unit, 
+void worklist_id_to_name(char buf[],
+			 struct city_production production,
 			 struct city *pcity)
 {
-  if (is_unit)
+  if (production.is_unit)
     sprintf(buf, "%s (%d)",
-	    get_unit_name(id), get_unit_type(id)->build_cost);
+	    utype_values_translation(utype_by_number(production.value)),
+	    unit_build_shield_cost(utype_by_number(production.value)));
   else if (pcity)
     sprintf(buf, "%s (%d)",
-	    get_impr_name_ex(pcity, id), get_improvement_type(id)->build_cost);
+	    get_impr_name_ex(pcity, production.value),
+	    impr_build_shield_cost(production.value));
   else
     sprintf(buf, "%s (%d)",
-	    get_improvement_name(id), get_improvement_type(id)->build_cost);
+	    improvement_name_translation(production.value),
+	    impr_build_shield_cost(production.value));
 }
 
 
@@ -828,20 +834,23 @@ void insert_into_worklist(struct worklist_dialog *pdialog,
 			  int before, cid cid)
 {
   int i, first_free;
-  int target = cid_id(cid);
-  bool is_unit = cid_is_unit(cid);
+  struct city_production target = cid_decode(cid);
 
   /* If this worklist is a city worklist, double check that the city
      really can (eventually) build the target.  We've made sure that
      the list of available targets is okay for this city, but a global
      worklist may try to insert an odd-ball unit or target. */
-  if (pdialog->pcity &&
-      ((is_unit  && !can_eventually_build_unit(pdialog->pcity, target)) ||
-       (!is_unit && !can_eventually_build_improvement(pdialog->pcity, target))))
+  if (pdialog->pcity
+      && ((target.is_unit
+	   && !can_eventually_build_unit(pdialog->pcity,
+					 utype_by_number(target.value)))
+	  || (!target.is_unit
+	      && !can_eventually_build_improvement(pdialog->pcity,
+						   target.value)))) {
     /* Nope, this city can't build this target, ever.  Don't put it into
        the worklist. */
     return;
-    
+  }
 
   /* Find the first free element in the worklist */
   for (first_free = 0; first_free < MAX_LEN_WORKLIST; first_free++)
@@ -876,7 +885,7 @@ void insert_into_worklist(struct worklist_dialog *pdialog,
   pdialog->worklist_ids[before] = cid;
   
   worklist_id_to_name(pdialog->worklist_names[before],
-		      target, is_unit, pdialog->pcity);
+		      target, pdialog->pcity);
   pdialog->worklist_names_ptrs[before] = pdialog->worklist_names[before];
 }
 
@@ -927,6 +936,7 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
 {
   int target;
   int i, len;
+  struct city_production production;
 
   /* Is there anything selected to insert? */
   if (retAvail->list_index == XAW_LIST_NONE)
@@ -948,12 +958,16 @@ void worklist_insert_common_callback(struct worklist_dialog *pdialog,
     }
   } else if (retAvail->list_index >= 
 	     pdialog->worklist_avail_num_improvements) {
-    /* target is an improvement or wonder */
-    insert_into_worklist(pdialog, where, cid_encode(TRUE, target));
+    /* target is a unit */
+    production.is_unit = true;
+    production.value = target;
+    insert_into_worklist(pdialog, where, cid_encode(production));
     where++;
   } else {
-    /* target is a unit */
-    insert_into_worklist(pdialog, where, cid_encode(FALSE, target));
+    /* target is an improvement or wonder */
+    production.is_unit = false;
+    production.value = target;
+    insert_into_worklist(pdialog, where, cid_encode(production));
     where++;
   }
 
@@ -1086,6 +1100,7 @@ void worklist_ok_callback(Widget w, XtPointer client_data, XtPointer call_data)
 {
   struct worklist_dialog *pdialog;
   struct worklist wl;
+  struct city_production production;
   int i;
   
   pdialog=(struct worklist_dialog *)client_data;
@@ -1096,17 +1111,17 @@ void worklist_ok_callback(Widget w, XtPointer client_data, XtPointer call_data)
   
   for (i = 0; i < MAX_LEN_WORKLIST; i++) {
     if (pdialog->worklist_ids[i] == WORKLIST_END) {
-      wl.wlefs[i] = WEF_END;
-      wl.wlids[i] = 0;
+      continue;
     } else if (pdialog->worklist_ids[i] >= B_LAST) {
-      wl.wlefs[i] = WEF_UNIT;
-      wl.wlids[i] = pdialog->worklist_ids[i] - B_LAST;
+      production.is_unit = true;
+      production.value = pdialog->worklist_ids[i] - B_LAST;
+      worklist_append(&wl, production);
     } else if (pdialog->worklist_ids[i] >= 0) {
-      wl.wlefs[i] = WEF_IMPR;
-      wl.wlids[i] = pdialog->worklist_ids[i];
+      production.is_unit = false;
+      production.value = pdialog->worklist_ids[i];
+      worklist_append(&wl, production);
     } else {
-      wl.wlefs[i] = WEF_END;
-      wl.wlids[i] = 0;
+      continue;
     }
   }
   strcpy(wl.name, pdialog->pwl->name);
@@ -1151,21 +1166,19 @@ void worklist_worklist_help_callback(Widget w, XtPointer client_data,
 {
   struct worklist_dialog *pdialog;
   XawListReturnStruct *ret;
-  int id;
-  bool is_unit = FALSE;
+  struct city_production production;
 
   pdialog=(struct worklist_dialog *)client_data;
 
   ret = XawListShowCurrent(pdialog->worklist);
   if(ret->list_index!=XAW_LIST_NONE) {
     cid cid = pdialog->worklist_ids[ret->list_index];
-    is_unit = cid_is_unit(cid);
-    id = cid_id(cid);
+    production = cid_decode(cid);
   } else {
-    id = -1;
+    production.value = -1;
   }
 
-  worklist_help(id, is_unit);
+  worklist_help(production);
 }
 
 void worklist_avail_help_callback(Widget w, XtPointer client_data, 
@@ -1173,8 +1186,7 @@ void worklist_avail_help_callback(Widget w, XtPointer client_data,
 {
   struct worklist_dialog *pdialog;
   XawListReturnStruct *ret;
-  int id;
-  bool is_unit = FALSE;
+  struct city_production production;
 
   pdialog=(struct worklist_dialog *)client_data;
 
@@ -1182,28 +1194,32 @@ void worklist_avail_help_callback(Widget w, XtPointer client_data,
   if(ret->list_index!=XAW_LIST_NONE) {
     if (ret->list_index >= pdialog->worklist_avail_num_targets) {
       /* target is a global worklist id */
-      id = -1;
+      production.value = -1;
     } else {
-      id = pdialog->worklist_avail_ids[ret->list_index];
-      is_unit = ret->list_index >= pdialog->worklist_avail_num_improvements;
+      production.value = pdialog->worklist_avail_ids[ret->list_index];
+      production.is_unit =
+        ret->list_index >= pdialog->worklist_avail_num_improvements;
     }
   } else {
-    id = -1;
+    production.value = -1;
   }
 
-  worklist_help(id, is_unit);
+  worklist_help(production);
 }
 
 
-void worklist_help(int id, bool is_unit)
+void worklist_help(struct city_production production)
 {
-  if(id >= 0) {
-    if (is_unit) {
-      popup_help_dialog_typed(get_unit_type(id)->name, HELP_UNIT);
-    } else if(is_wonder(id)) {
-      popup_help_dialog_typed(get_improvement_name(id), HELP_WONDER);
+  if (production.value >= 0) {
+    if (production.is_unit) {
+      popup_help_dialog_typed(utype_name_translation(utype_by_number(production.value)),
+					    HELP_UNIT);
+    } else if (is_great_wonder(production.value)) {
+      popup_help_dialog_typed(improvement_name_translation(production.value),
+						   HELP_WONDER);
     } else {
-      popup_help_dialog_typed(get_improvement_name(id), HELP_IMPROVEMENT);
+      popup_help_dialog_typed(improvement_name_translation(production.value),
+						   HELP_IMPROVEMENT);
     }
   }
   else
@@ -1240,18 +1256,17 @@ void worklist_populate_worklist(struct worklist_dialog *pdialog)
 {
   int i, n;
   int id;
-  int target;
-  bool is_unit;
+  struct city_production target;
 
   n = 0;
   if (pdialog->pcity) {
     /* First element is the current build target of the city. */
-    id = pdialog->pcity->currently_building;
+    id = pdialog->pcity->production.value;
 
     worklist_id_to_name(pdialog->worklist_names[n],
-			id, pdialog->pcity->is_building_unit, pdialog->pcity);
+			pdialog->pcity->production, pdialog->pcity);
 
-    if (pdialog->pcity->is_building_unit)
+    if (pdialog->pcity->production.is_unit)
       id += B_LAST;
     pdialog->worklist_names_ptrs[n] = pdialog->worklist_names[n];
     pdialog->worklist_ids[n] = id;
@@ -1261,11 +1276,11 @@ void worklist_populate_worklist(struct worklist_dialog *pdialog)
   /* Fill in the rest of the worklist list */
   for (i = 0; n < MAX_LEN_WORKLIST &&
 	 uni_id(pdialog->pwl, i) != WORKLIST_END; i++, n++) {
-    worklist_peek_ith(pdialog->pwl, &target, &is_unit, i);
+    worklist_peek_ith(pdialog->pwl, &target, i);
     id = uni_id(pdialog->pwl, i);
 
     worklist_id_to_name(pdialog->worklist_names[n],
-			target, is_unit, pdialog->pcity);
+			target, pdialog->pcity);
 
     pdialog->worklist_names_ptrs[n] = pdialog->worklist_names[n];
     pdialog->worklist_ids[n] = id;
@@ -1287,7 +1302,11 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
   Boolean b;
   int advanced_tech;
   int can_build, can_eventually_build;
-  
+  struct city_production production;
+
+  if (!can_client_issue_orders()) {
+    return;
+  }
 
   n = 0;
 
@@ -1303,7 +1322,7 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
   impr_type_iterate(i) {
     /* Can the player (eventually) build this improvement? */
     can_build = can_player_build_improvement(pplr,i);
-    can_eventually_build = could_player_eventually_build_improvement(pplr,i);
+    can_eventually_build = can_player_eventually_build_improvement(pplr,i);
 
     /* If there's a city, can the city build the improvement? */
     if (pdialog->pcity) {
@@ -1314,8 +1333,10 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
     
     if (( advanced_tech && can_eventually_build) ||
 	(!advanced_tech && can_build)) {
+      production.is_unit = false;
+      production.value = i;
       worklist_id_to_name(pdialog->worklist_avail_names[n],
-			  i, 0, pdialog->pcity);
+			  production, pdialog->pcity);
       pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
       pdialog->worklist_avail_ids[n++]=i;
     }
@@ -1337,10 +1358,12 @@ void worklist_populate_targets(struct worklist_dialog *pdialog)
 
     if (( advanced_tech && can_eventually_build) ||
 	(!advanced_tech && can_build)) {
+      production.is_unit = true;
+      production.value = i->index;
       worklist_id_to_name(pdialog->worklist_avail_names[n],
-			  i, 1, pdialog->pcity);
+			  production, pdialog->pcity);
       pdialog->worklist_avail_names_ptrs[n]=pdialog->worklist_avail_names[n];
-      pdialog->worklist_avail_ids[n++]=i;
+      pdialog->worklist_avail_ids[n++]=i->index;
     }
   } unit_type_iterate_end;
 
