@@ -15,17 +15,15 @@
 
 #include "shared.h"		/* bool type */
 
+#include "fc_types.h"
+
 #include "mapctrl_common.h"
 
-struct unit;
-struct city;
-
-extern struct city *city_workers_display;
-
 void popup_newcity_dialog(struct unit *punit, char *suggestname);
 
 void set_turn_done_button_state(bool state);
 
 void create_line_at_mouse_pos(void);
+void update_rect_at_mouse_pos(void);
 
 #endif  /* FC__MAPCTRL_G_H */
