@@ -23,12 +23,12 @@ enum client_pages {
   PAGE_LOAD,		/* Load saved game page. */
   PAGE_NETWORK,		/* Connect to network page.  */
   PAGE_NATION,		/* Select a nation page.  */
-  PAGE_GAME		/* In game page. */
+  PAGE_GAME,		/* In game page. */
+  PAGE_GGZ		/* In game page.  This one must be last. */
 };
 
-/**************************************************************************
-  Does a toplevel window page transition.
-**************************************************************************/
 void set_client_page(enum client_pages page);
+enum client_pages get_client_page(void);
+void update_start_page(void);
 
 #endif  /* FC__PAGES_G_H */
