@@ -448,22 +448,21 @@ void handle_city_info(struct packet_city_info *packet)
   } else {
     name_changed = (strcmp(city_name(pcity), packet->name) != 0);
 
-    /* Check if city desciptions should be updated */
-    if (draw_city_names && name_changed) {
-      update_descriptions = TRUE;
-    } else if (DRAW_CITY_PRODUCTIONS
-	       && (pcity->production.is_unit != packet->production_is_unit
-		   || pcity->production.value != packet->production_value
-		   || pcity->surplus[O_SHIELD] != packet->surplus[O_SHIELD]
-		   || pcity->shield_stock != packet->shield_stock)) {
-      update_descriptions = TRUE;
-    } else if (draw_city_names && DRAW_CITY_GROWTH &&
-	       (pcity->food_stock != packet->food_stock ||
-		pcity->surplus[O_FOOD] != packet->surplus[O_FOOD])) {
-      /* If either the food stock or surplus have changed, the time-to-grow
-	 is likely to have changed as well. */
-      update_descriptions = TRUE;
-    }
+    /* Descriptions should probably be updated if the
+     * city name, production or time-to-grow changes.
+     * Note that if either the food stock or surplus
+     * have changed, the time-to-grow is likely to
+     * have changed as well. */
+    update_descriptions = (draw_city_names && name_changed)
+      || (draw_city_productions
+          && (pcity->production.is_unit != packet->production_is_unit
+              || pcity->production.value != packet->production_value
+              || pcity->surplus[O_SHIELD] != packet->surplus[O_SHIELD]
+              || pcity->shield_stock != packet->shield_stock))
+      || (draw_city_growth
+          && (pcity->food_stock != packet->food_stock
+              || pcity->surplus[O_FOOD] != packet->surplus[O_FOOD]));
+
     assert(pcity->id == packet->id);
   }
   
@@ -1531,6 +1530,11 @@ void handle_player_info(struct packet_player_info *pinfo)
   pplayer->is_male=pinfo->is_male;
   team_add_player(pplayer, team_get_by_id(pinfo->team));
   pplayer->score.game = pinfo->score;
+  if (has_capability("CreatedName", aconnection.capability)) {
+    pplayer->was_created = pinfo->was_created;
+  } else {
+    pplayer->was_created = FALSE;
+  }
 
   pplayer->economic.gold=pinfo->gold;
   pplayer->economic.tax=pinfo->tax;
@@ -2459,6 +2463,9 @@ void handle_ruleset_terrain(struct packet_ruleset_terrain *p)
     pterrain->output[o] = p->output[o];
   } output_type_iterate_end;
 
+  if (pterrain->resources != NULL) {
+    free(pterrain->resources);
+  }
   pterrain->resources = fc_calloc(p->num_resources + 1,
 				  sizeof(*pterrain->resources));
   for (j = 0; j < p->num_resources; j++) {
@@ -2488,9 +2495,12 @@ void handle_ruleset_terrain(struct packet_ruleset_terrain *p)
   pterrain->fortress_time = p->fortress_time;
   pterrain->clean_pollution_time = p->clean_pollution_time;
   pterrain->clean_fallout_time = p->clean_fallout_time;
-  
+
   pterrain->flags = p->flags;
 
+  if (pterrain->helptext != NULL) {
+    free(pterrain->helptext);
+  }
   pterrain->helptext = mystrdup(p->helptext);
   
   tileset_setup_tile_type(tileset, pterrain);
