@@ -171,10 +171,10 @@ static inline void tile_type_vector_add(struct tile_type_vector *tthis,
  * vector is empty.  We can never run out of specialists.
        */
 struct cm_tile_type {
-  int production[NUM_STATS];
+  int production[O_MAX];
   double estimated_fitness; /* weighted sum of production */
   bool is_specialist;
-  enum specialist_type spec; /* valid only if is_specialist */
+  Specialist_type_id spec; /* valid only if is_specialist */
   struct tile_vector tiles;  /* valid only if !is_specialist */
   struct tile_type_vector better_types;
   struct tile_type_vector worse_types;
@@ -193,7 +193,7 @@ struct partial_solution {
   int *worker_counts;   /* number of workers on each type */
   int *prereqs_filled;  /* number of better types filled up */
 
-  int production[NUM_STATS]; /* raw production, cached for the heuristic */
+  int production[O_MAX]; /* raw production, cached for the heuristic */
   int idle;             /* number of idle workers */
 };
 
@@ -209,7 +209,7 @@ struct cm_state {
 
   /* the tile lattice */
   struct tile_type_vector lattice;
-  struct tile_type_vector lattice_by_prod[NUM_STATS];
+  struct tile_type_vector lattice_by_prod[O_MAX];
 
   /* the best known solution, and its fitness */
   struct partial_solution best;
@@ -219,7 +219,7 @@ struct cm_state {
    * this fails to satisfy the constraints, so we can stop investigating
    * this branch.  A solution with more production than this may still
    * fail (for being unhappy, for instance). */
-  int min_production[NUM_STATS];
+  int min_production[O_MAX];
 
   /* the current solution we're examining. */
   struct partial_solution current;
@@ -383,13 +383,11 @@ static void tile_type_vector_free_all(struct tile_type_vector *vec)
 static bool tile_type_equal(const struct cm_tile_type *a,
 			    const struct cm_tile_type *b)
 {
-  enum cm_stat stat;
-
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     if (a->production[stat] != b->production[stat])  {
       return FALSE;
     }
-  }
+  } output_type_iterate_end;
 
   if (a->is_specialist != b->is_specialist) {
     return FALSE;
@@ -407,13 +405,11 @@ static bool tile_type_equal(const struct cm_tile_type *a,
 static bool tile_type_better(const struct cm_tile_type *a,
 			     const struct cm_tile_type *b)
 {
-  enum cm_stat stat;
-
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     if (a->production[stat] < b->production[stat])  {
       return FALSE;
     }
-  }
+  } output_type_iterate_end;
 
   if (a->is_specialist && !b->is_specialist) {
     /* If A is a specialist and B isn't, and all of A's production is at
@@ -469,7 +465,7 @@ static int tile_type_num_tiles(const struct cm_tile_type *type)
   Note this isn't the same as the number of *tiles* that are better.  There
   may be more than one tile of each type (see tile_type_num_tiles).
 ****************************************************************************/
-static const int tile_type_num_prereqs(const struct cm_tile_type *ptype)
+static int tile_type_num_prereqs(const struct cm_tile_type *ptype)
 {
   return ptype->better_types.size;
 }
@@ -496,6 +492,7 @@ static const struct cm_tile_type *tile_type_get(const struct cm_state *state,
 ****************************************************************************/
 static const struct cm_tile *tile_get(const struct cm_tile_type *ptype, int j)
 {
+  assert(!ptype->is_specialist);
   assert(j >= 0);
   assert(j < ptype->tiles.size);
   return &ptype->tiles.p[j];
@@ -534,22 +531,21 @@ static struct cm_fitness worst_fitness(void)
   Compute the fitness of the given surplus (and disorder/happy status)
   according to the weights and minimums given in the parameter.
 ****************************************************************************/
-static struct cm_fitness compute_fitness(const int surplus[NUM_STATS],
+static struct cm_fitness compute_fitness(const int surplus[],
 					 bool disorder, bool happy,
 					const struct cm_parameter *parameter)
 {
-  enum cm_stat stat;
   struct cm_fitness fitness;
 
   fitness.sufficient = TRUE;
   fitness.weighted = 0;
 
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     fitness.weighted += surplus[stat] * parameter->factor[stat];
     if (surplus[stat] < parameter->minimal_surplus[stat]) {
       fitness.sufficient = FALSE;
     }
-  }
+  } output_type_iterate_end;
 
   if (happy) {
     fitness.weighted += parameter->happy_factor;
@@ -637,7 +633,7 @@ static void apply_solution(struct cm_state *state,
    * the city center). */
   memset(&pcity->specialists, 0, sizeof(pcity->specialists));
   city_map_iterate(x, y) {
-    if (is_city_center(x, y)) {
+    if (is_free_worked_tile(x, y)) {
       continue;
     }
     if (pcity->city_map[x][y] == C_TILE_WORKER) {
@@ -686,15 +682,12 @@ static void apply_solution(struct cm_state *state,
   values based on the city's data.
 ****************************************************************************/
 static void get_city_surplus(const struct city *pcity,
-			     int surplus[NUM_STATS],
+			     int surplus[],
 			     bool *disorder, bool *happy)
 {
-  surplus[FOOD] = pcity->food_surplus;
-  surplus[SHIELD] = pcity->shield_surplus;
-  surplus[TRADE] = pcity->trade_prod;
-  surplus[GOLD] = city_gold_surplus(pcity, pcity->tax_total);
-  surplus[LUXURY] = pcity->luxury_total;
-  surplus[SCIENCE] = pcity->science_total;
+  output_type_iterate(o) {
+    surplus[o] = pcity->surplus[o];
+  } output_type_iterate_end;
 
   *disorder = city_unhappy(pcity);
   *happy = city_happy(pcity);
@@ -708,7 +701,7 @@ static struct cm_fitness evaluate_solution(struct cm_state *state,
 {
   struct city *pcity = state->pcity;
   struct city backup;
-  int surplus[NUM_STATS];
+  int surplus[O_COUNT];
   bool disorder, happy;
 
   /* make a backup, apply and evaluate the solution, and restore.  This costs
@@ -766,8 +759,6 @@ static void convert_solution_to_result(struct cm_state *state,
 static int compare_tile_type_by_lattice_order(const struct cm_tile_type *a,
 					      const struct cm_tile_type *b)
 {
-  enum cm_stat stat;
-
   if (a == b) {
     return 0;
   }
@@ -778,11 +769,11 @@ static int compare_tile_type_by_lattice_order(const struct cm_tile_type *a,
   }
 
   /* With equal depth, break ties arbitrarily, more production first. */
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     if (a->production[stat] != b->production[stat]) {
       return b->production[stat] - a->production[stat];
     }
-  }
+  } output_type_iterate_end;
 
   /* If we get here, then we have two copies of identical types, an error */
   assert(0);
@@ -818,7 +809,7 @@ static int compare_tile_type_by_fitness(const void *va, const void *vb)
   return compare_tile_type_by_lattice_order(*a, *b);
 }
 
-static enum cm_stat compare_key;
+static Output_type_id compare_key;
 
 /****************************************************************************
   Compare by the production of type compare_key.
@@ -857,14 +848,10 @@ static void compute_tile_production(const struct city *pcity, int x, int y,
 {
   bool is_celebrating = base_city_celebrating(pcity);
 
-  out->production[FOOD]
-    = base_city_get_food_tile(x, y, pcity, is_celebrating);
-  out->production[SHIELD]
-    = base_city_get_shields_tile(x, y, pcity, is_celebrating);
-  out->production[TRADE]
-    = base_city_get_trade_tile(x, y, pcity, is_celebrating);
-  out->production[GOLD] = out->production[SCIENCE]
-    = out->production[LUXURY] = 0;
+  output_type_iterate(o) {
+    out->production[o] = base_city_get_output_tile(x, y, pcity,
+						   is_celebrating, o);
+  } output_type_iterate_end;
 }
 
 /****************************************************************************
@@ -918,19 +905,7 @@ static void tile_type_lattice_add(struct tile_type_vector *lattice,
 
 /*
  * Add the specialist types to the lattice.
- * This structure is necessary for now because each specialist
- * creates only one type of production and we need to map
- * indices from specialist_type to cm_stat.
  */
-struct spec_stat_pair {
-  enum specialist_type spec;
-  enum cm_stat stat;
-};
-const static struct spec_stat_pair pairs[SP_COUNT] =  {
-  { SP_ELVIS, LUXURY },
-  { SP_SCIENTIST, SCIENCE },
-  { SP_TAXMAN, GOLD }
-};
 
 /****************************************************************************
   Create lattice nodes for each type of specialist.  This adds a new
@@ -947,14 +922,13 @@ static void init_specialist_lattice_nodes(struct tile_type_vector *lattice,
   /* for each specialist type, create a tile_type that has as production
    * the bonus for the specialist (if the city is allowed to use it) */
   specialist_type_iterate(i) {
-    if (city_can_use_specialist(pcity, pairs[i].spec)) {
-      type.spec = pairs[i].spec;
-      type.production[pairs[i].stat]
-        = game.rgame.specialists[pairs[i].spec].bonus;
+    if (city_can_use_specialist(pcity, i)) {
+      type.spec = i;
+      output_type_iterate(output) {
+	type.production[output] = get_specialist_output(pcity, i, output);
+      } output_type_iterate_end;
 
       tile_type_lattice_add(lattice, &type, 0, 0);
-
-      type.production[pairs[i].stat] = 0;
     }
   } specialist_type_iterate_end;
 }
@@ -1109,7 +1083,7 @@ static void clean_lattice(struct tile_type_vector *lattice,
   much of the domain-specific knowledge.
 ****************************************************************************/
 static double estimate_fitness(const struct cm_state *state,
-			       const int production[NUM_STATS]);
+			       const int production[]);
 
 static void sort_lattice_by_fitness(const struct cm_state *state,
 				    struct tile_type_vector *lattice)
@@ -1148,7 +1122,7 @@ static void init_tile_lattice(const struct city *pcity,
     if (pcity->city_map[x][y] == C_TILE_UNAVAILABLE) {
       continue;
     }
-    if (!is_city_center(x, y)) {
+    if (!is_free_worked_tile(x, y)) {
       compute_tile_production(pcity, x, y, &type); /* clobbers type */
       tile_type_lattice_add(lattice, &type, x, y); /* copy type if needed */
     }
@@ -1210,7 +1184,6 @@ static void add_workers(struct partial_solution *soln,
 			int itype, int number,
 			const struct cm_state *state)
 {
-  enum cm_stat stat;
   const struct cm_tile_type *ptype = tile_type_get(state, itype);
   int newcount;
   int old_worker_count = soln->worker_counts[itype];
@@ -1249,11 +1222,11 @@ static void add_workers(struct partial_solution *soln,
   }
 
   /* update production */
-  for (stat = 0 ; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     newcount = soln->production[stat] + number * ptype->production[stat];
     assert(newcount >= 0);
     soln->production[stat] = newcount;
-  }
+  } output_type_iterate_end;
 }
 
 /****************************************************************************
@@ -1469,10 +1442,9 @@ static void complete_solution(struct partial_solution *soln,
 ****************************************************************************/
 static void compute_max_stats_heuristic(const struct cm_state *state,
 					const struct partial_solution *soln,
-					int production[NUM_STATS],
+					int production[],
 					int check_choice)
 {
-  enum cm_stat stat;
   struct partial_solution solnplus; /* will be soln, plus some tiles */
 
   /* Production is whatever the solution produces, plus the
@@ -1482,20 +1454,19 @@ static void compute_max_stats_heuristic(const struct cm_state *state,
   if (soln->idle == 1) {
     /* Then the total solution is soln + this new worker.  So we know the
        production exactly, and can shortcut the later code. */
-    enum cm_stat stat;
     const struct cm_tile_type *ptype = tile_type_get(state, check_choice);
 
     memcpy(production, soln->production, sizeof(soln->production));
-    for (stat = 0; stat < NUM_STATS; stat++) {
+    output_type_iterate(stat) {
       production[stat] += ptype->production[stat];
-    }
+    } output_type_iterate_end;
     return;
   }
 
   /* initialize solnplus here, after the shortcut check */
   init_partial_solution(&solnplus, num_types(state), state->pcity->size);
 
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     /* compute the solution that has soln, then the check_choice,
        then complete it with the best available tiles for the stat. */
     copy_partial_solution(&solnplus, soln, state);
@@ -1503,7 +1474,7 @@ static void compute_max_stats_heuristic(const struct cm_state *state,
     complete_solution(&solnplus, state, &state->lattice_by_prod[stat]);
 
     production[stat] = solnplus.production[stat];
-  }
+  } output_type_iterate_end;
 
   destroy_partial_solution(&solnplus);
 }
@@ -1516,16 +1487,15 @@ static void compute_max_stats_heuristic(const struct cm_state *state,
 ****************************************************************************/
 static bool choice_is_promising(struct cm_state *state, int newchoice)
 {
-  int production[NUM_STATS];
-  enum cm_stat stat;
+  int production[O_COUNT];
   bool beats_best = FALSE;
 
   compute_max_stats_heuristic(state, &state->current, production, newchoice);
 
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     if (production[stat] < state->min_production[stat]) {
       freelog(LOG_PRUNE_BRANCH, "--- pruning: insufficient %s (%d < %d)",
-	      cm_get_stat_name(stat), production[stat],
+	      get_output_name(stat), production[stat],
 	      state->min_production[stat]);
       return FALSE;
     }
@@ -1534,7 +1504,7 @@ static bool choice_is_promising(struct cm_state *state, int newchoice)
       /* may still fail to meet min at another production type, so
        * don't short-circuit */
     }
-  }
+  } output_type_iterate_end;
   if (!beats_best) {
     freelog(LOG_PRUNE_BRANCH, "--- pruning: best is better in all ways");
   }
@@ -1548,66 +1518,49 @@ static bool choice_is_promising(struct cm_state *state, int newchoice)
 ****************************************************************************/
 static void init_min_production(struct cm_state *state)
 {
-  int x = CITY_MAP_RADIUS, y = CITY_MAP_RADIUS;
-  int usage[NUM_STATS];
   struct city *pcity = state->pcity;
   bool is_celebrating = base_city_celebrating(pcity);
-  struct city backup;
 
-  /* make sure the city's numbers make sense (sometimes they don't,
-   * somehow) */
-  memcpy(&backup, pcity, sizeof(*pcity));
-  generic_city_refresh(pcity, FALSE, NULL);
+  /* We used to call generic_city_refresh here, but that's no longer
+   * necessary since it's called at the start of cm_query_result. */
 
   memset(state->min_production, 0, sizeof(state->min_production));
+  output_type_iterate(o) {
+    /* Calculate minimum output.  This assumes no waste.  Pruning
+     * is done mainly based on minimum production, so we want to find the
+     * absolute highest minimum possible.  However if the minimum we find
+     * here is incorrectly too high, then the algorithm will fail. */
+    int min;
 
-  /* If the city is content, then we know the food usage is just
-   * prod-surplus; otherwise, we know it's at least 2*size but we
-   * can't easily compute the settlers. */
-  if (!city_unhappy(pcity)) {
-    usage[FOOD] = pcity->food_prod - pcity->food_surplus;
-  } else {
-    usage[FOOD] = pcity->size * 2;
-  }
-  state->min_production[FOOD] = usage[FOOD]
-    + state->parameter.minimal_surplus[FOOD]
-    - base_city_get_food_tile(x, y, pcity, is_celebrating);
+    if (o != O_SHIELD && o != O_FOOD) {
+      /* min-production calculations are only possible for food and
+       * shields.  The others depend on complex trade calculations
+       * that cannot be accounted for since the bonus from trade
+       * routes depends on the amount of trade in an unpredictable way. */
+      continue;
+    }
 
-  /* surplus = (factories-waste) * production - shield_usage, so:
-   *   production = (surplus + shield_usage)/(factories-waste)
-   * waste >= 0, so:
-   *   production >= (surplus + usage)/factories
-   * Solving with surplus >= min_surplus, we get:
-   *   production >= (min_surplus + usage)/factories
-   * 'factories' is the pcity->shield_bonus/100.  Increase it a bit to avoid
-   * rounding errors.
-   *
-   * pcity->shield_prod = (factories-waste) * production.
-   * Therefore, shield_usage = pcity->shield_prod - pcity->shield_surplus
-   */
-  if (!city_unhappy(pcity)) {
-    double sbonus;
+    /* 1.  Calculate the minimum final production that is needed.
+     * 2.  Divide by the bonus (rounding down) to get the minimum citizen
+     *     production that is needed.
+     * 3.  Subtract off any "free" production (trade routes, tithes, and
+     *     city-center). */
+    min = pcity->usage[o] + state->parameter.minimal_surplus[o];
+    if (pcity->bonus[o] > 0) {
+      /* FIXME: how does this work if the bonus is 0?  Then no production
+       * is possible and we can probably short-cut everything. */
+      min = min * 100 / pcity->bonus[o];
+    }
+    city_map_iterate(x, y) {
+      if (is_free_worked_tile(x, y)) {
+	min -= base_city_get_output_tile(x, y, pcity, is_celebrating, o);
+      }
+    } city_map_iterate_end;
+    state->min_production[o] = MAX(min, 0);
+  } output_type_iterate_end;
 
-    usage[SHIELD] = pcity->shield_prod - pcity->shield_surplus;
-
-    sbonus = ((double)pcity->shield_bonus) / 100.0;
-    sbonus += .1;
-    state->min_production[SHIELD]
-      = (usage[SHIELD] + state->parameter.minimal_surplus[SHIELD]) / sbonus;
-    state->min_production[SHIELD]
-      -= base_city_get_shields_tile(x, y, pcity, is_celebrating);
-  } else {
-    /* Dunno what the usage is, so it's pointless to set the
-     * min_production */
-    usage[SHIELD] = 0;
-    state->min_production[SHIELD] = 0;
-  }
-
-  /* we should be able to get a min_production on gold and trade, too;
-     also, lux, if require_happy, but not otherwise */
-
-  /* undo any effects from the refresh */
-  memcpy(pcity, &backup, sizeof(*pcity));
+  /* We could get a minimum on luxury if we knew how many luxuries were
+   * needed to make us content. */
 }
 
 /****************************************************************************
@@ -1619,34 +1572,35 @@ static void init_min_production(struct cm_state *state)
   The only fields of the state used are the city and parameter.
 ****************************************************************************/
 static double estimate_fitness(const struct cm_state *state,
-			       const int production[NUM_STATS]) {
+			       const int production[]) {
   const struct city *pcity = state->pcity;
-  const struct player *pplayer = get_player(pcity->owner);
-  enum cm_stat stat;
-  double estimates[NUM_STATS];
+  const struct player *pplayer = city_owner(pcity);
+  double estimates[O_COUNT];
   double sum = 0;
 
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     estimates[stat] = production[stat];
-  }
+  } output_type_iterate_end;
 
   /* sci/lux/gold get benefit from the tax rates (in percentage) */
-  estimates[SCIENCE] += pplayer->economic.science * estimates[TRADE] / 100.0;
-  estimates[LUXURY] += pplayer->economic.luxury  * estimates[TRADE] / 100.0;
-  estimates[GOLD] += pplayer->economic.tax     * estimates[TRADE] / 100.0;
+  estimates[O_SCIENCE]
+    += pplayer->economic.science * estimates[O_TRADE] / 100.0;
+  estimates[O_LUXURY]
+    += pplayer->economic.luxury * estimates[O_TRADE] / 100.0;
+  estimates[O_GOLD]
+    += pplayer->economic.tax * estimates[O_TRADE] / 100.0;
 
-  /* now add in the bonuses (none for food or trade) (in percentage) */
-  estimates[SHIELD] *= pcity->shield_bonus / 100.0;
-  estimates[LUXURY] *= pcity->luxury_bonus / 100.0;
-  estimates[GOLD] *= pcity->tax_bonus / 100.0;
-  estimates[SCIENCE] *= pcity->science_bonus / 100.0;
+  /* now add in the bonuses from building effects (in percentage) */
+  output_type_iterate(stat) {
+    estimates[stat] *= pcity->bonus[stat] / 100.0;
+  } output_type_iterate_end;
 
   /* finally, sum it all up, weighted by the parameter, but give additional
    * weight to luxuries to take account of disorder/happy constraints */
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     sum += estimates[stat] * state->parameter.factor[stat];
-  }
-  sum += estimates[LUXURY];
+  } output_type_iterate_end;
+  sum += estimates[O_LUXURY];
   return sum;
 }
 
@@ -1703,11 +1657,10 @@ static bool bb_next(struct cm_state *state)
 struct cm_state *cm_init_state(struct city *pcity)
 {
   int numtypes;
-  enum cm_stat stat;
   struct cm_state *state = fc_malloc(sizeof(*state));
 
   freelog(LOG_CM_STATE, "creating cm_state for %s (size %d)",
-	  pcity->name, pcity->size);
+	  city_name(pcity), pcity->size);
 
   /* copy the arguments */
   state->pcity = pcity;
@@ -1718,14 +1671,14 @@ struct cm_state *cm_init_state(struct city *pcity)
   numtypes = tile_type_vector_size(&state->lattice);
 
   /* For the heuristic, make sorted copies of the lattice */
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     tile_type_vector_init(&state->lattice_by_prod[stat]);
     tile_type_vector_copy(&state->lattice_by_prod[stat], &state->lattice);
     compare_key = stat;
     qsort(state->lattice_by_prod[stat].p, state->lattice_by_prod[stat].size,
 	  sizeof(*state->lattice_by_prod[stat].p),
 	  compare_tile_type_by_stat);
-  }
+  } output_type_iterate_end;
 
   /* We have no best solution yet, so its value is the worst possible. */
   init_partial_solution(&state->best, numtypes, pcity->size);
@@ -1788,12 +1741,10 @@ static void end_search(struct cm_state *state)
 ****************************************************************************/
 void cm_free_state(struct cm_state *state)
 {
-  enum cm_stat stat;
-
   tile_type_vector_free_all(&state->lattice);
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     tile_type_vector_free(&state->lattice_by_prod[stat]);
-  }
+  } output_type_iterate_end;
   destroy_partial_solution(&state->best);
   destroy_partial_solution(&state->current);
   free(state->choice.stack);
@@ -1843,48 +1794,20 @@ void cm_query_result(struct city *pcity,
   cm_free_state(state);
 }
 
-
-/****************************************************************************
-  Return a translated name for the stat type.
-*****************************************************************************/
-const char *cm_get_stat_name(enum cm_stat stat)
-{
-  switch (stat) {
-  case FOOD:
-    return _("Food");
-  case SHIELD:
-    return _("Shield");
-  case TRADE:
-    return _("Trade");
-  case GOLD:
-    return _("Gold");
-  case LUXURY:
-    return _("Luxury");
-  case SCIENCE:
-    return _("Science");
-  case NUM_STATS:
-    break;
-  }
-  die("Unknown stat value in cm_get_stat_name: %d", stat);
-  return NULL;
-}
-
 /**************************************************************************
   Returns true if the two cm_parameters are equal.
 **************************************************************************/
 bool cm_are_parameter_equal(const struct cm_parameter *const p1,
 			    const struct cm_parameter *const p2)
 {
-  int i;
-
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     if (p1->minimal_surplus[i] != p2->minimal_surplus[i]) {
       return FALSE;
     }
     if (p1->factor[i] != p2->factor[i]) {
       return FALSE;
     }
-  }
+  } output_type_iterate_end;
   if (p1->require_happy != p2->require_happy) {
     return FALSE;
   }
@@ -1915,12 +1838,10 @@ void cm_copy_parameter(struct cm_parameter *dest,
 **************************************************************************/
 void cm_init_parameter(struct cm_parameter *dest)
 {
-  enum cm_stat stat;
-
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     dest->minimal_surplus[stat] = 0;
     dest->factor[stat] = 1;
-  }
+  } output_type_iterate_end;
 
   dest->happy_factor = 1;
   dest->require_happy = FALSE;
@@ -1934,12 +1855,10 @@ void cm_init_parameter(struct cm_parameter *dest)
 ***************************************************************************/
 void cm_init_emergency_parameter(struct cm_parameter *dest)
 {
-  enum cm_stat stat;
-
-  for (stat = 0; stat < NUM_STATS; stat++) {
+  output_type_iterate(stat) {
     dest->minimal_surplus[stat] = -FC_INFINITY;
     dest->factor[stat] = 1;
-  }
+  } output_type_iterate_end;
 
   dest->happy_factor = 1;
   dest->require_happy = FALSE;
@@ -1956,7 +1875,7 @@ int cm_count_worker(const struct city * pcity,
   int count = 0;
 
   city_map_iterate(x, y) {
-    if(result->worker_positions_used[x][y] && !is_city_center(x, y)) {
+    if (result->worker_positions_used[x][y] && !is_free_worked_tile(x, y)) {
       count++;
     }
   } city_map_iterate_end;
@@ -2008,16 +1927,12 @@ void cm_copy_result_from_city(const struct city *pcity,
 ****************************************************************************/
 #ifdef CM_DEBUG
 static void snprint_production(char *buffer, size_t bufsz,
-			      const int production[NUM_STATS])
+			       const int production[])
 {
-  int nout;
-
-  nout = snprintf(buffer, bufsz, "[%d %d %d %d %d %d]",
-		  production[FOOD], production[SHIELD],
-		  production[TRADE], production[GOLD],
-		  production[LUXURY], production[SCIENCE]);
-
-  assert(nout >= 0 && nout <= bufsz);
+  my_snprintf(buffer, bufsz, "[%d %d %d %d %d %d]",
+	      production[O_FOOD], production[O_SHIELD],
+	      production[O_TRADE], production[O_GOLD],
+	      production[O_LUXURY], production[O_SCIENCE]);
 }
 
 /****************************************************************************
@@ -2069,8 +1984,8 @@ static void print_partial_solution(int loglevel,
   freelog(loglevel, "tiles used:");
   for (i = 0; i < num_types(state); i++) {
     if (soln->worker_counts[i] != 0) {
-      snprintf(buf, sizeof(buf),
-          "  %d tiles of type ", soln->worker_counts[i]);
+      my_snprintf(buf, sizeof(buf), "  %d tiles of type ",
+		  soln->worker_counts[i]);
       print_tile_type(loglevel, tile_type_get(state, i), buf);
     }
   }
@@ -2124,12 +2039,10 @@ static void print_performance(struct one_perf *counts)
 void cm_print_city(const struct city *pcity)
 {
   freelog(LOG_NORMAL, "print_city(city='%s'(id=%d))",
-          pcity->name, pcity->id);
+          city_name(pcity), pcity->id);
   freelog(LOG_NORMAL,
-          "  size=%d, entertainers=%d, scientists=%d, taxmen=%d",
-          pcity->size, pcity->specialists[SP_ELVIS],
-          pcity->specialists[SP_SCIENTIST],
-          pcity->specialists[SP_TAXMAN]);
+	  "  size=%d, specialists=%s",
+	  pcity->size, specialists_string(pcity->specialists));
   freelog(LOG_NORMAL, "  workers at:");
   my_city_map_iterate(pcity, x, y) {
     if (pcity->city_map[x][y] == C_TILE_WORKER) {
@@ -2138,15 +2051,15 @@ void cm_print_city(const struct city *pcity)
   } my_city_map_iterate_end;
 
   freelog(LOG_NORMAL, "  food    = %3d (%+3d)",
-          pcity->food_prod, pcity->food_surplus);
+          pcity->prod[O_FOOD], pcity->surplus[O_FOOD]);
   freelog(LOG_NORMAL, "  shield  = %3d (%+3d)",
-          pcity->shield_prod, pcity->shield_surplus);
-  freelog(LOG_NORMAL, "  trade   = %3d", pcity->trade_prod);
+          pcity->prod[O_SHIELD], pcity->surplus[O_SHIELD]);
+  freelog(LOG_NORMAL, "  trade   = %3d", pcity->surplus[O_TRADE]);
 
-  freelog(LOG_NORMAL, "  gold    = %3d (%+3d)", pcity->tax_total,
-          city_gold_surplus(pcity, pcity->tax_total));
-  freelog(LOG_NORMAL, "  luxury  = %3d", pcity->luxury_total);
-  freelog(LOG_NORMAL, "  science = %3d", pcity->science_total);
+  freelog(LOG_NORMAL, "  gold    = %3d (%+3d)", pcity->prod[O_GOLD],
+	  pcity->surplus[O_GOLD]);
+  freelog(LOG_NORMAL, "  luxury  = %3d", pcity->prod[O_LUXURY]);
+  freelog(LOG_NORMAL, "  science = %3d", pcity->prod[O_SCIENCE]);
 }
 
 
@@ -2156,8 +2069,9 @@ void cm_print_city(const struct city *pcity)
 void cm_print_result(const struct city *pcity,
 		     const struct cm_result *result)
 {
-  int y, i, worker = cm_count_worker(pcity, result);
-  freelog(LOG_NORMAL, "print_result(result=%p)", result);
+  int y, worker = cm_count_worker(pcity, result);
+
+  freelog(LOG_NORMAL, "print_result(result=%p)", (void *)result);
   freelog(LOG_NORMAL,
       "print_result:  found_a_valid=%d disorder=%d happy=%d",
       result->found_a_valid, result->disorder, result->happy);
@@ -2178,7 +2092,7 @@ void cm_print_result(const struct city *pcity,
     for (x = 0; x < CITY_MAP_SIZE; x++) {
       if (!is_valid_city_coords(x, y)) {
         line[x] = '-';
-      } else if (is_city_center(x, y)) {
+      } else if (is_free_worked_tile(x, y)) {
         line[x] = 'c';
       } else if (result->worker_positions_used[x][y]) {
         line[x] = 'w';
@@ -2192,8 +2106,8 @@ void cm_print_result(const struct city *pcity,
       "print_result:  people: (workers/specialists) %d/%s",
       worker, specialists_string(result->specialists));
 
-  for (i = 0; i < NUM_STATS; i++) {
+  output_type_iterate(i) {
     freelog(LOG_NORMAL, "print_result:  %10s surplus=%d",
-        cm_get_stat_name(i), result->surplus[i]);
-  }
+        get_output_name(i), result->surplus[i]);
+  } output_type_iterate_end;
 }
