@@ -419,10 +419,7 @@ class Field:
 
         if self.dataio_type=="bitvector":
             return "DIO_BV_GET(&din, real_packet->%(name)s);"%self.__dict__
-
-        if self.dataio_type=="memory":
-            return "dio_get_%(dataio_type)s(&din, real_packet->%(name)s, %(array_size_u)s);"%self.__dict__
-        if self.dataio_type in ["string","bit_string","city_map","memory"] and \
+        if self.dataio_type in ["string","bit_string","city_map"] and \
            self.is_array!=2:
             return "dio_get_%(dataio_type)s(&din, real_packet->%(name)s, sizeof(real_packet->%(name)s));"%self.__dict__
         if self.is_struct and self.is_array==0:
@@ -464,7 +461,7 @@ class Field:
             array_size_u=self.array_size_u
             array_size_d=self.array_size_d
 
-        if not self.diff:
+        if not self.diff or self.dataio_type == "memory":
             if array_size_u != array_size_d:
                 extra='''
   if(%(array_size_u)s > %(array_size_d)s) {
@@ -472,7 +469,10 @@ class Field:
     %(array_size_u)s = %(array_size_d)s;
   }'''%self.get_dict(vars())
             else:
-                extra=""                
+                extra=""
+            if self.dataio_type == "memory":
+                return '''%(extra)s
+  dio_get_%(dataio_type)s(&din, real_packet->%(name)s, %(array_size_u)s);'''%self.get_dict(vars())
             return '''
 {
   int i;
@@ -734,7 +734,10 @@ static char *stats_%(name)s_names[] = {%(names)s};
             delta_header=""
 
         if self.want_post_send:
-            post="  post_send_%(packet_name)s(pc, real_packet);\n"
+            if self.no_packet:
+                post="  post_send_%(packet_name)s(pc, NULL);\n"
+            else:
+                post="  post_send_%(packet_name)s(pc, real_packet);\n"
         else:
             post=""
 
@@ -1101,12 +1104,12 @@ class Packet:
         only_client=len(self.dirs)==1 and self.dirs[0]=="sc"
         only_server=len(self.dirs)==1 and self.dirs[0]=="cs"
         if only_client:
-            restrict='''  if(is_server) {
+            restrict='''  if (pc->is_server) {
     freelog(LOG_ERROR, "Receiving %(name)s at the server.");
   }
 '''%self.get_dict(vars())
         elif only_server:
-            restrict='''  if(!is_server) {
+            restrict='''  if (!pc->is_server) {
     freelog(LOG_ERROR, "Receiving %(name)s at the client.");
   }
 '''%self.get_dict(vars())
@@ -1137,12 +1140,12 @@ class Packet:
         only_client=len(self.dirs)==1 and self.dirs[0]=="cs"
         only_server=len(self.dirs)==1 and self.dirs[0]=="sc"
         if only_client:
-            restrict='''  if(is_server) {
+            restrict='''  if (pc->is_server) {
     freelog(LOG_ERROR, "Sending %(name)s from the server.");
   }
 '''%self.get_dict(vars())
         elif only_server:
-            restrict='''  if(!is_server) {
+            restrict='''  if (!pc->is_server) {
     freelog(LOG_ERROR, "Sending %(name)s from the client.");
   }
 '''%self.get_dict(vars())
@@ -1191,7 +1194,7 @@ class Packet:
         if not self.want_lsend: return ""
         return '''%(lsend_prototype)s
 {
-  conn_list_iterate(*dest, pconn) {
+  conn_list_iterate(dest, pconn) {
     send_%(name)s(pconn%(extra_send_args2)s);
   } conn_list_iterate_end;
 }
@@ -1508,7 +1511,10 @@ bool server_handle_packet(enum packet_type type, void *packet,
                 b=", "+b
             if p.handle_via_packet:
                 f.write('struct %s;\n'%p.name)
-                f.write('void handle_%s(struct player *pplayer, struct %s *packet);\n'%(a,p.name))
+                if p.handle_per_conn:
+                    f.write('void handle_%s(struct connection *pc, struct %s *packet);\n'%(a,p.name))
+                else:
+                    f.write('void handle_%s(struct player *pplayer, struct %s *packet);\n'%(a,p.name))
             else:
                 if p.handle_per_conn:
                     f.write('void handle_%s(struct connection *pc%s);\n'%(a,b))
@@ -1582,7 +1588,11 @@ bool server_handle_packet(enum packet_type type, void *packet,
             b=",\n      "+b
 
         if p.handle_via_packet:
-            args="pplayer, packet"
+             if p.handle_per_conn:
+                 args="pconn, packet"
+             else:
+                 args="pplayer, packet"
+
         else:
             if p.handle_per_conn:
                 args="pconn"+b
