@@ -28,7 +28,11 @@
    unless IDEX_DIE set.
 ***************************************************************************/
 
-#include <stdlib.h>
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include <assert.h>
 
 #include "city.h"
 #include "hash.h"
@@ -39,7 +43,7 @@
 
 
 #ifndef IDEX_DIE
-#define IDEX_DIE 0
+#define IDEX_DIE FALSE
 #endif
 #define LOG_IDEX_ERR (IDEX_DIE ? LOG_FATAL : LOG_ERROR)
 
@@ -50,35 +54,44 @@ static struct hash_table *idex_unit_hash = NULL;
 
 /**************************************************************************
    Initialize.  Should call this at the start before use.
-   May also call later, to re-initialize (eg, client disconnects).
 ***************************************************************************/
 void idex_init(void)
 {
-  if (idex_city_hash) {
-    hash_free(idex_city_hash);
-    idex_city_hash = NULL;
-  }
-  if (idex_unit_hash) {
-    hash_free(idex_unit_hash);
-    idex_unit_hash = NULL;
-  }
+  assert(idex_city_hash == NULL);
+  assert(idex_unit_hash == NULL);
+
   idex_city_hash = hash_new(hash_fval_int, hash_fcmp_int);
   idex_unit_hash = hash_new(hash_fval_int, hash_fcmp_int);
 }
 
+/**************************************************************************
+   Free the hashs.
+***************************************************************************/
+void idex_free(void)
+{
+  hash_free(idex_city_hash);
+  idex_city_hash = NULL;
+
+  hash_free(idex_unit_hash);
+  idex_unit_hash = NULL;
+}
+
 /**************************************************************************
    Register a city into idex, with current pcity->id.
    Call this when pcity created.
 ***************************************************************************/
 void idex_register_city(struct city *pcity)
 {
-  struct city *old = hash_replace(idex_city_hash, &pcity->id, pcity);
+  struct city *old = (struct city *)
+    hash_replace(idex_city_hash, &pcity->id, pcity);
   if (old) {
     /* error */
     freelog(LOG_IDEX_ERR, "IDEX: city collision: new %d %p %s, old %d %p %s",
-	    pcity->id, (void*)pcity, pcity->name,
-	    old->id, (void*)old, old->name);
-    if (IDEX_DIE) abort();
+	    pcity->id, (void*)pcity, city_name(pcity),
+	    old->id, (void*)old, city_name(old));
+    if (IDEX_DIE) {
+      die("byebye");
+    }
   }
 }
 
@@ -88,13 +101,18 @@ void idex_register_city(struct city *pcity)
 ***************************************************************************/
 void idex_register_unit(struct unit *punit)
 {
-  struct unit *old = hash_replace(idex_unit_hash, &punit->id, punit);
+  struct unit *old = (struct unit *)
+    hash_replace(idex_unit_hash, &punit->id, punit);
   if (old) {
     /* error */
     freelog(LOG_IDEX_ERR, "IDEX: unit collision: new %d %p %s, old %d %p %s",
-	    punit->id, (void*)punit, unit_name(punit->type),
-	    old->id, (void*)old, unit_name(old->type));
-    if (IDEX_DIE) abort();
+	    punit->id, (void*)punit,
+	    unit_rule_name(punit),
+	    old->id, (void*)old,
+	    unit_rule_name(old));
+    if (IDEX_DIE) {
+      die("byebye");
+    }
   }
 }
 
@@ -104,19 +122,24 @@ void idex_register_unit(struct unit *punit)
 ***************************************************************************/
 void idex_unregister_city(struct city *pcity)
 {
-  struct city *old = hash_delete_entry(idex_city_hash, &pcity->id);
+  struct city *old = (struct city *)
+    hash_delete_entry(idex_city_hash, &pcity->id);
   if (!old) {
     /* error */
     freelog(LOG_IDEX_ERR, "IDEX: city unreg missing: %d %p %s",
-	    pcity->id, (void*)pcity, pcity->name);
-    if (IDEX_DIE) abort();
+	    pcity->id, (void*)pcity, city_name(pcity));
+    if (IDEX_DIE) {
+      die("byebye");
+    }
   } else if (old != pcity) {
     /* error */
     freelog(LOG_IDEX_ERR,
 	    "IDEX: city unreg mismatch: unreg %d %p %s, old %d %p %s",
-	    pcity->id, (void*)pcity, pcity->name,
-	    old->id, (void*)old, old->name);
-    if (IDEX_DIE) abort();
+	    pcity->id, (void*)pcity, city_name(pcity),
+	    old->id, (void*)old, city_name(old));
+    if (IDEX_DIE) {
+      die("byebye");
+    }
   }
 }
 
@@ -126,19 +149,27 @@ void idex_unregister_city(struct city *pcity)
 ***************************************************************************/
 void idex_unregister_unit(struct unit *punit)
 {
-  struct unit *old = hash_delete_entry(idex_unit_hash, &punit->id);
+  struct unit *old = (struct unit *)
+    hash_delete_entry(idex_unit_hash, &punit->id);
   if (!old) {
     /* error */
     freelog(LOG_IDEX_ERR, "IDEX: unit unreg missing: %d %p %s",
-	    punit->id, (void*)punit, unit_name(punit->type));
-    if (IDEX_DIE) abort();
+	    punit->id, (void*)punit,
+	    unit_rule_name(punit));
+    if (IDEX_DIE) {
+      die("byebye");
+    }
   } else if (old != punit) {
     /* error */
     freelog(LOG_IDEX_ERR,
 	    "IDEX: unit unreg mismatch: unreg %d %p %s, old %d %p %s",
-	    punit->id, (void*)punit, unit_name(punit->type),
-	    old->id, (void*)old, unit_name(old->type));
-    if (IDEX_DIE) abort();
+	    punit->id, (void*)punit,
+	    unit_rule_name(punit),
+	    old->id, (void*)old,
+	    unit_rule_name(old));
+    if (IDEX_DIE) {
+      die("byebye");
+    }
   }
 }
 
@@ -148,7 +179,7 @@ void idex_unregister_unit(struct unit *punit)
 ***************************************************************************/
 struct city *idex_lookup_city(int id)
 {
-  return hash_lookup_data(idex_city_hash, &id);
+  return (struct city *)hash_lookup_data(idex_city_hash, &id);
 }
 
 /**************************************************************************
@@ -157,6 +188,5 @@ struct city *idex_lookup_city(int id)
 ***************************************************************************/
 struct unit *idex_lookup_unit(int id)
 {
-  return hash_lookup_data(idex_unit_hash, &id);
+  return (struct unit *)hash_lookup_data(idex_unit_hash, &id);
 }
-
