@@ -14,19 +14,21 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-
 #include <assert.h>
 #include <string.h>		/* strlen */
 
 #include "city.h"
 #include "fcintl.h"
+#include "game.h"
 #include "log.h"
 #include "mem.h"
+#include "movement.h"
 #include "packets.h"
 #include "rand.h"
 #include "shared.h"
 #include "support.h"
 #include "unit.h"
+#include "unitlist.h"
 
 #include "map.h"
 
@@ -54,114 +56,26 @@ struct terrain_misc terrain_control;
 const int DIR_DX[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
 const int DIR_DY[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
 
-/* Names of specials.
- * (These must correspond to enum tile_special_type in terrain.h.)
- */
-static const char *tile_special_type_names[] =
-{
-  N_("Special1"),
-  N_("Road"),
-  N_("Irrigation"),
-  N_("Railroad"),
-  N_("Mine"),
-  N_("Pollution"),
-  N_("Hut"),
-  N_("Fortress"),
-  N_("Special2"),
-  N_("River"),
-  N_("Farmland"),
-  N_("Airbase"),
-  N_("Fallout")
-};
-
 /****************************************************************************
   Return a bitfield of the specials on the tile that are infrastructure.
 ****************************************************************************/
-enum tile_special_type get_tile_infrastructure_set(const struct tile *ptile)
+bv_special get_tile_infrastructure_set(const struct tile *ptile,
+					  int *pcount)
 {
-  return (ptile->special
-	  & (S_ROAD | S_RAILROAD | S_IRRIGATION | S_FARMLAND | S_MINE
-	     | S_FORTRESS | S_AIRBASE));
-}
+  bv_special pspresent;
+  int i, count = 0;
 
-/***************************************************************
-  Return a (static) string with terrain name;
-  eg: "Hills"
-  eg: "Hills (Coals)"
-  eg: "Hills (Coals) [Pollution]"
-***************************************************************/
-const char *map_get_tile_info_text(const struct tile *ptile)
-{
-  static char s[64];
-  bool first;
-
-  sz_strlcpy(s, tile_types[ptile->terrain].terrain_name);
-  if (tile_has_special(ptile, S_RIVER)) {
-    sz_strlcat(s, "/");
-    sz_strlcat(s, get_special_name(S_RIVER));
-  }
-
-  first = TRUE;
-  if (tile_has_special(ptile, S_SPECIAL_1)) {
-    if (first) {
-      first = FALSE;
-      sz_strlcat(s, " (");
-    } else {
-      sz_strlcat(s, "/");
+  BV_CLR_ALL(pspresent);
+  for (i = 0; infrastructure_specials[i] != S_LAST; i++) {
+    if (tile_has_special(ptile, infrastructure_specials[i])) {
+      BV_SET(pspresent, infrastructure_specials[i]);
+      count++;
     }
-    sz_strlcat(s, tile_types[ptile->terrain].special_1_name);
   }
-  if (tile_has_special(ptile, S_SPECIAL_2)) {
-    if (first) {
-      first = FALSE;
-      sz_strlcat(s, " (");
-    } else {
-      sz_strlcat(s, "/");
-    }
-    sz_strlcat(s, tile_types[ptile->terrain].special_2_name);
+  if (pcount) {
+    *pcount = count;
   }
-  if (!first) {
-    sz_strlcat(s, ")");
-  }
-
-  first = TRUE;
-  if (tile_has_special(ptile, S_POLLUTION)) {
-    if (first) {
-      first = FALSE;
-      sz_strlcat(s, " [");
-    } else {
-      sz_strlcat(s, "/");
-    }
-    sz_strlcat(s, get_special_name(S_POLLUTION));
-  }
-  if (tile_has_special(ptile, S_FALLOUT)) {
-    if (first) {
-      first = FALSE;
-      sz_strlcat(s, " [");
-    } else {
-      sz_strlcat(s, "/");
-    }
-    sz_strlcat(s, get_special_name(S_FALLOUT));
-  }
-  if (!first) {
-    sz_strlcat(s, "]");
-  }
-
-  return s;
-}
-
-/***************************************************************
-  Return a (static) string with a tile's food/prod/trade
-***************************************************************/
-const char *map_get_tile_fpt_text(const struct tile *ptile)
-{
-  static char s[64];
-  
-  my_snprintf(s, sizeof(s), "%d/%d/%d",
-	      get_food_tile(ptile),
-	      get_shields_tile(ptile),
-	      get_trade_tile(ptile));
-  return s;
+  return pspresent;
 }
 
 /***************************************************************
@@ -205,7 +119,7 @@ void map_init(void)
   map.num_continents        = 0;
   map.num_oceans            = 0;
   map.num_start_positions   = 0;
-  map.have_specials         = FALSE;
+  map.have_resources = FALSE;
   map.have_rivers_overlay   = FALSE;
   map.have_huts             = FALSE;
 }
@@ -319,7 +233,7 @@ void map_init_topology(bool set_sizes)
 
   if (!set_sizes) {
     /* Set map.size based on map.xsize and map.ysize. */
-    map.size = (float)(map.xsize * map.ysize) / 1000.0 + 0.5;
+    map.size = (float)(map_num_tiles()) / 1000.0 + 0.5;
   }
   
   /* sanity check for iso topologies*/
@@ -353,16 +267,21 @@ void map_init_topology(bool set_sizes)
 ***************************************************************/
 static void tile_init(struct tile *ptile)
 {
-  ptile->terrain  = T_UNKNOWN;
-  ptile->special  = S_NO_SPECIAL;
-  ptile->known    = 0;
   ptile->continent = 0;
+
+  BV_CLR_ALL(ptile->tile_known);
+  vision_layer_iterate(v) {
+    BV_CLR_ALL(ptile->tile_seen[v]);
+  } vision_layer_iterate_end;
+
+  tile_clear_all_specials (ptile);
+  ptile->resource = NULL;
+  ptile->terrain  = T_UNKNOWN;
   ptile->city     = NULL;
-  unit_list_init(&ptile->units);
+  ptile->units    = unit_list_new();
   ptile->worked   = NULL; /* pointer to city working tile */
-  ptile->assigned = 0; /* bitvector */
   ptile->owner    = NULL; /* Tile not claimed by any nation. */
-  ptile->client.hilite = HILITE_NONE; /* Area Selection in client. */
+  ptile->owner_source = NULL;
   ptile->spec_sprite = NULL;
 }
 
@@ -448,7 +367,7 @@ struct tile *index_to_tile(int index)
     return NULL;
   }
 
-  if (index >= 0 && index < MAX_MAP_INDEX) {
+  if (index >= 0 && index < MAP_INDEX_SIZE) {
     return map.tiles + index;
   } else {
     /* Unwrapped index coordinates are impossible, so the best we can do is
@@ -457,28 +376,13 @@ struct tile *index_to_tile(int index)
   }
 }
 
-/**************************************************************************
-  Return the player who owns this tile (or NULL if none).
-**************************************************************************/
-struct player *map_get_owner(const struct tile *ptile)
-{
-  return ptile->owner;
-}
-
-/**************************************************************************
-  Set the owner of a tile (may be NULL).
-**************************************************************************/
-void map_set_owner(struct tile *ptile, struct player *owner)
-{
-  ptile->owner = owner;
-}
-
 /***************************************************************
 ...
 ***************************************************************/
 static void tile_free(struct tile *ptile)
 {
-  unit_list_unlink_all(&ptile->units);
+  unit_list_unlink_all(ptile->units);
+  unit_list_free(ptile->units);
   if (ptile->spec_sprite) {
     free(ptile->spec_sprite);
     ptile->spec_sprite = NULL;
@@ -492,28 +396,21 @@ static void tile_free(struct tile *ptile)
 void map_allocate(void)
 {
   freelog(LOG_DEBUG, "map_allocate (was %p) (%d,%d)",
-	  map.tiles, map.xsize, map.ysize);
+	  (void *)map.tiles, map.xsize, map.ysize);
 
   assert(map.tiles == NULL);
-  map.tiles = fc_malloc(map.xsize * map.ysize * sizeof(struct tile));
+  map.tiles = fc_malloc(MAP_INDEX_SIZE * sizeof(*map.tiles));
+
+  /* Note this use of whole_map_iterate may be a bit sketchy, since the
+   * tile values (ptile->index, etc.) haven't been set yet.  It might be
+   * better to do a manual loop here. */
   whole_map_iterate(ptile) {
-    int index, nat_x, nat_y, map_x, map_y;
-
-    index = ptile - map.tiles;
-    index_to_native_pos(&nat_x, &nat_y, index);
-    index_to_map_pos(&map_x, &map_y, index);
-    CHECK_INDEX(index);
-    CHECK_MAP_POS(map_x, map_y);
-    CHECK_NATIVE_POS(nat_x, nat_y);
-
-    /* HACK: these fields are declared const to keep anyone from changing
-     * them.  But we have to set them somewhere!  This should be the only
-     * place. */
-    *(int *)&ptile->index = index;
-    *(int *)&ptile->x = map_x;
-    *(int *)&ptile->y = map_y;
-    *(int *)&ptile->nat_x = nat_x;
-    *(int *)&ptile->nat_y = nat_y;
+    ptile->index = ptile - map.tiles;
+    index_to_map_pos(&ptile->x, &ptile->y, ptile->index);
+    index_to_native_pos(&ptile->nat_x, &ptile->nat_y, ptile->index);
+    CHECK_INDEX(ptile->index);
+    CHECK_MAP_POS(ptile->x, ptile->y);
+    CHECK_NATIVE_POS(ptile->nat_x, ptile->nat_y);
 
     tile_init(ptile);
   } whole_map_iterate_end;
@@ -539,41 +436,6 @@ void map_free(void)
   }
 }
 
-/***************************************************************
-...
-***************************************************************/
-enum tile_special_type get_special_by_name(const char * name)
-{
-  int i;
-  enum tile_special_type st = 1;
-
-  for (i = 0; i < ARRAY_SIZE(tile_special_type_names); i++) {
-    if (0 == strcmp(name, tile_special_type_names[i]))
-      return st;
-      
-    st <<= 1;
-  }
-
-  return S_NO_SPECIAL;
-}
-
-/***************************************************************
-...
-***************************************************************/
-const char *get_special_name(enum tile_special_type type)
-{
-  int i;
-
-  for (i = 0; i < ARRAY_SIZE(tile_special_type_names); i++) {
-    if ((type & 0x1) == 1) {
-      return _(tile_special_type_names[i]);
-    }
-    type >>= 1;
-  }
-
-  return NULL;
-}
-
 /****************************************************************************
   Return the "distance" (which is really the Manhattan distance, and should
   rarely be used) for a given vector.
@@ -594,6 +456,8 @@ static int map_vector_to_distance(int dx, int dy)
 ****************************************************************************/
 int map_vector_to_real_distance(int dx, int dy)
 {
+  const int absdx = abs(dx), absdy = abs(dy);
+
   if (topo_has_flag(TF_HEX)) {
     if (topo_has_flag(TF_ISO)) {
       /* Iso-hex: you can't move NE or SW. */
@@ -601,10 +465,10 @@ int map_vector_to_real_distance(int dx, int dy)
 	  || (dx > 0 && dy < 0)) {
 	/* Diagonal moves in this direction aren't allowed, so it will take
 	 * the full number of moves. */
-	return abs(dx) + abs(dy);
+        return absdx + absdy;
       } else {
 	/* Diagonal moves in this direction *are* allowed. */
-	return MAX(abs(dx), abs(dy));
+        return MAX(absdx, absdy);
       }
     } else {
       /* Hex: you can't move SE or NW. */
@@ -612,14 +476,14 @@ int map_vector_to_real_distance(int dx, int dy)
 	  || (dx < 0 && dy < 0)) {
 	/* Diagonal moves in this direction aren't allowed, so it will take
 	 * the full number of moves. */
-	return abs(dx) + abs(dy);
+	return absdx + absdy;
       } else {
 	/* Diagonal moves in this direction *are* allowed. */
-	return MAX(abs(dx), abs(dy));
+        return MAX(absdx, absdy);
       }
     }
   } else {
-    return MAX(abs(dx), abs(dy));
+    return MAX(absdx, absdy);
   }
 }
 
@@ -683,13 +547,7 @@ int map_distance(const struct tile *tile0, const struct tile *tile1)
 *************************************************************************/
 bool is_cardinally_adj_to_ocean(const struct tile *ptile)
 {
-  cardinal_adjc_iterate(ptile, tile1) {
-    if (is_ocean(map_get_terrain(tile1))) {
-      return TRUE;
-    }
-  } cardinal_adjc_iterate_end;
-
-  return FALSE;
+  return count_terrain_flag_near_tile(ptile, TRUE, FALSE, TER_OCEANIC) > 0;
 }
 
 /****************************************************************************
@@ -697,147 +555,8 @@ bool is_cardinally_adj_to_ocean(const struct tile *ptile)
 ****************************************************************************/
 bool is_safe_ocean(const struct tile *ptile)
 {
-  adjc_iterate(ptile, tile1) {
-    Terrain_type_id ter = map_get_terrain(tile1);
-    if (!terrain_has_flag(ter, TER_UNSAFE_COAST) && ter != T_UNKNOWN) {
-      return TRUE;
-    }
-  } adjc_iterate_end;
-
-  return FALSE;
-}
-
-/***************************************************************
-Returns whether you can put a city on land near enough to use
-the tile.
-***************************************************************/
-bool is_sea_usable(const struct tile *ptile)
-{
-  map_city_radius_iterate(ptile, tile1) {
-    if (!is_ocean(map_get_terrain(tile1))) {
-      return TRUE;
-    }
-  } map_city_radius_iterate_end;
-
-  return FALSE;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int get_tile_food_base(const struct tile *ptile)
-{
-  if (tile_has_special(ptile, S_SPECIAL_1)) 
-    return tile_types[ptile->terrain].food_special_1;
-  else if (tile_has_special(ptile, S_SPECIAL_2))
-    return tile_types[ptile->terrain].food_special_2;
-  else
-    return tile_types[ptile->terrain].food;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int get_tile_shield_base(const struct tile *ptile)
-{
-  if (tile_has_special(ptile, S_SPECIAL_1))
-    return tile_types[ptile->terrain].shield_special_1;
-  else if(tile_has_special(ptile, S_SPECIAL_2))
-    return tile_types[ptile->terrain].shield_special_2;
-  else
-    return tile_types[ptile->terrain].shield;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int get_tile_trade_base(const struct tile *ptile)
-{
-  if (tile_has_special(ptile, S_SPECIAL_1))
-    return tile_types[ptile->terrain].trade_special_1;
-  else if (tile_has_special(ptile, S_SPECIAL_2))
-    return tile_types[ptile->terrain].trade_special_2;
-  else
-    return tile_types[ptile->terrain].trade;
-}
-
-/***************************************************************
-  Return a (static) string with special(s) name(s);
-  eg: "Mine"
-  eg: "Road/Farmland"
-***************************************************************/
-const char *map_get_infrastructure_text(enum tile_special_type spe)
-{
-  static char s[64];
-  char *p;
-  
-  s[0] = '\0';
-
-  /* Since railroad requires road, Road/Railroad is redundant */
-  if (contains_special(spe, S_RAILROAD))
-    cat_snprintf(s, sizeof(s), "%s/", _("Railroad"));
-  else if (contains_special(spe, S_ROAD))
-    cat_snprintf(s, sizeof(s), "%s/", _("Road"));
-
-  /* Likewise for farmland on irrigation */
-  if (contains_special(spe, S_FARMLAND))
-    cat_snprintf(s, sizeof(s), "%s/", _("Farmland"));
-  else if (contains_special(spe, S_IRRIGATION))
-    cat_snprintf(s, sizeof(s), "%s/", _("Irrigation"));
-
-  if (contains_special(spe, S_MINE))
-    cat_snprintf(s, sizeof(s), "%s/", _("Mine"));
-
-  if (contains_special(spe, S_FORTRESS))
-    cat_snprintf(s, sizeof(s), "%s/", _("Fortress"));
-
-  if (contains_special(spe, S_AIRBASE))
-    cat_snprintf(s, sizeof(s), "%s/", _("Airbase"));
-
-  p = s + strlen(s) - 1;
-  if (*p == '/')
-    *p = '\0';
-
-  return s;
-}
-
-/****************************************************************************
-  Return the prerequesites needed before building the given infrastructure.
-****************************************************************************/
-enum tile_special_type map_get_infrastructure_prerequisite(enum tile_special_type spe)
-{
-  enum tile_special_type prereq = S_NO_SPECIAL;
-
-  if (contains_special(spe, S_RAILROAD)) {
-    prereq |= S_ROAD;
-  }
-  if (contains_special(spe, S_FARMLAND)) {
-    prereq |= S_IRRIGATION;
-  }
-
-  return prereq;
-}
-
-/***************************************************************
-...
-***************************************************************/
-enum tile_special_type get_preferred_pillage(enum tile_special_type pset)
-{
-  if (contains_special(pset, S_FARMLAND))
-    return S_FARMLAND;
-  if (contains_special(pset, S_IRRIGATION))
-    return S_IRRIGATION;
-  if (contains_special(pset, S_MINE))
-    return S_MINE;
-  if (contains_special(pset, S_FORTRESS))
-    return S_FORTRESS;
-  if (contains_special(pset, S_AIRBASE))
-    return S_AIRBASE;
-  if (contains_special(pset, S_RAILROAD))
-    return S_RAILROAD;
-  if (contains_special(pset, S_ROAD))
-    return S_ROAD;
-  return S_NO_SPECIAL;
+  return count_terrain_flag_near_tile(ptile, FALSE, TRUE,
+				      TER_UNSAFE_COAST) < 100;
 }
 
 /***************************************************************
@@ -845,239 +564,25 @@ enum tile_special_type get_preferred_pillage(enum tile_special_type pset)
 ***************************************************************/
 bool is_water_adjacent_to_tile(const struct tile *ptile)
 {
-  if (is_ocean(ptile->terrain)
-      || tile_has_special(ptile, S_RIVER)
-      || tile_has_special(ptile, S_IRRIGATION))
+  if (ptile->terrain != T_UNKNOWN
+      && (is_ocean(ptile->terrain)
+	  || tile_has_special(ptile, S_RIVER)
+	  || tile_has_special(ptile, S_IRRIGATION))) {
     return TRUE;
+  }
 
   cardinal_adjc_iterate(ptile, tile1) {
-    if (is_ocean(tile1->terrain)
-	|| tile_has_special(tile1, S_RIVER)
-	|| tile_has_special(tile1, S_IRRIGATION))
+    if (ptile->terrain != T_UNKNOWN
+	&& (is_ocean(tile1->terrain)
+	    || tile_has_special(tile1, S_RIVER)
+	    || tile_has_special(tile1, S_IRRIGATION))) {
       return TRUE;
+    }
   } cardinal_adjc_iterate_end;
 
   return FALSE;
 }
 
-/***************************************************************
-...
-***************************************************************/
-int map_build_road_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].road_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_build_irrigation_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].irrigation_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_build_mine_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].mining_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_transform_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].transform_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_build_rail_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].rail_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_build_airbase_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].airbase_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_build_fortress_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].fortress_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_clean_pollution_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].clean_pollution_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-...
-***************************************************************/
-int map_clean_fallout_time(const struct tile *ptile)
-{
-  return tile_types[ptile->terrain].clean_fallout_time * ACTIVITY_FACTOR;
-}
-
-/***************************************************************
-  Time to complete given activity on given tile.
-***************************************************************/
-int map_activity_time(enum unit_activity activity, const struct tile *ptile)
-{
-  switch (activity) {
-  case ACTIVITY_POLLUTION:
-    return map_clean_pollution_time(ptile);
-  case ACTIVITY_ROAD:
-    return map_build_road_time(ptile);
-  case ACTIVITY_MINE:
-    return map_build_mine_time(ptile);
-  case ACTIVITY_IRRIGATE:
-    return map_build_irrigation_time(ptile);
-  case ACTIVITY_FORTRESS:
-    return map_build_fortress_time(ptile);
-  case ACTIVITY_RAILROAD:
-    return map_build_rail_time(ptile);
-  case ACTIVITY_TRANSFORM:
-    return map_transform_time(ptile);
-  case ACTIVITY_AIRBASE:
-    return map_build_airbase_time(ptile);
-  case ACTIVITY_FALLOUT:
-    return map_clean_fallout_time(ptile);
-  default:
-    return 0;
-  }
-}
-
-/***************************************************************
-...
-***************************************************************/
-static void clear_infrastructure(struct tile *ptile)
-{
-  map_clear_special(ptile, S_INFRASTRUCTURE_MASK);
-}
-
-/***************************************************************
-...
-***************************************************************/
-static void clear_dirtiness(struct tile *ptile)
-{
-  map_clear_special(ptile, S_POLLUTION | S_FALLOUT);
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_irrigate_tile(struct tile *ptile)
-{
-  Terrain_type_id now, result;
-  
-  now = ptile->terrain;
-  result = tile_types[now].irrigation_result;
-
-  if (now == result) {
-    if (map_has_special(ptile, S_IRRIGATION)) {
-      map_set_special(ptile, S_FARMLAND);
-    } else {
-      map_set_special(ptile, S_IRRIGATION);
-    }
-  } else if (result != T_NONE) {
-    map_set_terrain(ptile, result);
-    if (is_ocean(result)) {
-      clear_infrastructure(ptile);
-      clear_dirtiness(ptile);
-
-      /* FIXME: When rest of code can handle
-       * rivers in oceans, don't clear this! */
-      map_clear_special(ptile, S_RIVER);
-    }
-    reset_move_costs(ptile);
-  }
-  map_clear_special(ptile, S_MINE);
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_mine_tile(struct tile *ptile)
-{
-  Terrain_type_id now, result;
-  
-  now = ptile->terrain;
-  result = tile_types[now].mining_result;
-  
-  if (now == result) {
-    map_set_special(ptile, S_MINE);
-  } else if (result != T_NONE) {
-    map_set_terrain(ptile, result);
-    if (is_ocean(result)) {
-      clear_infrastructure(ptile);
-      clear_dirtiness(ptile);
-
-      /* FIXME: When rest of code can handle
-       * rivers in oceans, don't clear this! */
-      map_clear_special(ptile, S_RIVER);
-    }
-    reset_move_costs(ptile);
-  }
-  map_clear_special(ptile, S_FARMLAND);
-  map_clear_special(ptile, S_IRRIGATION);
-}
-
-/***************************************************************
-...
-***************************************************************/
-void change_terrain(struct tile *ptile, Terrain_type_id type)
-{
-  map_set_terrain(ptile, type);
-  if (is_ocean(type)) {
-    clear_infrastructure(ptile);
-    clear_dirtiness(ptile);
-    map_clear_special(ptile, S_RIVER);	/* FIXME: When rest of code can handle
-					   rivers in oceans, don't clear this! */
-  }
-
-  reset_move_costs(ptile);
-
-  /* Clear mining/irrigation if resulting terrain type cannot support
-     that feature.  (With current rules, this should only clear mines,
-     but I'm including both cases in the most general form for possible
-     future ruleset expansion. -GJW) */
-  
-  if (tile_types[type].mining_result != type)
-    map_clear_special(ptile, S_MINE);
-
-  if (tile_types[type].irrigation_result != type)
-    map_clear_special(ptile, S_FARMLAND | S_IRRIGATION);
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_transform_tile(struct tile *ptile)
-{
-  Terrain_type_id now, result;
-  
-  now = ptile->terrain;
-  result = tile_types[now].transform_result;
-  
-  if (result != T_NONE) {
-    change_terrain(ptile, result);
-  }
-}
-
 /**************************************************************************
 This function returns true if the tile at the given location can be
 "reclaimed" from ocean into land.  This is the case only when there are
@@ -1115,8 +620,15 @@ static int tile_move_cost_ptrs(struct unit *punit,
 			       const struct tile *t1, const struct tile *t2)
 {
   bool cardinal_move;
+  struct unit_class *pclass = NULL;
+  bool native = TRUE;
 
-  if (game.slow_invasions
+  if (punit) {
+    pclass = unit_class(punit);
+    native = is_native_terrain(punit, t2->terrain);
+  }
+
+  if (game.info.slow_invasions
       && punit 
       && is_ground_unit(punit) 
       && is_ocean(t1->terrain)
@@ -1125,15 +637,28 @@ static int tile_move_cost_ptrs(struct unit *punit,
      * if "slowinvasions" server option is turned on. */
     return punit->moves_left;
   }
-  if (punit && !is_ground_unit(punit))
+
+  if (punit && !pclass->move.terrain_affects) {
     return SINGLE_MOVE;
-  if (tile_has_special(t1, S_RAILROAD) && tile_has_special(t2, S_RAILROAD))
+  }
+
+  /* Railroad check has to be before F_IGTER check so that F_IGTER
+   * units are not penalized. F_IGTER affects also entering and
+   * leaving ships, so F_IGTER check has to be before native terrain
+   * check. We want to give railroad bonus only to native units. */
+  if (tile_has_special(t1, S_RAILROAD) && tile_has_special(t2, S_RAILROAD)
+      && native) {
     return MOVE_COST_RAIL;
-/* return (unit_move_rate(punit)/RAIL_MAX) */
-  if (punit && unit_flag(punit, F_IGTER))
+  }
+  if (punit && unit_has_type_flag(punit, F_IGTER)) {
     return SINGLE_MOVE/3;
-  if (tile_has_special(t1, S_ROAD) && tile_has_special(t2, S_ROAD))
+  }
+  if (!native) {
+    return SINGLE_MOVE;
+  }
+  if (tile_has_special(t1, S_ROAD) && tile_has_special(t2, S_ROAD)) {
     return MOVE_COST_ROAD;
+  }
 
   if (tile_has_special(t1, S_RIVER) && tile_has_special(t2, S_RIVER)) {
     cardinal_move = is_move_cardinal(t1, t2);
@@ -1156,30 +681,34 @@ static int tile_move_cost_ptrs(struct unit *punit,
     }
   }
 
-  return(get_tile_type(t2->terrain)->movement_cost*SINGLE_MOVE);
+  return t2->terrain->movement_cost * SINGLE_MOVE;
 }
 
-/***************************************************************
-  tile_move_cost_ai is used to fill the move_cost array of struct
-  tile. The cached values of this array are used in server/gotohand.c
-  and client/goto.c. tile_move_cost_ai returns the move cost as
+/****************************************************************************
+  map_move_cost_ai returns the move cost as
   calculated by tile_move_cost_ptrs (with no unit pointer to get
   unit-independent results) EXCEPT if either of the source or
   destination tile is an ocean tile. Then the result of the method
   shows if a ship can take the step from the source position to the
   destination position (return value is MOVE_COST_FOR_VALID_SEA_STEP)
-  or not (return value is maxcost).
+  or not.  An arbitrarily high value will be returned if the move is
+  impossible.
 
-  A ship can take the step if:
-    - both tiles are ocean or
-    - one of the tiles is ocean and the other is a city or is unknown
-***************************************************************/
-static int tile_move_cost_ai(struct tile *tile0, struct tile *tile1,
-			     int maxcost)
+  FIXME: this function can't be used for air units because it returns
+  sea<->land moves as impossible.
+****************************************************************************/
+int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1)
 {
+  const int maxcost = 72; /* Arbitrary. */
+
   assert(!is_server
 	 || (tile0->terrain != T_UNKNOWN && tile1->terrain != T_UNKNOWN));
 
+  /* A ship can take the step if:
+   * - both tiles are ocean or
+   * - one of the tiles is ocean and the other is a city or is unknown
+   *
+   * Note tileX->terrain will only be T_UNKNOWN at the client. */
   if (is_ocean(tile0->terrain) && is_ocean(tile1->terrain)) {
     return MOVE_COST_FOR_VALID_SEA_STEP;
   }
@@ -1195,71 +724,15 @@ static int tile_move_cost_ai(struct tile *tile0, struct tile *tile1,
   }
 
   if (is_ocean(tile0->terrain) || is_ocean(tile1->terrain)) {
+    /* FIXME: Shouldn't this return MOVE_COST_FOR_VALID_AIR_STEP?
+     * Note that MOVE_COST_FOR_VALID_AIR_STEP is currently equal to
+     * MOVE_COST_FOR_VALID_SEA_STEP. */
     return maxcost;
   }
 
   return tile_move_cost_ptrs(NULL, tile0, tile1);
 }
 
-/***************************************************************
- ...
-***************************************************************/
-static void debug_log_move_costs(const char *str, struct tile *tile0)
-{
-  /* the %x don't work so well for oceans, where
-     move_cost[]==-3 ,.. --dwp
-  */
-  freelog(LOG_DEBUG, "%s (%d, %d) [%x%x%x%x%x%x%x%x]", str,
-	  tile0->x, tile0->y,
-	  tile0->move_cost[0], tile0->move_cost[1],
-	  tile0->move_cost[2], tile0->move_cost[3],
-	  tile0->move_cost[4], tile0->move_cost[5],
-	  tile0->move_cost[6], tile0->move_cost[7]);
-}
-
-/***************************************************************
-  Recalculate tile->move_cost[] for (x,y), and for adjacent
-  tiles in direction back to (x,y).  That is, call this when
-  something has changed on (x,y), eg roads, city, transform, etc.
-***************************************************************/
-void reset_move_costs(struct tile *ptile)
-{
-  int maxcost = 72; /* should be big enough without being TOO big */
-
-  debug_log_move_costs("Resetting move costs for", ptile);
-
-  /* trying to move off the screen is the default */
-  memset(ptile->move_cost, maxcost, sizeof(ptile->move_cost));
-
-  adjc_dir_iterate(ptile, tile1, dir) {
-    ptile->move_cost[dir] = tile_move_cost_ai(ptile, tile1, maxcost);
-    /* reverse: not at all obfuscated now --dwp */
-    tile1->move_cost[DIR_REVERSE(dir)] =
-	tile_move_cost_ai(tile1, ptile, maxcost);
-  } adjc_dir_iterate_end;
-  debug_log_move_costs("Reset move costs for", ptile);
-}
-
-/***************************************************************
-  Initialize tile->move_cost[] for all tiles, where move_cost[i]
-  is the unit-independent cost to move _from_ that tile, to
-  adjacent tile in direction specified by i.
-***************************************************************/
-void initialize_move_costs(void)
-{
-  int maxcost = 72; /* should be big enough without being TOO big */
-
-  whole_map_iterate(ptile) {
-    /* trying to move off the screen is the default */
-    memset(ptile->move_cost, maxcost, sizeof(ptile->move_cost));
-
-    adjc_dir_iterate(ptile, tile1, dir) {
-      ptile->move_cost[dir] = tile_move_cost_ai(ptile, tile1, maxcost);
-    }
-    adjc_dir_iterate_end;
-  } whole_map_iterate_end;
-}
-
 /***************************************************************
   The cost to move punit from where it is to tile x,y.
   It is assumed the move is a valid one, e.g. the tiles are adjacent.
@@ -1277,131 +750,6 @@ bool is_tiles_adjacent(const struct tile *tile0, const struct tile *tile1)
   return real_map_distance(tile0, tile1) == 1;
 }
 
-/***************************************************************
-...
-***************************************************************/
-Continent_id map_get_continent(const struct tile *ptile)
-{
-  return ptile->continent;
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_set_continent(struct tile *ptile, Continent_id val)
-{
-  ptile->continent = val;
-}
-
-/***************************************************************
-...
-***************************************************************/
-Terrain_type_id map_get_terrain(const struct tile *ptile)
-{
-  return ptile->terrain;
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_set_terrain(struct tile *ptile, Terrain_type_id ter)
-{
-  ptile->terrain = ter;
-}
-
-/***************************************************************
-...
-***************************************************************/
-enum tile_special_type map_get_special(const struct tile *ptile)
-{
-  return ptile->special;
-}
-
-/***************************************************************
- Returns TRUE iff the given special is found at the given map
- position.
-***************************************************************/
-bool map_has_special(const struct tile *ptile, enum tile_special_type special)
-{
-  return contains_special(ptile->special, special);
-}
-
-/***************************************************************
- Returns TRUE iff the given tile has the given special.
-***************************************************************/
-bool tile_has_special(const struct tile *ptile,
-		      enum tile_special_type special)
-{
-  return contains_special(ptile->special, special);
-}
-  
-/***************************************************************
- Returns TRUE iff the given special is found in the given set.
-***************************************************************/
-bool contains_special(enum tile_special_type set,
-		      enum tile_special_type to_test_for)
-{
-  enum tile_special_type masked = set & to_test_for;
-
-  assert(0 == (int) S_NO_SPECIAL);
-
-  /*
-   * contains_special should only be called with one S_* in
-   * to_test_for.
-   */
-  assert(masked == S_NO_SPECIAL || masked == to_test_for);
-
-  return masked == to_test_for;
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_set_special(struct tile *ptile, enum tile_special_type spe)
-{
-  ptile->special |= spe;
-
-  if (contains_special(spe, S_ROAD) || contains_special(spe, S_RAILROAD)) {
-    reset_move_costs(ptile);
-  }
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_clear_special(struct tile *ptile, enum tile_special_type spe)
-{
-  ptile->special &= ~spe;
-
-  if (contains_special(spe, S_ROAD) || contains_special(spe, S_RAILROAD)) {
-    reset_move_costs(ptile);
-  }
-}
-
-/***************************************************************
-  Remove any specials which may exist at these map co-ordinates.
-***************************************************************/
-void map_clear_all_specials(struct tile *ptile)
-{
-  ptile->special = S_NO_SPECIAL;
-}
-
-/***************************************************************
-...
-***************************************************************/
-struct city *map_get_city(const struct tile *ptile)
-{
-  return ptile->city;
-}
-
-/***************************************************************
-...
-***************************************************************/
-void map_set_city(struct tile *ptile, struct city *pcity)
-{
-  ptile->city = pcity;
-}
-
 /***************************************************************
   Are (x1,y1) and (x2,y2) really the same when adjusted?
   This function might be necessary ALOT of places...
@@ -1601,18 +949,18 @@ struct tile *rand_map_pos_filtered(void *data,
 {
   struct tile *ptile;
   int tries = 0;
-  const int max_tries = map.xsize * map.ysize / ACTIVITY_FACTOR;
+  const int max_tries = MAP_INDEX_SIZE / ACTIVITY_FACTOR;
 
   /* First do a few quick checks to find a spot.  The limit on number of
    * tries could use some tweaking. */
   do {
-    ptile = map.tiles + myrand(map.xsize * map.ysize);
+    ptile = map.tiles + myrand(MAP_INDEX_SIZE);
   } while (filter && !filter(ptile, data) && ++tries < max_tries);
 
   /* If that fails, count all available spots and pick one.
    * Slow but reliable. */
   if (tries == max_tries) {
-    int count = 0, positions[map.xsize * map.ysize];
+    int count = 0, positions[MAP_INDEX_SIZE];
 
     whole_map_iterate(ptile) {
       if (filter(ptile, data)) {
