@@ -18,10 +18,7 @@
 
 #include "fc_types.h"
 
-#include "game.h"
-#include "player.h"
-#include "terrain.h"
-#include "unit.h"
+#include "tile.h"
 
 /*
  * The value of MOVE_COST_FOR_VALID_SEA_STEP has no particular
@@ -31,136 +28,11 @@
 #define MOVE_COST_FOR_VALID_SEA_STEP	(-3)
 #define MOVE_COST_FOR_VALID_AIR_STEP	(-3)
 
-/* For client Area Selection */
-enum tile_hilite {
-  HILITE_NONE = 0, HILITE_CITY
-};
-
-/* Convenience macro for accessing tile coordinates.  This should only be
- * used for debugging. */
-#define TILE_XY(ptile) ((ptile) ? (ptile)->x : -1), \
-                       ((ptile) ? (ptile)->y : -1)
-
-struct tile {
-  const int x, y; /* Cartesian (map) coordinates of the tile. */
-  const int nat_x, nat_y; /* Native coordinates of the tile. */
-  const int index; /* Index coordinate of the tile. */
-  Terrain_type_id terrain;
-  enum tile_special_type special;
-  struct city *city;
-  struct unit_list units;
-  unsigned int known;   /* A bitvector on the server side, an
-			   enum known_type on the client side.
-			   Player_no is index */
-  int assigned; /* these can save a lot of CPU usage -- Syela */
-  struct city *worked;      /* city working tile, or NULL if none */
-  Continent_id continent;
-  signed char move_cost[8]; /* don't know if this helps! */
-  struct player *owner;     /* Player owning this tile, or NULL. */
-  struct {
-    /* Area Selection in client. */
-    enum tile_hilite hilite;
-  } client;
-  char *spec_sprite;
-};
-
-
 /****************************************************************
 miscellaneous terrain information
 *****************************************************************/
 #define terrain_misc packet_ruleset_terrain_control
 
-/****************************************************************
-tile_type for each terrain type
-expand with government bonuses??
-*****************************************************************/
-struct tile_type {
-  const char *terrain_name; /* Translated string - doesn't need freeing. */
-  char terrain_name_orig[MAX_LEN_NAME];	/* untranslated copy */
-  char graphic_str[MAX_LEN_NAME];
-  char graphic_alt[MAX_LEN_NAME];
-
-  /* Server-only. */
-  char identifier; /* Single-character identifier used in savegames. */
-#define UNKNOWN_TERRAIN_IDENTIFIER 'u'
-
-  int movement_cost;
-  int defense_bonus;
-
-  int food;
-  int shield;
-  int trade;
-
-  const char *special_1_name; /* Translated string - doesn't need freeing. */
-  char special_1_name_orig[MAX_LEN_NAME]; /* untranslated copy */
-  int food_special_1;
-  int shield_special_1;
-  int trade_special_1;
-
-  const char *special_2_name; /* Translated string - doesn't need freeing. */
-  char special_2_name_orig[MAX_LEN_NAME]; /* untranslated copy */
-  int food_special_2;
-  int shield_special_2;
-  int trade_special_2;
-
-  /* I would put the above special data in this struct too,
-     but don't want to make unnecessary changes right now --dwp */
-  struct {
-    char graphic_str[MAX_LEN_NAME];
-    char graphic_alt[MAX_LEN_NAME];
-  } special[2];
-
-  int road_trade_incr;
-  int road_time;
-
-  Terrain_type_id irrigation_result;
-  int irrigation_food_incr;
-  int irrigation_time;
-
-  Terrain_type_id mining_result;
-  int mining_shield_incr;
-  int mining_time;
-
-  Terrain_type_id transform_result;
-  int transform_time;
-  int rail_time;
-  int airbase_time;
-  int fortress_time;
-  int clean_pollution_time;
-  int clean_fallout_time;
-
-  Terrain_type_id warmer_wetter_result, warmer_drier_result;
-  Terrain_type_id cooler_wetter_result, cooler_drier_result;
-
-  bv_terrain_flags flags;
-
-  char *helptext;
-};
-
-/* The direction8 gives the 8 possible directions.  These may be used in
- * a number of ways, for instance as an index into the DIR_DX/DIR_DY
- * arrays.  Not all directions may be valid; see is_valid_dir and
- * is_cardinal_dir. */
-enum direction8 {
-  /* The DIR8/direction8 naming system is used to avoid conflict with
-   * DIR4/direction4 in client/tilespec.h
-   *
-   * Changing the order of the directions will break network compatability.
-   *
-   * Some code assumes that the first 4 directions are the reverses of the
-   * last 4 (in no particular order).  See client/goto.c. */
-  DIR8_NORTHWEST = 0,
-  DIR8_NORTH = 1,
-  DIR8_NORTHEAST = 2,
-  DIR8_WEST = 3,
-  DIR8_EAST = 4,
-  DIR8_SOUTHWEST = 5,
-  DIR8_SOUTH = 6,
-  DIR8_SOUTHEAST = 7
-};
-#define DIR8_LAST 8
-#define DIR8_COUNT DIR8_LAST
-
 struct civ_map {
   int topology_id;
   enum direction8 valid_dirs[8], cardinal_dirs[8];
@@ -182,9 +54,9 @@ struct civ_map {
   int wetness;
   int steepness;
   int num_start_positions;
-  bool have_specials;
+  bool have_resources;
   bool have_huts;
-  bool have_rivers_overlay;	/* only applies if !have_specials */
+  bool have_rivers_overlay;	/* only applies if !have_resources */
   int num_continents;
   int num_oceans;               /* not updated at the client */
   struct tile *tiles;
@@ -192,7 +64,7 @@ struct civ_map {
   /* Only used by server. */
   struct start_position {
     struct tile *tile;
-    Nation_Type_id nation; /* May be NO_NATION_SELECTED. */
+    struct nation_type *nation; /* May be NO_NATION_SELECTED. */
   } *start_positions;	/* allocated at runtime */
 };
 
@@ -217,8 +89,6 @@ void map_init_topology(bool set_sizes);
 void map_allocate(void);
 void map_free(void);
 
-const char *map_get_tile_info_text(const struct tile *ptile);
-
 int map_vector_to_real_distance(int dx, int dy);
 int map_vector_to_sq_distance(int dx, int dy);
 int map_distance(const struct tile *tile0, const struct tile *tile1);
@@ -232,20 +102,14 @@ bool base_get_direction_for_step(const struct tile *src_tile,
 int get_direction_for_step(const struct tile *src_tile,
 			   const struct tile *dst_tile);
 
-void map_set_continent(struct tile *ptile, Continent_id val);
-Continent_id map_get_continent(const struct tile *ptile);
-
-void initialize_move_costs(void);
-void reset_move_costs(struct tile *ptile);
-
-/* Maximum value of index (for sanity checks and allocations) */
-#define MAX_MAP_INDEX (map.xsize * map.ysize)
+/* Number of index coordinates (for sanity checks and allocations) */
+#define MAP_INDEX_SIZE (map.xsize * map.ysize)
 
 #ifdef DEBUG
 #define CHECK_MAP_POS(x,y) assert(is_normal_map_pos((x),(y)))
 #define CHECK_NATIVE_POS(x, y) assert((x) >= 0 && (x) < map.xsize \
 				      && (y) >= 0 && (y) < map.ysize)
-#define CHECK_INDEX(index) assert((index) >= 0 && (index) < MAX_MAP_INDEX)
+#define CHECK_INDEX(index) assert((index) >= 0 && (index) < MAP_INDEX_SIZE)
 #else
 #define CHECK_MAP_POS(x,y) ((void)0)
 #define CHECK_NATIVE_POS(x, y) ((void)0)
@@ -354,33 +218,9 @@ struct tile *map_pos_to_tile(int x, int y);
 struct tile *native_pos_to_tile(int nat_x, int nat_y);
 struct tile *index_to_tile(int index);
 
-struct player *map_get_owner(const struct tile *ptile);
-void map_set_owner(struct tile *ptile, struct player *pplayer);
-struct city *map_get_city(const struct tile *ptile);
-void map_set_city(struct tile *ptile, struct city *pcity);
-Terrain_type_id map_get_terrain(const struct tile *ptile);
-enum tile_special_type map_get_special(const struct tile *ptile);
-void map_set_terrain(struct tile *ptile, Terrain_type_id ter);
-void map_set_special(struct tile *ptile, enum tile_special_type spe);
-void map_clear_special(struct tile *ptile, enum tile_special_type spe);
-void map_clear_all_specials(struct tile *ptile);
-
 bool is_real_map_pos(int x, int y);
 bool is_normal_map_pos(int x, int y);
 
-/* implemented in server/maphand.c and client/climisc.c */
-enum known_type map_get_known(const struct tile *ptile,
-			      struct player *pplayer);
-
-/* special testing */
-bool map_has_special(const struct tile *ptile,
-		     enum tile_special_type to_test_for);
-bool tile_has_special(const struct tile *ptile,
-		      enum tile_special_type to_test_for);
-bool contains_special(enum tile_special_type all,
-		      enum tile_special_type to_test_for);
-
-
 bool is_singular_tile(const struct tile *ptile, int dist);
 bool normalize_map_pos(int *x, int *y);
 struct tile *nearest_real_tile(int x, int y);
@@ -401,34 +241,11 @@ bool is_tiles_adjacent(const struct tile *ptile0, const struct tile *ptile1);
 bool is_move_cardinal(const struct tile *src_tile,
 		      const struct tile *dst_tile);
 int map_move_cost(struct unit *punit, const struct tile *ptile);
-enum tile_special_type get_special_by_name(const char * name);
-const char *get_special_name(enum tile_special_type type);
+int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1);
 bool is_safe_ocean(const struct tile *ptile);
 bool is_cardinally_adj_to_ocean(const struct tile *ptile);
-bool is_sea_usable(const struct tile *ptile);
-int get_tile_food_base(const struct tile *ptile);
-int get_tile_shield_base(const struct tile *ptile);
-int get_tile_trade_base(const struct tile *ptile);
-enum tile_special_type get_tile_infrastructure_set(const struct tile *ptile);
-const char *map_get_infrastructure_text(enum tile_special_type spe);
-enum tile_special_type map_get_infrastructure_prerequisite(enum tile_special_type spe);
-enum tile_special_type get_preferred_pillage(enum tile_special_type pset);
-
-void map_irrigate_tile(struct tile *ptile);
-void map_mine_tile(struct tile *ptile);
-void change_terrain(struct tile *ptile, Terrain_type_id type);
-void map_transform_tile(struct tile *ptile);
-
-int map_build_road_time(const struct tile *ptile);
-int map_build_irrigation_time(const struct tile *ptile);
-int map_build_mine_time(const struct tile *ptile);
-int map_transform_time(const struct tile *ptile);
-int map_build_rail_time(const struct tile *ptile);
-int map_build_airbase_time(const struct tile *ptile);
-int map_build_fortress_time(const struct tile *ptile);
-int map_clean_pollution_time(const struct tile *ptile);
-int map_clean_fallout_time(const struct tile *ptile);
-int map_activity_time(enum unit_activity activity, const struct tile *ptile);
+bv_special get_tile_infrastructure_set(const struct tile *ptile,
+					  int *count);
 
 bool can_channel_land(const struct tile *ptile);
 bool can_reclaim_ocean(const struct tile *ptile);
@@ -437,6 +254,16 @@ extern struct civ_map map;
 
 extern struct terrain_misc terrain_control;
 
+#define vision_layer_iterate(vision)					    \
+  {									    \
+    enum vision_layer vision;						    \
+									    \
+  for (vision = 0; vision < V_COUNT; vision++) {
+
+#define vision_layer_iterate_end					    \
+  }									    \
+}
+
 /* This iterates outwards from the starting point. Every tile within max_dist
  * will show up exactly once, in an outward (based on real map distance)
  * order.  The returned values are always real and are normalized.  The
@@ -502,16 +329,28 @@ extern struct terrain_misc terrain_control;
  * positions will be automatically discarded. 
  */
 #define circle_iterate(center_tile, sq_radius, tile_itr)		    \
-{                                                                           \
-  int _sq_radius = (sq_radius);						    \
-  int _cr_radius = (int)sqrt((double)_sq_radius);			    \
-  square_dxy_iterate(center_tile, _cr_radius, tile_itr, _dx, _dy) {	    \
-    if (map_vector_to_sq_distance(_dx, _dy) <= _sq_radius) {
+  circle_dxyr_iterate(center_tile, sq_radius, tile_itr, _dx, _dy, _dr)
 
 #define circle_iterate_end                                                  \
-    }                                                                       \
-  } square_dxy_iterate_end;                                                 \
-}									       
+  circle_dxyr_iterate_end
+
+/* dx, dy, dr are distance from center to tile in x, y and square distance;
+ * do not rely on x, y distance, since they do not work for hex topologies */
+#define circle_dxyr_iterate(center_tile, sq_radius,			    \
+			    tile_itr, dx, dy, dr)			    \
+{									    \
+  const int _sq_radius = (sq_radius);					    \
+  const int _cr_radius = (int)sqrt((double)MAX(_sq_radius, 0));		    \
+									    \
+  square_dxy_iterate(center_tile, _cr_radius, tile_itr, dx, dy) {	    \
+    const int dr = map_vector_to_sq_distance(dx, dy);			    \
+									    \
+    if (dr <= _sq_radius) {
+
+#define circle_dxyr_iterate_end						    \
+    }									    \
+  } square_dxy_iterate_end;						    \
+}
 
 /* Iterate through all map positions adjacent to the given center map
  * position, with normalization.  The order of positions is unspecified. */
@@ -577,7 +416,7 @@ extern struct terrain_misc terrain_control;
 #define whole_map_iterate(ptile)					    \
 {                                                                           \
   int _index; /* We use index positions for cache efficiency. */	    \
-  for (_index = 0; _index < MAX_MAP_INDEX; _index++) {			    \
+  for (_index = 0; _index < MAP_INDEX_SIZE; _index++) {			    \
     struct tile *ptile = map.tiles + _index;				    \
 
 #define whole_map_iterate_end                                               \
@@ -586,12 +425,6 @@ extern struct terrain_misc terrain_control;
 
 BV_DEFINE(dir_vector, 8);
 
-struct unit_order {
-  enum unit_orders order;
-  enum direction8 dir;		/* Only valid for ORDER_MOVE. */
-  enum unit_activity activity;	/* Only valid for ORDER_ACTIVITY. */
-};
-
 /* return the reverse of the direction */
 #define DIR_REVERSE(dir) (7 - (dir))
 
@@ -707,9 +540,4 @@ static inline bool is_border_tile(const struct tile *ptile, int dist)
 	  || ptile->nat_y >= map.ysize - ydist);
 }
 
-/* An arbitrary somewhat integer value.  Activity times are multiplied by
- * this amount, and divided by them later before being used.  This may
- * help to avoid rounding errors; however it should probably be removed. */
-#define ACTIVITY_FACTOR 10
-
 #endif  /* FC__MAP_H */
