@@ -649,7 +649,8 @@ void send_attribute_block(const struct player *pplayer,
     if (packet.chunk_length < ATTRIBUTE_CHUNK_SIZE) {
       /* Last chunk is not full. Make sure that delta does
        * not use random data. */
-      memset(packet.data, 0, ATTRIBUTE_CHUNK_SIZE - packet.chunk_length);
+      memset(packet.data + packet.chunk_length, 0,
+             ATTRIBUTE_CHUNK_SIZE - packet.chunk_length);
     }
 
     send_packet_player_attribute_chunk(pconn, &packet);
