@@ -22,25 +22,31 @@ struct data_in;
 #include "improvement.h"
 #include "map.h"
 #include "player.h"
+#include "requirements.h"
 #include "shared.h"		/* MAX_LEN_NAME, MAX_LEN_ADDR */
 #include "spaceship.h"
+#include "team.h"
 #include "unittype.h"
 #include "worklist.h"
 
 
 #define MAX_LEN_USERNAME        10        /* see below */
 #define MAX_LEN_MSG             1536
-#define MAX_ATTRIBUTE_BLOCK     (256*1024)	/* largest attribute block */
-#define ATTRIBUTE_CHUNK_SIZE    (1024*2)  /* attribute chunk size to use */
 #define MAX_LEN_ROUTE		2000	  /* MAX_LEN_PACKET/2 - header */
 
+/* The size of opaque (void *) data sent in the network packet.  To avoid
+ * fragmentation issues, this SHOULD NOT be larger than the standard
+ * ethernet or PPP 1500 byte frame size (with room for headers).
+ *
+ * Do not spend much time optimizing, you have no idea of the actual dynamic
+ * path characteristics between systems, such as VPNs and tunnels.
+ */
+#define ATTRIBUTE_CHUNK_SIZE    (1400)
+
 enum report_type {
   REPORT_WONDERS_OF_THE_WORLD,
   REPORT_TOP_5_CITIES,
-  REPORT_DEMOGRAPHIC,
-  REPORT_SERVER_OPTIONS,   /* obsolete */
-  REPORT_SERVER_OPTIONS1,
-  REPORT_SERVER_OPTIONS2
+  REPORT_DEMOGRAPHIC
 };
 
 enum spaceship_place_type {
@@ -85,9 +91,6 @@ void post_receive_packet_chat_msg(struct connection *pc,
 void pre_send_packet_player_attribute_chunk(struct connection *pc,
 					    struct packet_player_attribute_chunk
 					    *packet);
-void post_receive_packet_player_attribute_chunk(struct connection *pc, struct packet_player_attribute_chunk
-						*packet);
-
 void post_receive_packet_game_state(struct connection *pc,
 				    struct packet_game_state *packet);
 void post_send_packet_game_state(struct connection *pc,
