@@ -35,11 +35,13 @@ struct advance advances[A_LAST];
    server/ruleset.c (for the server)
    client/packhand.c (for the client) */
 
-static int techcoststyle1[A_LAST];
+/* Precalculated costs according to techcost style 1.  These do not include
+ * the sciencebox multiplier. */
+static double techcoststyle1[A_LAST];
 
 static const char *flag_names[] = {
-  "Bonus_Tech", "Boat_Fast", "Bridge", "Railroad", "Fortress",
-  "Watchtower", "Population_Pollution_Inc", "Trade_Revenue_Reduce",
+  "Bonus_Tech", "Bridge", "Railroad", "Fortress",
+  "Population_Pollution_Inc", 
   "Airbase", "Farmland", "Reduce_Trireme_Loss1", "Reduce_Trireme_Loss2", 
   "Build_Airborne"
 };
@@ -47,73 +49,105 @@ static const char *flag_names[] = {
    in common/tech.h */
 
 /**************************************************************************
-...
+  Return the advance for the given advance index.
+**************************************************************************/
+struct advance *advance_by_number(const Tech_type_id atype)
+{
+  if (atype < 0 || atype >= game.control.num_tech_types) {
+    /* This isn't an error; some callers depend on it. */
+    return NULL;
+  }
+  return &advances[atype];
+}
+
+/**************************************************************************
+  Returns state of the tech for current pplayer.
+  This can be: TECH_KNOW, TECH_UNKNOWN or TECH_REACHABLE
+  Should be called with existing techs or A_FUTURE
+
+  If pplayer is NULL this checks whether any player knows the tech (used
+  by the client).
 **************************************************************************/
 enum tech_state get_invention(const struct player *pplayer,
-			      Tech_Type_id tech)
+			      Tech_type_id tech)
 {
-  assert(tech >= 0 || tech < game.num_tech_types);
+  assert(tech == A_FUTURE
+         || (tech >= 0 && tech < game.control.num_tech_types));
 
-  return pplayer->research.inventions[tech].state;
+  if (!pplayer) {
+    if (tech != A_FUTURE && game.info.global_advances[tech]) {
+      return TECH_KNOWN;
+    } else {
+      return TECH_UNKNOWN;
+    }
+  } else {
+    return get_player_research(pplayer)->inventions[tech].state;
+  }
 }
 
 /**************************************************************************
 ...
 **************************************************************************/
-void set_invention(struct player *pplayer, Tech_Type_id tech,
+void set_invention(struct player *pplayer, Tech_type_id tech,
 		   enum tech_state value)
 {
-  if (pplayer->research.inventions[tech].state == value) {
+  struct player_research *research = get_player_research(pplayer);
+
+  if (research->inventions[tech].state == value) {
     return;
   }
 
-  pplayer->research.inventions[tech].state = value;
+  research->inventions[tech].state = value;
 
   if (value == TECH_KNOWN) {
-    game.global_advances[tech]++;
-    improvements_update_obsolete();
+    game.info.global_advances[tech] = TRUE;
   }
 }
 
 /**************************************************************************
   Returns if the given tech has to be researched to reach the
   goal. The goal itself isn't a requirement of itself.
+
+  pplayer may be NULL; however the function will always return FALSE in
+  that case.
 **************************************************************************/
-bool is_tech_a_req_for_goal(struct player *pplayer, Tech_Type_id tech,
-			    Tech_Type_id goal)
+bool is_tech_a_req_for_goal(const struct player *pplayer, Tech_type_id tech,
+			    Tech_type_id goal)
 {
   if (tech == goal) {
     return FALSE;
+  } else if (!pplayer) {
+    /* FIXME: We need a proper implementation here! */
+    return FALSE;
   } else {
-    return BV_ISSET(pplayer->research.inventions[goal].required_techs, tech);
+    return
+      BV_ISSET(get_player_research(pplayer)->inventions[goal].required_techs,
+               tech);
   }
 }
 
 /**************************************************************************
   Marks all techs which are requirements for goal in
-  pplayer->research.inventions[goal].required_techs. Works recursive.
+  pplayer->research->inventions[goal].required_techs. Works recursive.
 **************************************************************************/
 static void build_required_techs_helper(struct player *pplayer,
-					Tech_Type_id tech,
-					Tech_Type_id goal)
+					Tech_type_id tech,
+					Tech_type_id goal)
 {
   /* The is_tech_a_req_for_goal condition is true if the tech is
    * already marked */
   if (!tech_is_available(pplayer, tech)
       || get_invention(pplayer, tech) == TECH_KNOWN
-      || is_tech_a_req_for_goal(pplayer, tech, goal)
-      || tech == A_NONE) {
+      || is_tech_a_req_for_goal(pplayer, tech, goal)) {
     return;
   }
 
   /* Mark the tech as required for the goal */
-  BV_SET(pplayer->research.inventions[goal].required_techs, tech);
+  BV_SET(get_player_research(pplayer)->inventions[goal].required_techs, tech);
 
   if (advances[tech].req[0] == goal || advances[tech].req[1] == goal) {
-    /* TRANS: Obscure ruleset error */
-    freelog(LOG_FATAL, _("tech \"%s\": requires itself"),
-	    get_tech_name(pplayer, goal));
-    assert(0);
+    freelog(LOG_FATAL, "tech \"%s\": requires itself",
+	    advance_name_by_player(pplayer, goal));
     exit(EXIT_FAILURE);
   }
 
@@ -123,26 +157,27 @@ static void build_required_techs_helper(struct player *pplayer,
 
 /**************************************************************************
   Updates required_techs, num_required_techs and bulbs_required in
-  pplayer->research.inventions[goal].
+  pplayer->research->inventions[goal].
 **************************************************************************/
-static void build_required_techs(struct player *pplayer, Tech_Type_id goal)
+static void build_required_techs(struct player *pplayer, Tech_type_id goal)
 {
   int counter;
+  struct player_research *research = get_player_research(pplayer);
 
-  BV_CLR_ALL(pplayer->research.inventions[goal].required_techs);
-
+  BV_CLR_ALL(research->inventions[goal].required_techs);
+  
   if (get_invention(pplayer, goal) == TECH_KNOWN) {
-    pplayer->research.inventions[goal].num_required_techs = 0;
-    pplayer->research.inventions[goal].bulbs_required = 0;
+    research->inventions[goal].num_required_techs = 0;
+    research->inventions[goal].bulbs_required = 0;
     return;
   }
   
   build_required_techs_helper(pplayer, goal, goal);
 
   /* Include the goal tech */
-  pplayer->research.inventions[goal].bulbs_required =
+  research->inventions[goal].bulbs_required =
       base_total_bulbs_required(pplayer, goal);
-  pplayer->research.inventions[goal].num_required_techs = 1;
+  research->inventions[goal].num_required_techs = 1;
 
   counter = 0;
   tech_type_iterate(i) {
@@ -154,23 +189,26 @@ static void build_required_techs(struct player *pplayer, Tech_Type_id goal)
      * This is needed to get a correct result for the
      * base_total_bulbs_required call.
      */
-    pplayer->research.techs_researched++;
+    research->techs_researched++;
     counter++;
 
-    pplayer->research.inventions[goal].num_required_techs++;
-    pplayer->research.inventions[goal].bulbs_required +=
+    research->inventions[goal].num_required_techs++;
+    research->inventions[goal].bulbs_required +=
 	base_total_bulbs_required(pplayer, i);
   } tech_type_iterate_end;
 
   /* Undo the changes made above */
-  pplayer->research.techs_researched -= counter;
+  research->techs_researched -= counter;
 }
 
 /**************************************************************************
   Returns TRUE iff the given tech is ever reachable by the given player
   by checking tech tree limitations.
+
+  pplayer may be NULL in which case a simplified result is returned
+  (used by the client).
 **************************************************************************/
-bool tech_is_available(struct player *pplayer, Tech_Type_id id)
+bool tech_is_available(const struct player *pplayer, Tech_type_id id)
 {
   if (!tech_exists(id)) {
     return FALSE;
@@ -186,12 +224,17 @@ bool tech_is_available(struct player *pplayer, Tech_Type_id id)
 }
 
 /**************************************************************************
-  Marks reachable techs. Calls build_required_techs to update the
-  cache of requirements.
+  Mark as TECH_REACHABLE each tech which is available, not known and
+  which has all requirements fullfiled.
+  If there is no such a tech mark A_FUTURE as researchable.
+  
+  Recalculate research->num_known_tech_with_flag
+  Should be called always after set_invention()
 **************************************************************************/
 void update_research(struct player *pplayer)
 {
   enum tech_flag_id flag;
+  int researchable = 0;
 
   tech_type_iterate(i) {
     if (i == A_NONE) {
@@ -209,29 +252,35 @@ void update_research(struct player *pplayer)
 	  && get_invention(pplayer, advances[i].req[0]) == TECH_KNOWN
 	  && get_invention(pplayer, advances[i].req[1]) == TECH_KNOWN) {
 	set_invention(pplayer, i, TECH_REACHABLE);
+	researchable++;
       }
     }
     build_required_techs(pplayer, i);
   } tech_type_iterate_end;
+  
+  /* No techs we can research? Mark A_FUTURE as researchable */
+  if (researchable == 0) {
+    set_invention(pplayer, A_FUTURE, TECH_REACHABLE);
+  }
 
   for (flag = 0; flag < TF_LAST; flag++) {
-    pplayer->research.num_known_tech_with_flag[flag] = 0;
+    get_player_research(pplayer)->num_known_tech_with_flag[flag] = 0;
 
     tech_type_iterate(i) {
-      if (get_invention(pplayer, i) == TECH_KNOWN && tech_flag(i, flag)) {
-	pplayer->research.num_known_tech_with_flag[flag]++;
+      if (get_invention(pplayer, i) == TECH_KNOWN && advance_has_flag(i, flag)) {
+	get_player_research(pplayer)->num_known_tech_with_flag[flag]++;
       }
     } tech_type_iterate_end;
   }
 }
 
 /**************************************************************************
-...don't use this function directly, call get_next_tech instead.
+  Return the next tech we should research to advance towards our goal.
+  Returns A_UNSET if nothing is available or the goal is already known.
 **************************************************************************/
-static Tech_Type_id get_next_tech_rec(struct player *pplayer,
-				      Tech_Type_id goal)
+Tech_type_id get_next_tech(const struct player *pplayer, Tech_type_id goal)
 {
-  Tech_Type_id sub_goal;
+  Tech_type_id sub_goal;
 
   if (!tech_is_available(pplayer, goal)
       || get_invention(pplayer, goal) == TECH_KNOWN) {
@@ -240,29 +289,14 @@ static Tech_Type_id get_next_tech_rec(struct player *pplayer,
   if (get_invention(pplayer, goal) == TECH_REACHABLE) {
     return goal;
   }
-  sub_goal = get_next_tech_rec(pplayer, advances[goal].req[0]);
+  sub_goal = get_next_tech(pplayer, advances[goal].req[0]);
   if (sub_goal != A_UNSET) {
     return sub_goal;
   } else {
-    return get_next_tech_rec(pplayer, advances[goal].req[1]);
+    return get_next_tech(pplayer, advances[goal].req[1]);
   }
 }
 
-/**************************************************************************
-... this could be simpler, but we might have or get loops in the tech tree
-    so i try to avoid endless loops.
-    if return value > A_LAST then we have a bug
-    caller should do something in that case.
-**************************************************************************/
-Tech_Type_id get_next_tech(struct player *pplayer, Tech_Type_id goal)
-{
-  if (!tech_is_available(pplayer, goal)
-      || get_invention(pplayer, goal) == TECH_KNOWN) {
-    return A_UNSET;
-  }
-  return (get_next_tech_rec(pplayer, goal));
-}
-
 /**************************************************************************
 Returns 1 if the tech "exists" in this game, 0 otherwise.
 A tech doesn't exist if one of:
@@ -271,9 +305,9 @@ A tech doesn't exist if one of:
   to A_LAST (this function returns 0 if either req is A_LAST, rather
   than both, to be on the safe side)
 **************************************************************************/
-bool tech_exists(Tech_Type_id id)
+bool tech_exists(Tech_type_id id)
 {
-  if (id < 0 || id >= game.num_tech_types) {
+  if (id < 0 || id >= game.control.num_tech_types) {
     return FALSE;
   } else {
     return advances[id].req[0] != A_LAST && advances[id].req[1] != A_LAST;
@@ -281,13 +315,13 @@ bool tech_exists(Tech_Type_id id)
 }
 
 /**************************************************************************
-Does a linear search of advances[].name
-Returns A_LAST if none match.
+ Does a linear search of advances[].name.translated
+ Returns A_LAST if none match.
 **************************************************************************/
-Tech_Type_id find_tech_by_name(const char *s)
+Tech_type_id find_advance_by_translated_name(const char *s)
 {
   tech_type_iterate(i) {
-    if (strcmp(advances[i].name, s) == 0) {
+    if (0 == strcmp(advance_name_translation(i), s)) {
       return i;
     }
   } tech_type_iterate_end;
@@ -295,13 +329,15 @@ Tech_Type_id find_tech_by_name(const char *s)
 }
 
 /**************************************************************************
-  Does a linear search of advances[].name_orig
-  Returns A_LAST if none match.
+ Does a linear search of advances[].name.vernacular
+ Returns A_LAST if none match.
 **************************************************************************/
-Tech_Type_id find_tech_by_name_orig(const char *s)
+Tech_type_id find_advance_by_rule_name(const char *s)
 {
+  const char *qs = Qn_(s);
+
   tech_type_iterate(i) {
-    if (mystrcasecmp(advances[i].name_orig, s) == 0) {
+    if (0 == mystrcasecmp(advance_rule_name(i), qs)) {
       return i;
     }
   } tech_type_iterate_end;
@@ -311,7 +347,7 @@ Tech_Type_id find_tech_by_name_orig(const char *s)
 /**************************************************************************
  Return TRUE if the tech has this flag otherwise FALSE
 **************************************************************************/
-bool tech_flag(Tech_Type_id tech, enum tech_flag_id flag)
+bool advance_has_flag(Tech_type_id tech, enum tech_flag_id flag)
 {
   assert(flag >= 0 && flag < TF_LAST);
   return TEST_BIT(advances[tech].flags, flag);
@@ -321,7 +357,7 @@ bool tech_flag(Tech_Type_id tech, enum tech_flag_id flag)
  Convert flag names to enum; case insensitive;
  returns TF_LAST if can't match.
 **************************************************************************/
-enum tech_flag_id tech_flag_from_str(const char *s)
+enum tech_flag_id find_advance_flag_by_rule_name(const char *s)
 {
   enum tech_flag_id i;
 
@@ -339,12 +375,12 @@ enum tech_flag_id tech_flag_from_str(const char *s)
  Search for a tech with a given flag starting at index
  Returns A_LAST if no tech has been found
 **************************************************************************/
-Tech_Type_id find_tech_by_flag(int index, enum tech_flag_id flag)
+Tech_type_id find_advance_by_flag(int index, enum tech_flag_id flag)
 {
-  Tech_Type_id i;
-  for(i=index;i<game.num_tech_types;i++)
+  Tech_type_id i;
+  for(i = index;i < game.control.num_tech_types; i++)
   {
-    if(tech_flag(i,flag)) return i;
+    if(advance_has_flag(i,flag)) return i;
   }
   return A_LAST;
 }
@@ -352,43 +388,53 @@ Tech_Type_id find_tech_by_flag(int index, enum tech_flag_id flag)
 /**************************************************************************
   Returns the number of bulbs which are required to finished the
   currently researched tech denoted by
-  pplayer->research.researching. This is _NOT_ the number of bulbs
+  pplayer->research->researching. This is _NOT_ the number of bulbs
   which are left to get the advance. Use the term
-  "total_bulbs_required(pplayer) - pplayer->research.bulbs_researched"
+  "total_bulbs_required(pplayer) - pplayer->research->bulbs_researched"
   if you want this.
 **************************************************************************/
-int total_bulbs_required(struct player *pplayer)
+int total_bulbs_required(const struct player *pplayer)
 {
-  return base_total_bulbs_required(pplayer, pplayer->research.researching);
+  return base_total_bulbs_required(pplayer,
+    get_player_research(pplayer)->researching);
 }
 
-/**************************************************************************
- Function to determine cost for technology. Equation is determined
- from game.rgame.tech_cost_style and game.rgame.tech_leakage.
+/****************************************************************************
+  Function to determine cost for technology.  The equation is determined
+  from game.info.tech_cost_style and game.info.tech_leakage.
 
- tech_cost_style:
- 0 - Civ (I|II) style. Every new tech add game.researchcost to the
-     cost of the next tech.
- 1 - Cost of technology is 
-       MAX((1 + parents) * (researchcost / 2) * sqrt(1 + parents), researchcost)
-     where num_parents == number of requirement for tech, counted recursively.
- 2 - Cost are read from tech.ruleset. Missing costs are generated by
-     style 1.
+  tech_cost_style:
+  0 - Civ (I|II) style. Every new tech adds N to the cost of the next tech.
+  1 - Cost of technology is 
+        (1 + parents) * 10 * sqrt(1 + parents)
+      where num_parents == number of requirement for tech (recursive).
+  2 - Cost are read from tech.ruleset. Missing costs are generated by
+      style 1.
 
- tech_leakage:
- 0 - No reduction of the technology cost.
- 1 - Technology cost is reduced depending on the number of players
-     which already know the tech and you have an embassy with.
- 2 - Technology cost is reduced depending on the number of all players
-     (human, AI and barbarians) which already know the tech.
- 3 - Technology cost is reduced depending on the number of normal
-     players (human and AI) which already know the tech.
-**************************************************************************/
-int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
+  tech_leakage:
+  0 - No reduction of the technology cost.
+  1 - Technology cost is reduced depending on the number of players
+      which already know the tech and you have an embassy with.
+  2 - Technology cost is reduced depending on the number of all players
+      (human, AI and barbarians) which already know the tech.
+  3 - Technology cost is reduced depending on the number of normal
+      players (human and AI) which already know the tech.
+
+  At the end we multiply by the sciencebox value, as a percentage.  The
+  cost can never be less than 1.
+
+  pplayer may be NULL in which case a simplified result is returned (used
+  by client and manual code).
+****************************************************************************/
+int base_total_bulbs_required(const struct player *pplayer,
+			      Tech_type_id tech)
 {
-  int cost, tech_cost_style = game.rgame.tech_cost_style;
+  int tech_cost_style = game.info.tech_cost_style;
+  double base_cost;
 
-  if (!is_future_tech(tech) && get_invention(pplayer, tech) == TECH_KNOWN) {
+  if (pplayer
+      && !is_future_tech(tech)
+      && get_invention(pplayer, tech) == TECH_KNOWN) {
     /* A non-future tech which is already known costs nothing. */
     return 0;
   }
@@ -405,28 +451,32 @@ int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
 
   switch (tech_cost_style) {
   case 0:
-    cost = pplayer->research.techs_researched * game.researchcost;
+    if (pplayer) {
+      base_cost = get_player_research(pplayer)->techs_researched 
+	* game.info.base_tech_cost;
+    } else {
+      base_cost = 0;
+    }
     break;
   case 1:
-    cost = techcoststyle1[tech];
+    base_cost = techcoststyle1[tech];
     break;
   case 2:
-    cost = (advances[tech].preset_cost * game.researchcost) /
-	GAME_DEFAULT_RESEARCHCOST;
+    base_cost = advances[tech].preset_cost;
     break;
   default:
-    die("Invalid tech_cost_style %d %d", game.rgame.tech_cost_style,
+    die("Invalid tech_cost_style %d %d", game.info.tech_cost_style,
 	tech_cost_style);
-    cost = 0;
+    base_cost = 0.0;
   }
 
   /* Research becomes more expensive this year and after. */
-  if (game.rgame.tech_cost_double_year != 0
-      && game.year >= game.rgame.tech_cost_double_year) {
-    cost *= 2;
+  if (game.info.tech_cost_double_year != 0
+      && game.info.year >= game.info.tech_cost_double_year) {
+    base_cost *= 2.0;
   }
 
-  switch (game.rgame.tech_leakage) {
+  switch (game.info.tech_leakage) {
   case 0:
     /* no change */
     break;
@@ -438,12 +488,13 @@ int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
       players_iterate(other) {
 	players++;
 	if (get_invention(other, tech) == TECH_KNOWN
-	    && player_has_embassy(pplayer, other)) {
+	    && pplayer && player_has_embassy(pplayer, other)) {
 	  players_with_tech_and_embassy++;
 	}
       } players_iterate_end;
 
-      cost = ((players - players_with_tech_and_embassy) * cost) / players;
+      base_cost *= (double)(players - players_with_tech_and_embassy);
+      base_cost /= (double)players;
     }
     break;
 
@@ -458,7 +509,8 @@ int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
 	}
       } players_iterate_end;
 
-      cost = ((players - players_with_tech) * cost) / players;
+      base_cost *= (double)(players - players_with_tech);
+      base_cost /= (double)players;
     }
     break;
 
@@ -476,59 +528,68 @@ int base_total_bulbs_required(struct player *pplayer, Tech_Type_id tech)
 	}
       } players_iterate_end;
 
-      cost = ((players - players_with_tech) * cost) / players;
+      base_cost *= (double)(players - players_with_tech);
+      base_cost /= (double)players;
     }
     break;
 
   default:
-    die("Invalid tech_leakage %d", game.rgame.tech_leakage);
+    die("Invalid tech_leakage %d", game.info.tech_leakage);
   }
 
   /* Assign a science penalty to the AI at easier skill levels.  This code
    * can also be adpoted to create an extra-hard AI skill level where the AI
    * gets science benefits */
 
-  /* FIXME: the is_server is a kludge to unbreak CVS. GB promises 
-   * to fix correctly a.s.a.p. 2003/5/18 -mck */
-  if (is_server && pplayer->ai.control) {
+  if (pplayer && pplayer->ai.control) {
     assert(pplayer->ai.science_cost > 0);
-    cost = (cost * pplayer->ai.science_cost) / 100;
+    base_cost *= (double)pplayer->ai.science_cost / 100.0;
   }
 
-  /* If we have many players, tech cost may drop to 0.  */
-  if (cost == 0) {
-    cost = 1;
-  }
+  base_cost *= (double)game.info.sciencebox / 100.0;
 
-  return cost;
+  return MAX(base_cost, 1);
 }
 
 /**************************************************************************
  Returns the number of technologies the player need to research to get
  the goal technology. This includes the goal technology. Technologies
  are only counted once.
+
+  pplayer may be NULL; however the wrong value will be return in this case.
 **************************************************************************/
-int num_unknown_techs_for_goal(struct player *pplayer, Tech_Type_id goal)
+int num_unknown_techs_for_goal(const struct player *pplayer,
+			       Tech_type_id goal)
 {
-  return pplayer->research.inventions[goal].num_required_techs;
+  if (!pplayer) {
+    /* FIXME: need an implementation for this! */
+    return 0;
+  }
+  return get_player_research(pplayer)->inventions[goal].num_required_techs;
 }
 
 /**************************************************************************
  Function to determine cost (in bulbs) of reaching goal
  technology. These costs _include_ the cost for researching the goal
  technology itself.
+
+  pplayer may be NULL; however the wrong value will be return in this case.
 **************************************************************************/
-int total_bulbs_required_for_goal(struct player *pplayer,
-				  Tech_Type_id goal)
+int total_bulbs_required_for_goal(const struct player *pplayer,
+				  Tech_type_id goal)
 {
-  return pplayer->research.inventions[goal].bulbs_required;
+  if (!pplayer) {
+    /* FIXME: need an implementation for this! */
+    return 0;
+  }
+  return get_player_research(pplayer)->inventions[goal].bulbs_required;
 }
 
 /**************************************************************************
  Returns number of requirements for the given tech. To not count techs
  double a memory (the counted array) is needed.
 **************************************************************************/
-static int precalc_tech_data_helper(Tech_Type_id tech, bool *counted)
+static int precalc_tech_data_helper(Tech_type_id tech, bool *counted)
 {
   if (tech == A_NONE || !tech_exists(tech) || counted[tech]) {
     return 0;
@@ -554,17 +615,18 @@ void precalc_tech_data()
   } tech_type_iterate_end;
 
   tech_type_iterate(tech) {
-    techcoststyle1[tech] = MAX((advances[tech].num_reqs + 1)
-                               * sqrt(advances[tech].num_reqs + 1)
-                               * (game.researchcost / 2),
-                               game.researchcost);
+    double reqs = advances[tech].num_reqs + 1;
+    const double base = game.info.base_tech_cost / 2;
+    const double cost = base * reqs * sqrt(reqs);
+
+    techcoststyle1[tech] = MAX(cost, game.info.base_tech_cost);
   } tech_type_iterate_end;
 }
 
 /**************************************************************************
  Is the given tech a future tech.
 **************************************************************************/
-bool is_future_tech(Tech_Type_id tech)
+bool is_future_tech(Tech_type_id tech)
 {
   return tech == A_FUTURE;
 }
@@ -574,47 +636,123 @@ bool is_future_tech(Tech_Type_id tech)
 #include "specvec.h"
 
 /**************************************************************************
- Return the name of the given tech. You don't have to free the return
- pointer.
-**************************************************************************/
-const char *get_tech_name(struct player *pplayer, Tech_Type_id tech)
-{
-  static struct string_vector future;
-  int i;
+  Return the rule name of the given tech (including A_FUTURE). 
+  You don't have to free the return pointer.
 
+  pplayer may be NULL.
+**************************************************************************/
+const char *advance_name_by_player(const struct player *pplayer, Tech_type_id tech)
+{
   /* We don't return a static buffer because that would break anything that
    * needed to work with more than one name at a time. */
+  static struct string_vector future;
+
   switch (tech) {
-  case A_NOINFO:
-    /* TRANS: "Unknown" tech */
-    return _("(Unknown)");
-  case A_UNSET:
-    /* TRANS: "None" tech */
-    return _("None");
   case A_FUTURE:
-    /* pplayer->future_tech == 0 means "Future Tech. 1". */
-    for (i = future.size; i <= pplayer->future_tech; i++) {
-      char *ptr = NULL;
-
-      string_vector_append(&future, &ptr);
+    if (pplayer) {
+      struct player_research *research = get_player_research(pplayer);
+      int i;
+  
+      /* pplayer->future_tech == 0 means "Future Tech. 1". */
+      for (i = future.size; i <= research->future_tech; i++) {
+        char *ptr = NULL;
+  
+        string_vector_append(&future, &ptr);
+      }
+      if (!future.p[research->future_tech]) {
+        char buffer[1024];
+  
+        my_snprintf(buffer, sizeof(buffer), "%s %d",
+                    advance_rule_name(tech),
+                    research->future_tech + 1);
+        future.p[research->future_tech] = mystrdup(buffer);
+      }
+      return future.p[research->future_tech];
+    } else {
+      return advance_rule_name(tech);
     }
-    if (!future.p[pplayer->future_tech]) {
-      char buffer[1024];
-
-      my_snprintf(buffer, sizeof(buffer), _("Future Tech. %d"),
-		  pplayer->future_tech + 1);
-      future.p[pplayer->future_tech] = mystrdup(buffer);
-    }
-    return future.p[pplayer->future_tech];
   default:
     /* Includes A_NONE */
-    if (!tech_exists(tech)) {
-      /* TRANS: "Unknown" tech */
-      return _("(Unknown)");
+    return advance_rule_name(tech);
+  };
+}
+
+/**************************************************************************
+  Return the translated name of the given tech (including A_FUTURE). 
+  You don't have to free the return pointer.
+
+  pplayer may be NULL.
+**************************************************************************/
+const char *advance_name_for_player(const struct player *pplayer, Tech_type_id tech)
+{
+  /* We don't return a static buffer because that would break anything that
+   * needed to work with more than one name at a time. */
+  static struct string_vector future;
+
+  switch (tech) {
+  case A_FUTURE:
+    if (pplayer) {
+      struct player_research *research = get_player_research(pplayer);
+      int i;
+  
+      /* pplayer->future_tech == 0 means "Future Tech. 1". */
+      for (i = future.size; i <= research->future_tech; i++) {
+        char *ptr = NULL;
+  
+        string_vector_append(&future, &ptr);
+      }
+      if (!future.p[research->future_tech]) {
+        char buffer[1024];
+  
+        my_snprintf(buffer, sizeof(buffer), _("Future Tech. %d"),
+                    research->future_tech + 1);
+        future.p[research->future_tech] = mystrdup(buffer);
+      }
+      return future.p[research->future_tech];
     } else {
-      return advances[tech].name;
+      return advance_name_translation(tech);
     }
+  default:
+    /* Includes A_NONE */
+    return advance_name_translation(tech);
+  };
+}
+
+/**************************************************************************
+  Return the translated name of the given research (including A_FUTURE). 
+  You don't have to free the return pointer.
+
+  pplayer must not be NULL.
+**************************************************************************/
+const char *advance_name_researching(const struct player *pplayer)
+{
+  return advance_name_for_player(pplayer,
+    get_player_research(pplayer)->researching);
+}
+
+/**************************************************************************
+  Return the (translated) name of the given advance/technology.
+  You don't have to free the return pointer.
+**************************************************************************/
+const char *advance_name_translation(Tech_type_id tech)
+{
+/*  assert(tech_exists(tech)); now called for A_UNSET, A_FUTURE */
+  if (NULL == advances[tech].name.translated) {
+    /* delayed (unified) translation */
+    advances[tech].name.translated = ('\0' == advances[tech].name.vernacular[0])
+				     ? advances[tech].name.vernacular
+				     : Q_(advances[tech].name.vernacular);
   }
+  return advances[tech].name.translated;
+}
+
+/****************************************************************************
+  Return the (untranslated) rule name of the advance/technology.
+  You don't have to free the return pointer.
+****************************************************************************/
+const char *advance_rule_name(Tech_type_id tech)
+{
+  return Qn_(advances[tech].name.vernacular); 
 }
 
 /**************************************************************************
@@ -623,15 +761,45 @@ const char *get_tech_name(struct player *pplayer, Tech_Type_id tech)
 **************************************************************************/
 bool techs_have_fixed_costs()
 {
-  return ((game.rgame.tech_cost_style == 1
-	   || game.rgame.tech_cost_style == 2)
-	  && game.rgame.tech_leakage == 0);
+  return ((game.info.tech_cost_style == 1
+	   || game.info.tech_cost_style == 2)
+	  && game.info.tech_leakage == 0);
+}
+
+/****************************************************************************
+  Initialize tech structures.
+****************************************************************************/
+void techs_init(void)
+{
+  int i;
+
+  for (i = 0; i < ARRAY_SIZE(advances); i++) {
+    advances[i].index = i;
+  }
+
+  /* Initialize dummy tech A_NONE */
+  /* TRANS: "None" tech */
+  sz_strlcpy(advances[A_NONE].name.vernacular, N_("None"));
+  advances[A_NONE].name.translated = NULL;
+
+  /* Initialize dummy tech A_UNSET */
+  sz_strlcpy(advances[A_UNSET].name.vernacular, N_("None"));
+  advances[A_UNSET].name.translated = NULL;
+
+  /* Initialize dummy tech A_FUTURE */
+  sz_strlcpy(advances[A_FUTURE].name.vernacular, N_("Future Tech."));
+  advances[A_FUTURE].name.translated = NULL;
+
+  /* Initialize dummy tech A_UNKNOWN */
+  /* TRANS: "Unknown" advance/technology */
+  sz_strlcpy(advances[A_UNKNOWN].name.vernacular, N_("(Unknown)"));
+  advances[A_UNKNOWN].name.translated = NULL;
 }
 
 /***************************************************************
  De-allocate resources associated with the given tech.
 ***************************************************************/
-static void tech_free(Tech_Type_id tech)
+static void tech_free(Tech_type_id tech)
 {
   struct advance *p = &advances[tech];
 
@@ -649,9 +817,19 @@ static void tech_free(Tech_Type_id tech)
 ***************************************************************/
 void techs_free(void)
 {
-  Tech_Type_id i;
+  Tech_type_id i;
 
-  for (i = A_FIRST; i < game.num_tech_types; i++) {
+  for (i = A_FIRST; i < game.control.num_tech_types; i++) {
     tech_free(i);
   }
 }
+
+/***************************************************************
+ Fill the structure with some sane values
+***************************************************************/
+void player_research_init(struct player_research* research)
+{
+  memset(research, 0, sizeof(*research));
+  research->tech_goal = A_UNSET;
+  research->changed_from = -1;
+}
