@@ -23,6 +23,7 @@
 #include "city.h"
 #include "events.h"
 #include "fcintl.h"
+#include "game.h"
 #include "idex.h"
 #include "log.h"
 #include "map.h"
@@ -55,7 +56,7 @@ void handle_city_name_suggestion_req(struct player *pplayer, int value)
   freelog(LOG_VERBOSE, "handle_city_name_suggest_req(unit_pos=(%d,%d))",
 	  punit->tile->x, punit->tile->y);
 
-  dlsend_packet_city_name_suggestion_info(&pplayer->connections, value, 
+  dlsend_packet_city_name_suggestion_info(pplayer->connections, value, 
       city_name_suggestion(pplayer, punit->tile));
 }
 
@@ -76,7 +77,9 @@ void handle_city_change_specialist(struct player *pplayer, int city_id,
       || from < 0 || from >= SP_COUNT
       || !city_can_use_specialist(pcity, to)
       || pcity->specialists[from] == 0) {
-    freelog(LOG_ERROR, "Error in specialist change request from client.");
+    /* This could easily just be due to clicking faster on the specialist
+     * than the server can cope with. */
+    freelog(LOG_VERBOSE, "Error in specialist change request from client.");
     return;
   }
 
@@ -99,19 +102,19 @@ void handle_city_make_specialist(struct player *pplayer, int city_id,
   if (!pcity) {
     return;
   }
-  if (is_city_center(worker_x, worker_y)) {
+  if (is_free_worked_tile(worker_x, worker_y)) {
     auto_arrange_workers(pcity);
     sync_cities();
     return;
   }
   if (is_worker_here(pcity, worker_x, worker_y)) {
     server_remove_worker_city(pcity, worker_x, worker_y);
-    pcity->specialists[SP_ELVIS]++;
+    pcity->specialists[DEFAULT_SPECIALIST]++;
     city_refresh(pcity);
     sync_cities();
   } else {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT,
-		     _("Game: You don't have a worker here.")); 
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+		     _("You don't have a worker here.")); 
   }
   sanity_check_city(pcity);
 }
@@ -135,7 +138,7 @@ void handle_city_make_worker(struct player *pplayer, int city_id,
     return;
   }
 
-  if (is_city_center(worker_x, worker_y)) {
+  if (is_free_worked_tile(worker_x, worker_y)) {
     auto_arrange_workers(pcity);
     sync_cities();
     return;
@@ -164,21 +167,22 @@ void handle_city_make_worker(struct player *pplayer, int city_id,
 ...
 **************************************************************************/
 void really_handle_city_sell(struct player *pplayer, struct city *pcity,
-			     Impr_Type_id id)
+			     Impr_type_id id)
 {  
   if (pcity->did_sell) {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT, 
-		  _("Game: You have already sold something here this turn."));
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, 
+		  _("You have already sold something here this turn."));
     return;
   }
 
-  if (!can_sell_building(pcity, id))
+  if (!can_city_sell_building(pcity, id))
     return;
 
   pcity->did_sell=TRUE;
-  notify_player_ex(pplayer, pcity->tile, E_IMP_SOLD,
-		   _("Game: You sell %s in %s for %d gold."), 
-		   get_improvement_name(id), pcity->name,
+  notify_player(pplayer, pcity->tile, E_IMP_SOLD,
+		   _("You sell %s in %s for %d gold."), 
+		   improvement_name_translation(id),
+		   city_name(pcity),
 		   impr_sell_gold(id));
   do_sell_building(pplayer, pcity, id);
 
@@ -196,7 +200,7 @@ void handle_city_sell(struct player *pplayer, int city_id, int build_id)
 {
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
 
-  if (!pcity || build_id >= game.num_impr_types) {
+  if (!pcity || build_id >= game.control.num_impr_types) {
     return;
   }
   really_handle_city_sell(pplayer, pcity, build_id);
@@ -207,58 +211,59 @@ void handle_city_sell(struct player *pplayer, int city_id, int build_id)
 **************************************************************************/
 void really_handle_city_buy(struct player *pplayer, struct city *pcity)
 {
-  const char *name;
+  /*const char *name;*/
   int cost, total;
 
+  /* This function corresponds to city_can_buy() in the client. */
+
   assert(pcity && player_owns_city(pplayer, pcity));
  
-  if (pcity->turn_founded == game.turn) {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT,
-		  _("Game: Cannot buy in city created this turn."));
+  if (pcity->turn_founded == game.info.turn) {
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+		  _("Cannot buy in city created this turn."));
     return;
   }
 
   if (pcity->did_buy) {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT,
-		  _("Game: You have already bought this turn."));
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+		  _("You have already bought this turn."));
     return;
   }
 
-  if (get_current_construction_bonus(pcity, EFT_PROD_TO_GOLD) > 0) {
-    assert(!pcity->is_building_unit);
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT,
-                     _("Game: You don't buy %s!"),
-		     improvement_types[pcity->currently_building].name);
+  if (!pcity->production.is_unit
+      && improvement_has_flag(pcity->production.value, IF_GOLD)) {
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+                     _("You don't buy %s!"),
+		     improvement_name_translation(pcity->production.value));
     return;
   }
 
-  if (pcity->is_building_unit && pcity->anarchy != 0) {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT, 
-		     _("Game: Can't buy units when city is in disorder."));
+  if (pcity->production.is_unit && pcity->anarchy != 0) {
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND, 
+		     _("Can't buy units when city is in disorder."));
     return;
   }
 
-  if (pcity->is_building_unit) {
-    name = unit_types[pcity->currently_building].name;
-    total = unit_build_shield_cost(pcity->currently_building);
+  if (pcity->production.is_unit) {
+    /*name = utype_name_translation(utype_by_number(pcity->production.value));*/
+    total = unit_build_shield_cost(utype_by_number(pcity->production.value));
   } else {
-    name = get_improvement_name(pcity->currently_building);
-    total = impr_build_shield_cost(pcity->currently_building);
+    /*name = improvement_name_translation(pcity->production.value);*/
+    total = impr_build_shield_cost(pcity->production.value);
   }
   cost = city_buy_cost(pcity);
-  if (cost == 0 || cost > pplayer->economic.gold) {
+  if (cost <= 0) {
+    return; /* sanity */
+  }
+  if (cost > pplayer->economic.gold) {
+    /* In case something changed while player tried to buy, or player 
+     * tried to cheat! */
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+		     _("%d gold required.  You only have %d gold."), cost,
+                     pplayer->economic.gold);
     return;
   }
 
-  /*
-   * Need to make this more restrictive.  AI is sometimes buying
-   * things that force it to sell buildings due to upkeep problems.
-   * upkeep expense is only known in ai_manage_taxes().
-   * Also, we should sort this list so cheapest things are bought first,
-   * and/or take danger into account.
-   * AJS, 1999110
-   */
-
   pplayer->economic.gold-=cost;
   if (pcity->shield_stock < total){
     /* As we never put penalty on disbanded_shields, we can
@@ -269,14 +274,10 @@ void really_handle_city_buy(struct player *pplayer, struct city *pcity)
   }
   city_refresh(pcity);
   
-  conn_list_do_buffer(&pplayer->connections);
-  notify_player_ex(pplayer, pcity->tile, 
-                   pcity->is_building_unit?E_UNIT_BUY:E_IMP_BUY,
-		   _("Game: %s bought in %s for %d gold."), 
-		   name, pcity->name, cost);
+  conn_list_do_buffer(pplayer->connections);
   send_city_info(pplayer, pcity);
   send_player_info(pplayer,pplayer);
-  conn_list_do_unbuffer(&pplayer->connections);
+  conn_list_do_unbuffer(pplayer->connections);
 }
 
 /**************************************************************************
@@ -336,29 +337,34 @@ void handle_city_change(struct player *pplayer, int city_id, int build_id,
 			bool is_build_id_unit_id)
 {
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
+  struct city_production prod = {
+    .is_unit = is_build_id_unit_id,
+    .value = build_id
+  };
 
   if (!pcity) {
     return;
   }
 
-  if (pcity->is_building_unit == is_build_id_unit_id
-      && pcity->currently_building == build_id) {
+  if (pcity->production.is_unit == is_build_id_unit_id
+      && pcity->production.value == build_id) {
     /* The client probably shouldn't send such a packet. */
     return;
   }
 
-   if (is_build_id_unit_id && !can_build_unit(pcity, build_id))
-     return;
+  if (is_build_id_unit_id
+      && !can_build_unit(pcity, utype_by_number(build_id))) {
+    return;
+  }
    if (!is_build_id_unit_id && !can_build_improvement(pcity, build_id))
      return;
-  if (pcity->did_buy && pcity->shield_stock > 0) {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT,
-		     _("Game: You have bought this turn, can't change."));
+  if (!city_can_change_build(pcity)) {
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+		     _("You have bought this turn, can't change."));
     return;
   }
 
-  change_build_target(pplayer, pcity, build_id, is_build_id_unit_id,
-		      E_NOEVENT);
+  change_build_target(pplayer, pcity, prod, E_CITY_PRODUCTION_CHANGED);
 
   sanity_check_city(pcity);
   city_refresh(pcity);
@@ -378,8 +384,8 @@ void handle_city_rename(struct player *pplayer, int city_id, char *name)
   }
 
   if (!is_allowed_city_name(pplayer, name, message, sizeof(message))) {
-    notify_player_ex(pplayer, pcity->tile, E_NOEVENT,
-		     _("Game: %s"),  message);
+    notify_player(pplayer, pcity->tile, E_BAD_COMMAND,
+		  message);
     return;
   }
 
@@ -388,10 +394,12 @@ void handle_city_rename(struct player *pplayer, int city_id, char *name)
   send_city_info(NULL, pcity);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-void handle_city_options_req(struct player *pplayer, int city_id, int value)
+/****************************************************************************
+  Handles a packet from the client that requests the city options for the
+  given city be changed.
+****************************************************************************/
+void handle_city_options_req(struct player *pplayer, int city_id,
+			     bv_city_options options)
 {
   struct city *pcity = player_find_city_by_id(pplayer, city_id);
 
@@ -399,22 +407,7 @@ void handle_city_options_req(struct player *pplayer, int city_id, int value)
     return;
   }
 
-  pcity->city_options = value;
+  pcity->city_options = options;
+
   send_city_info(pplayer, pcity);
 }
-
-/***************************************************************
-  Tell the client the cost of inciting a revolt or bribing a unit.
-  Only send result back to the requesting connection, not all
-  connections for that player.
-***************************************************************/
-void handle_city_incite_inq(struct connection *pc, int city_id)
-{
-  struct player *pplayer = pc->player;
-  struct city *pcity = find_city_by_id(city_id);
-
-  if (pplayer && pcity) {
-    dsend_packet_city_incite_info(pc, city_id,
-				  city_incite_cost(pplayer, pcity));
-  }
-}
