@@ -14,27 +14,28 @@
 #ifndef FC__CITYTURN_H
 #define FC__CITYTURN_H
 
-struct city;
-struct player;
-struct unit;
+#include "shared.h"		/* bool type */
+
+#include "fc_types.h"
+
 struct conn_list;
+struct cm_result;
 
-int unit_being_aggressive(struct unit *punit);
-
-int city_refresh(struct city *pcity);          /* call if city has changed */
+void city_refresh(struct city *pcity);          /* call if city has changed */
 void global_city_refresh(struct player *pplayer); /* tax/govt changed */
 
 void auto_arrange_workers(struct city *pcity); /* will arrange the workers */
-int add_adjust_workers(struct city *pcity);   /* will add workers */
+void apply_cmresult_to_city(struct city *pcity, struct cm_result *cmr);
 
-void city_auto_remove_worker(struct city *pcity); 
+bool city_reduce_size(struct city *pcity, int pop_loss);
 void send_global_city_turn_notifications(struct conn_list *dest);
 void send_city_turn_notifications(struct conn_list *dest, struct city *pcity);
-void begin_cities_turn(struct player *pplayer);
 void update_city_activities(struct player *pplayer);
-void city_incite_cost(struct city *pcity);
-void remove_obsolete_buildings_city(struct city *pcity, int refresh);
-void remove_obsolete_buildings(struct player *plr);
+int city_incite_cost(struct player *pplayer, struct city *pcity);
+void remove_obsolete_buildings_city(struct city *pcity, bool refresh);
+void remove_obsolete_buildings(struct player *pplayer);
+
+void advisor_choose_build(struct player *pplayer, struct city *pcity);
 
 void nullify_prechange_production(struct city *pcity);
 #endif  /* FC__CITYTURN_H */
