@@ -19,15 +19,11 @@
 
 struct connection;
 struct conn_list;
-struct packet_authentication_reply;
-struct packet_login_request;
-struct packet_server_join_req;
 
 void establish_new_connection(struct connection *pconn);
-void reject_new_connection(const char *msg, struct connection *pconn);
+void reject_new_connection(struct connection *pconn, const char *message);
 
-bool handle_login_request(struct connection *pconn,
-                          struct packet_server_join_req *req);
+bool server_join_request(struct connection *pconn, void *packet);
 
 void lost_connection_to_client(struct connection *pconn);
 
