@@ -21,15 +21,18 @@
 #include "capability.h"
 #include "events.h"
 #include "fcintl.h"
+#include "game.h"
 #include "improvement.h"
 #include "log.h"
 #include "mem.h"
+#include "movement.h"
 #include "packets.h"
 #include "rand.h"
 #include "registry.h"
 #include "shared.h"
 #include "support.h"
 
+#include "connecthand.h"
 #include "maphand.h"
 #include "plrhand.h"
 #include "unittools.h"
@@ -49,7 +52,7 @@ static void init_game_id(void)
     "0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ";
   int i;
 
-  for (i = 0; i < sizeof(game.id) - 1; i++) {
+  for (i = 0; i < ARRAY_SIZE(game.id) - 1; i++) {
     game.id[i] = chars[myrand(sizeof(chars) - 1)];
   }
   game.id[i] = '\0';
@@ -61,7 +64,7 @@ static void init_game_id(void)
 static void place_starting_unit(struct tile *ptile, struct player *pplayer,
 				char crole)
 {
-  Unit_Type_id utype;
+  struct unit_type *utype;
   enum unit_flag_id role;
 
   assert(!is_non_allied_unit_tile(ptile, pplayer));
@@ -70,17 +73,15 @@ static void place_starting_unit(struct tile *ptile, struct player *pplayer,
    * other cases, huts are avoided as start positions).  Remove any such hut,
    * and make sure to tell the client, since we may have already sent this
    * tile (with the hut) earlier: */
-  if (map_has_special(ptile, S_HUT)) {
-    map_clear_special(ptile, S_HUT);
+  if (tile_has_special(ptile, S_HUT)) {
+    tile_clear_special(ptile, S_HUT);
     update_tile_knowledge(ptile);
     freelog(LOG_VERBOSE, "Removed hut on start position for %s",
-	    pplayer->name);
+	    player_name(pplayer));
   }
 
   /* Expose visible area. */
-  circle_iterate(ptile, game.rgame.init_vis_radius_sq, ctile) {
-    show_area(pplayer, ctile, 0);
-  } circle_iterate_end;
+  map_show_circle(pplayer, ptile, game.info.init_vis_radius_sq);
 
   switch(crole) {
   case 'c':
@@ -121,30 +122,56 @@ static void place_starting_unit(struct tile *ptile, struct player *pplayer,
   /* Create the unit of an appropriate type, if it exists */
   if (num_role_units(role) > 0) {
     utype = first_role_unit_for_player(pplayer, role);
-    if (utype == U_LAST) {
+    if (utype == NULL) {
       utype = get_role_unit(role, 0);
     }
 
-    /* We cannot currently handle sea units as start units. */
-    if (unit_types[utype].move_type == SEA_MOVING) {
-      freelog(LOG_ERROR, _("Sea moving start units are not yet supported, "
-                           "%s not created."), unit_types[utype].name);
-      notify_player(pplayer, _("Sea moving start units are not yet supported. "
-                               "Nobody gets %s."), unit_types[utype].name);
+    /* We cannot currently handle sea units as start units.
+     * TODO: remove this code block when we can. */
+    if (utype->move_type == SEA_MOVING) {
+      freelog(LOG_ERROR, "Sea moving start units are not yet supported, "
+                           "%s not created.",
+                         utype_rule_name(utype));
+      notify_player(pplayer, NULL, E_BAD_COMMAND,
+		    _("Sea moving start units are not yet supported. "
+		      "Nobody gets %s."),
+		    utype_name_translation(utype));
       return;
     }
 
-    (void) create_unit(pplayer, ptile, utype, FALSE, 0, -1);
+    (void) create_unit(pplayer, ptile, utype, FALSE, 0, 0);
   }
 }
 
+/****************************************************************************
+  Find a valid position not far from our starting position.
+****************************************************************************/
+static struct tile *find_dispersed_position(struct player *pplayer,
+                                            struct start_position *p)
+{
+  struct tile *ptile;
+  int x, y;
+
+  do {
+    x = p->tile->x + myrand(2 * game.info.dispersion + 1) 
+        - game.info.dispersion;
+    y = p->tile->y + myrand(2 * game.info.dispersion + 1)
+        - game.info.dispersion;
+  } while (!((ptile = map_pos_to_tile(x, y))
+             && tile_get_continent(p->tile) == tile_get_continent(ptile)
+             && !is_ocean(tile_get_terrain(ptile))
+             && !is_non_allied_unit_tile(ptile, pplayer)));
+
+  return ptile;
+}
+
 /****************************************************************************
   Initialize a new game: place the players' units onto the map, etc.
 ****************************************************************************/
 void init_new_game(void)
 {
   const int NO_START_POS = -1;
-  int start_pos[game.nplayers];
+  int start_pos[game.info.nplayers];
   bool pos_used[map.num_start_positions];
   int i, num_used = 0;
 
@@ -156,13 +183,15 @@ void init_new_game(void)
   /* First set up some data fields. */
   freelog(LOG_VERBOSE, "Placing players at start positions.");
   for (i = 0; i < map.num_start_positions; i++) {
-    Nation_Type_id n = map.start_positions[i].nation;
+    struct nation_type *n = map.start_positions[i].nation;
 
     pos_used[i] = FALSE;
-    freelog(LOG_VERBOSE, "%3d : (%2d,%2d) : %d : %s",
-	    i, map.start_positions[i].tile->x,
+    freelog(LOG_VERBOSE, "%3d : (%2d,%2d) : \"%s\" (%d)",
+	    i,
+	    map.start_positions[i].tile->x,
 	    map.start_positions[i].tile->y,
-	    n, (n >= 0 ? get_nation_name(n) : ""));
+	    n ? nation_rule_name(n) : "",
+	    n ? n->index : -1);
   }
   players_iterate(pplayer) {
     start_pos[pplayer->player_no] = NO_START_POS;
@@ -175,7 +204,9 @@ void init_new_game(void)
       assert(pplayer->nation != NO_NATION_SELECTED);
       if (pplayer->nation == map.start_positions[i].nation) {
 	freelog(LOG_VERBOSE, "Start_pos %d matches player %d (%s).",
-		i, pplayer->player_no, get_nation_name(pplayer->nation));
+		i,
+		pplayer->player_no,
+		nation_rule_name(nation_of_player(pplayer)));
 	start_pos[pplayer->player_no] = i;
 	pos_used[i] = TRUE;
 	num_used++;
@@ -194,7 +225,9 @@ void init_new_game(void)
 	  if (which == 0) {
 	    freelog(LOG_VERBOSE,
 		    "Randomly assigning player %d (%s) to pos %d.",
-		    pplayer->player_no, get_nation_name(pplayer->nation), i);
+		    pplayer->player_no,
+		    nation_rule_name(nation_of_player(pplayer)),
+		    i);
 	    start_pos[pplayer->player_no] = i;
 	    pos_used[i] = TRUE;
 	    num_used++;
@@ -212,52 +245,46 @@ void init_new_game(void)
     struct start_position pos
       = map.start_positions[start_pos[pplayer->player_no]];
 
-    /* don't give any units to observer */
-    if (pplayer->is_observer) {
-      continue;
-    }
-
     /* Place the first unit. */
-    place_starting_unit(pos.tile, pplayer, game.start_units[0]);
+    place_starting_unit(pos.tile, pplayer, game.info.start_units[0]);
   } players_iterate_end;
 
   /* Place all other units. */
   players_iterate(pplayer) {
-    int i, x, y;
+    int i;
     struct tile *ptile;
+    struct nation_type *nation = nation_of_player(pplayer);
     struct start_position p
       = map.start_positions[start_pos[pplayer->player_no]];
 
-    /* don't give any units to observer */
-    if (pplayer->is_observer) {
-      continue;
-    }
-
-    for (i = 1; i < strlen(game.start_units); i++) {
-      do {
-	x = p.tile->x + myrand(2 * game.dispersion + 1) - game.dispersion;
-	y = p.tile->y + myrand(2 * game.dispersion + 1) - game.dispersion;
-      } while (!((ptile = map_pos_to_tile(x, y))
-		 && map_get_continent(p.tile) == map_get_continent(ptile)
-		 && !is_ocean(map_get_terrain(ptile))
-		 && !is_non_allied_unit_tile(ptile, pplayer)));
-
+    /* Place global start units */
+    for (i = 1; i < strlen(game.info.start_units); i++) {
+      ptile = find_dispersed_position(pplayer, &p);
 
       /* Create the unit of an appropriate type. */
-      place_starting_unit(ptile, pplayer, game.start_units[i]);
+      place_starting_unit(ptile, pplayer, game.info.start_units[i]);
+    }
+
+    /* Place nation specific start units (not role based!) */
+    i = 0;
+    while (nation->init_units[i] != NULL && i < MAX_NUM_UNIT_LIST) {
+      ptile = find_dispersed_position(pplayer, &p);
+      create_unit(pplayer, ptile, nation->init_units[i], FALSE, 0, 0);
+      i++;
     }
   } players_iterate_end;
 
-  /* Initialise list of improvements with world-wide equiv_range */
-  improvement_status_init(game.improvements, ARRAY_SIZE(game.improvements));
+  shuffle_players();
 }
 
 /**************************************************************************
-...
+  This is called once at the start of each phase to alert the clients to
+  the new phase.  game.info.phase should be incremented before calling it.
 **************************************************************************/
-void send_start_turn_to_clients(void)
+void send_start_phase_to_clients(void)
 {
-  lsend_packet_start_turn(&game.game_connections);
+  /* This function is so simple it could probably be dropped... */
+  dlsend_packet_start_phase(game.est_connections, game.info.phase);
 }
 
 /**************************************************************************
@@ -269,18 +296,17 @@ void send_year_to_clients(int year)
   struct packet_new_year apacket;
   int i;
   
-  for(i=0; i<game.nplayers; i++) {
+  for(i=0; i<game.info.nplayers; i++) {
     struct player *pplayer = &game.players[i];
-    pplayer->turn_done = FALSE;
     pplayer->nturns_idle++;
   }
 
   apacket.year = year;
-  apacket.turn = game.turn;
-  lsend_packet_new_year(&game.game_connections, &apacket);
+  apacket.turn = game.info.turn;
+  lsend_packet_new_year(game.est_connections, &apacket);
 
   /* Hmm, clients could add this themselves based on above packet? */
-  notify_conn_ex(&game.game_connections, NULL, E_NEXT_YEAR, _("Year: %s"),
+  notify_conn(game.est_connections, NULL, E_NEXT_YEAR, _("Year: %s"),
 		 textyear(year));
 }
 
@@ -297,57 +323,35 @@ void send_game_state(struct conn_list *dest, int state)
 
 /**************************************************************************
   Send game_info packet; some server options and various stuff...
-  dest==NULL means game.game_connections
+  dest==NULL means game.est_connections
+
+  It may be sent at any time. It MUST be sent before any player info, 
+  as it contains the number of players.  To avoid inconsistency, it
+  SHOULD be sent after rulesets and any other server settings.
 **************************************************************************/
 void send_game_info(struct conn_list *dest)
 {
   struct packet_game_info ginfo;
-  int i;
 
-  if (!dest)
-    dest = &game.game_connections;
-
-  ginfo.gold = game.gold;
-  ginfo.tech = game.tech;
-  ginfo.researchcost = game.researchcost;
-  ginfo.skill_level = game.skill_level;
-  ginfo.timeout = game.timeout;
-  ginfo.end_year = game.end_year;
-  ginfo.year = game.year;
-  ginfo.turn = game.turn;
-  ginfo.min_players = game.min_players;
-  ginfo.max_players = game.max_players;
-  ginfo.nplayers = game.nplayers;
-  ginfo.globalwarming = game.globalwarming;
-  ginfo.heating = game.heating;
-  ginfo.nuclearwinter = game.nuclearwinter;
-  ginfo.cooling = game.cooling;
-  ginfo.diplomacy = game.diplomacy;
-  ginfo.techpenalty = game.techpenalty;
-  ginfo.foodbox = game.foodbox;
-  ginfo.civstyle = game.civstyle;
-  ginfo.spacerace = game.spacerace;
-  ginfo.unhappysize = game.unhappysize;
-  ginfo.angrycitizen = game.angrycitizen;
-  ginfo.diplcost = game.diplcost;
-  ginfo.freecost = game.freecost;
-  ginfo.conquercost = game.conquercost;
-  ginfo.cityfactor = game.cityfactor;
-  for (i = 0; i < A_LAST /*game.num_tech_types */ ; i++)
-    ginfo.global_advances[i] = game.global_advances[i];
-  for (i = 0; i < B_LAST /*game.num_impr_types */ ; i++)
-    ginfo.global_wonders[i] = game.global_wonders[i];
-  /* the following values are computed every
-     time a packet_game_info packet is created */
-  if (game.timeout != 0) {
-    ginfo.seconds_to_turndone =
-	game.turn_start + game.timeout - time(NULL);
-  } else {
-    /* unused but at least initialized */
-    ginfo.seconds_to_turndone = -1;
+  if (!dest) {
+    dest = game.est_connections;
   }
 
-  conn_list_iterate(*dest, pconn) {
+  ginfo = game.info;
+
+  /* the following values are computed every
+     time a packet_game_info packet is created */
+  if (game.info.timeout > 0 && game.phase_timer) {
+    /* Sometimes this function is called before the phase_timer is
+     * initialized.  In that case we want to send the dummy value. */
+    ginfo.seconds_to_phasedone
+      = game.info.seconds_to_phasedone - read_timer_seconds(game.phase_timer);
+  } else {
+    /* unused but at least initialized */
+    ginfo.seconds_to_phasedone = -1.0;
+  }
+
+  conn_list_iterate(dest, pconn) {
     /* ? fixme: check for non-players: */
     ginfo.player_idx = (pconn->player ? pconn->player->player_no : -1);
     send_packet_game_info(pconn, &ginfo);
@@ -356,43 +360,43 @@ void send_game_info(struct conn_list *dest)
 }
 
 /**************************************************************************
-  adjusts game.timeout based on various server options
+  adjusts game.info.timeout based on various server options
 
-  timeoutint: adjust game.timeout every timeoutint turns
-  timeoutinc: adjust game.timeout by adding timeoutinc to it.
-  timeoutintinc: every time we adjust game.timeout, we add timeoutintinc
+  timeoutint: adjust game.info.timeout every timeoutint turns
+  timeoutinc: adjust game.info.timeout by adding timeoutinc to it.
+  timeoutintinc: every time we adjust game.info.timeout, we add timeoutintinc
                  to timeoutint.
-  timeoutincmult: every time we adjust game.timeout, we multiply timeoutinc
+  timeoutincmult: every time we adjust game.info.timeout, we multiply timeoutinc
                   by timeoutincmult
 **************************************************************************/
 int update_timeout(void)
 {
   /* if there's no timer or we're doing autogame, do nothing */
-  if (game.timeout < 1 || game.timeoutint == 0) {
-    return game.timeout;
+  if (game.info.timeout < 1 || game.timeoutint == 0) {
+    return game.info.timeout;
   }
 
   if (game.timeoutcounter >= game.timeoutint) {
-    game.timeout += game.timeoutinc;
+    game.info.timeout += game.timeoutinc;
     game.timeoutinc *= game.timeoutincmult;
 
     game.timeoutcounter = 1;
     game.timeoutint += game.timeoutintinc;
 
-    if (game.timeout > GAME_MAX_TIMEOUT) {
-      notify_conn_ex(&game.game_connections, NULL, E_NOEVENT,
+    if (game.info.timeout > GAME_MAX_TIMEOUT) {
+      notify_conn(game.est_connections, NULL, E_SETTING,
 		     _("The turn timeout has exceeded its maximum value, "
 		       "fixing at its maximum"));
-      freelog(LOG_DEBUG, "game.timeout exceeded maximum value");
-      game.timeout = GAME_MAX_TIMEOUT;
+      freelog(LOG_DEBUG, "game.info.timeout exceeded maximum value");
+      game.info.timeout = GAME_MAX_TIMEOUT;
       game.timeoutint = 0;
       game.timeoutinc = 0;
-    } else if (game.timeout < 0) {
-      notify_conn_ex(&game.game_connections, NULL, E_NOEVENT,
+    } else if (game.info.timeout < 0) {
+      notify_conn(game.est_connections, NULL, E_SETTING,
 		     _("The turn timeout is smaller than zero, "
 		       "fixing at zero."));
-      freelog(LOG_DEBUG, "game.timeout less than zero");
-      game.timeout = 0;
+      freelog(LOG_DEBUG, "game.info.timeout less than zero");
+      game.info.timeout = 0;
     }
   } else {
     game.timeoutcounter++;
@@ -400,25 +404,29 @@ int update_timeout(void)
 
   freelog(LOG_DEBUG, "timeout=%d, inc=%d incmult=%d\n   "
 	  "int=%d, intinc=%d, turns till next=%d",
-	  game.timeout, game.timeoutinc, game.timeoutincmult,
+	  game.info.timeout, game.timeoutinc, game.timeoutincmult,
 	  game.timeoutint, game.timeoutintinc,
 	  game.timeoutint - game.timeoutcounter);
 
-  return game.timeout;
+  return game.info.timeout;
 }
 
 /**************************************************************************
-  adjusts game.turn_start when enemy moves an unit, we see it and the 
-  remaining timeout is smaller than the option
-  It's possible to use a simular function to do that per player.
+  adjusts game.seconds_to_turn_done when enemy moves a unit, we see it and
+  the remaining timeout is smaller than the timeoutaddenemymove option.
+
+  It's possible to use a similar function to do that per-player.  In
+  theory there should be a separate timeout for each player and the
+  added time should only go onto the victim's timer.
 **************************************************************************/
 void increase_timeout_because_unit_moved(void)
 {
-  if (game.timeout != 0){
-    int seconds_to_turndone = game.turn_start + game.timeout - time(NULL);
+  if (game.info.timeout > 0 && game.timeoutaddenemymove > 0) {
+    double maxsec = (read_timer_seconds(game.phase_timer)
+		     + (double)game.timeoutaddenemymove);
 
-    if (seconds_to_turndone < game.timeoutaddenemymove){
-      game.turn_start = time(NULL) - game.timeout + game.timeoutaddenemymove;
+    if (maxsec > game.info.seconds_to_phasedone) {
+      game.info.seconds_to_phasedone = maxsec;
       send_game_info(NULL);
     }	
   }
@@ -462,32 +470,48 @@ static const char *get_challenge_fullname(struct connection *pc)
 **************************************************************************/
 const char *new_challenge_filename(struct connection *pc)
 {
-  if (!has_capability("new_hack", pc->capability)) {
-    return "";
-  }
-
   gen_challenge_filename(pc);
   return get_challenge_filename(pc);
 }
 
 
+/************************************************************************** 
+  Call this on a connection with HACK access to send it a set of ruleset
+  choices.  Probably this should be called immediately when granting
+  HACK access to a connection.
+**************************************************************************/
+static void send_ruleset_choices(struct connection *pc)
+{
+  struct packet_ruleset_choices packet;
+  static char **rulesets = NULL;
+  int i;
+
+  if (!rulesets) {
+    /* This is only read once per server invocation.  Add a new ruleset
+     * and you have to restart the server. */
+    rulesets = datafilelist(RULESET_SUFFIX);
+  }
+
+  for (i = 0; i < MAX_NUM_RULESETS && rulesets[i]; i++) {
+    sz_strlcpy(packet.rulesets[i], rulesets[i]);
+  }
+  packet.ruleset_count = i;
+
+  send_packet_ruleset_choices(pc, &packet);
+}
+
+
 /************************************************************************** 
 opens a file specified by the packet and compares the packet values with
 the file values. Sends an answer to the client once it's done.
 **************************************************************************/
 void handle_single_want_hack_req(struct connection *pc,
-    				 const struct packet_single_want_hack_req
-				 *packet)
+                                 struct packet_single_want_hack_req *packet)
 {
   struct section_file file;
   char *token = NULL;
   bool you_have_hack = FALSE;
 
-  if (!has_capability("new_hack", pc->capability)) {
-    dsend_packet_single_want_hack_reply(pc, FALSE);
-    return ;
-  }
-
   if (section_file_load_nodup(&file, get_challenge_fullname(pc))) {
     token = secfile_lookup_str_default(&file, NULL, "challenge.token");
     you_have_hack = (token && strcmp(token, packet->token) == 0);
@@ -503,4 +527,7 @@ void handle_single_want_hack_req(struct connection *pc,
   }
 
   dsend_packet_single_want_hack_reply(pc, you_have_hack);
+
+  send_ruleset_choices(pc);
+  send_conn_info(pc->self, NULL);
 }
