@@ -39,17 +39,11 @@
 #include "utilities.h"
 
 
-/* Old-style terrain enumeration: deprecated. */
-enum {
-  T_ARCTIC, T_DESERT, T_FOREST, T_GRASSLAND, T_HILLS, T_JUNGLE,
-  T_MOUNTAINS, T_OCEAN, T_PLAINS, T_SWAMP, T_TUNDRA,
-};
-
 /* Wrappers for easy access.  They are a macros so they can be a lvalues.*/
 #define rmap(ptile) (river_map[ptile->index])
 
 static void make_huts(int number);
-static void add_specials(int prob);
+static void add_resources(int prob);
 static void mapgenerator2(void);
 static void mapgenerator3(void);
 static void mapgenerator4(void);
@@ -228,6 +222,105 @@ static bool terrain_is_too_high(struct tile *ptile,
   return TRUE;
 }
 
+/****************************************************************************
+  Pick a terrain based on the target property and a property to avoid.
+
+  If the target property is given, then all terrains with that property
+  will be considered and one will be picked at random based on the amount
+  of the property each terrain has.  If no target property is given all
+  terrains will be assigned equal likelihood.
+
+  If the preferred property is given, only terrains with (some of) that
+  property will be chosen.
+
+  If the avoid property is given, then any terrain with (any of) that
+  property will be avoided.
+
+  This function must always return a valid terrain.
+****************************************************************************/
+static struct terrain *pick_terrain(enum mapgen_terrain_property target,
+				    enum mapgen_terrain_property prefer,
+				    enum mapgen_terrain_property avoid)
+{
+  int sum = 0;
+
+  /* Find the total weight. */
+  terrain_type_iterate(pterrain) {
+    if (avoid != MG_LAST && pterrain->property[avoid] > 0) {
+      continue;
+    }
+    if (prefer != MG_LAST && pterrain->property[prefer] == 0) {
+      continue;
+    }
+
+    if (target != MG_LAST) {
+      sum += pterrain->property[target];
+    } else {
+      sum++;
+    }
+  } terrain_type_iterate_end;
+
+  /* Now pick. */
+  sum = myrand(sum);
+
+  /* Finally figure out which one we picked. */
+  terrain_type_iterate(pterrain) {
+    int property;
+
+    if (avoid != MG_LAST && pterrain->property[avoid] > 0) {
+      continue;
+    }
+    if (prefer != MG_LAST && pterrain->property[prefer] == 0) {
+      continue;
+    }
+
+    if (target != MG_LAST) {
+      property = pterrain->property[target];
+    } else {
+      property = 1;
+    }
+    if (sum < property) {
+      return pterrain;
+    }
+    sum -= property;
+  } terrain_type_iterate_end;
+
+  /* This can happen with sufficient quantities of preferred and avoided
+   * characteristics.  Drop a requirement and try again. */
+  if (prefer != MG_LAST) {
+    return pick_terrain(target, MG_LAST, avoid);
+  } else if (avoid != MG_LAST) {
+    return pick_terrain(target, prefer, MG_LAST);
+  } else {
+    return pick_terrain(MG_LAST, prefer, avoid);
+  }
+}
+
+/**************************************************************************
+  Picks an ocean terrain to match the given depth (as a percentage).
+
+  FIXME: this should return NULL if there is no available ocean.
+**************************************************************************/
+static struct terrain *pick_ocean(int depth)
+{
+  /* FIXME: pick_terrain_by_flag may return NULL if there is no match. */
+  struct terrain *best_terrain = pick_terrain_by_flag(TER_OCEANIC);
+  int best_match = abs(depth - best_terrain->property[MG_OCEAN_DEPTH]);
+
+  terrain_type_iterate(pterrain) {
+    if (terrain_has_flag(pterrain, TER_OCEANIC)) {
+      int match = abs(depth - pterrain->property[MG_OCEAN_DEPTH]);
+
+      if (match < best_match) {
+	best_match = match;
+	best_terrain = pterrain;
+      }
+    }
+  } terrain_type_iterate_end;
+
+  return best_terrain;
+}
+
 /**************************************************************************
   make_relief() will convert all squares that are higher than thill to
   mountains and hills. Note that thill will be adjusted according to
@@ -247,15 +340,12 @@ static void make_relief(void)
 	  (myrand(10) > 5 
 	   || !terrain_is_too_high(ptile, hmap_mountain_level, hmap(ptile))))
 	 || terrain_is_too_flat(ptile, hmap_mountain_level, hmap(ptile)))) {
-      /* Randomly place hills and mountains on "high" tiles.  But don't
-       * put hills near the poles (they're too green). */
-      if (myrand(100) > 70 || tmap_is(ptile, TT_NHOT)) {
-	map_set_terrain(ptile, T_MOUNTAINS);
-	map_set_placed(ptile);
-      } else {
-	map_set_terrain(ptile, T_HILLS);
-	map_set_placed(ptile);
-      }
+      struct terrain *pterrain
+	= pick_terrain(MG_MOUNTAINOUS, MG_LAST,
+		       (tmap_is(ptile, TT_NHOT) ? MG_GREEN : MG_LAST));
+
+      tile_set_terrain(ptile, pterrain);
+      map_set_placed(ptile);
     }
   } whole_map_iterate_end;
 }
@@ -272,7 +362,7 @@ static void make_polar(void)
 	|| (tmap_is(ptile, TT_COLD)
 	    && (myrand(10) > 7)
 	    && is_temperature_type_near(ptile, TT_FROZEN))) { 
-      map_set_terrain(ptile, T_ARCTIC);
+      tile_set_terrain(ptile, pick_terrain(MG_FROZEN, MG_LAST, MG_LAST));
     }
   } whole_map_iterate_end;
 }
@@ -286,8 +376,9 @@ static bool ok_for_separate_poles(struct tile *ptile)
     return TRUE;
   }
   adjc_iterate(ptile, tile1) {
-    if (!is_ocean(map_get_terrain(tile1)) && 
-        map_get_continent(tile1) != 0) {
+    if (tile1->terrain != T_UNKNOWN
+	&& !is_ocean(tile_get_terrain(tile1))
+        && tile_get_continent(tile1) != 0) {
       return FALSE;
     }
   } adjc_iterate_end;
@@ -308,8 +399,8 @@ static void make_polar_land(void)
 	 (myrand(10) > 7) &&
 	 is_temperature_type_near(ptile, TT_FROZEN) &&
 	 ok_for_separate_poles(ptile))) {
-      map_set_terrain(ptile, T_UNKNOWN);
-      map_set_continent(ptile, 0);
+      tile_set_terrain(ptile, T_UNKNOWN);
+      tile_set_continent(ptile, 0);
     } 
   } whole_map_iterate_end;
 }
@@ -318,7 +409,7 @@ static void make_polar_land(void)
   Recursively generate terrains.
 **************************************************************************/
 static void place_terrain(struct tile *ptile, int diff, 
-                           Terrain_type_id ter, int *to_be_placed,
+                           struct terrain *pterrain, int *to_be_placed,
 			   wetness_c        wc,
 			   temperature_type tc,
 			   miscellaneous_c  mc)
@@ -327,7 +418,7 @@ static void place_terrain(struct tile *ptile, int diff,
     return;
   }
   assert(not_placed(ptile));
-  map_set_terrain(ptile, ter);
+  tile_set_terrain(ptile, pterrain);
   map_set_placed(ptile);
   (*to_be_placed)--;
   
@@ -340,7 +431,8 @@ static void place_terrain(struct tile *ptile, int diff,
  	&& test_miscellaneous(tile1, mc)
 	&& Delta < diff 
 	&& myrand(10) > 4) {
-	place_terrain(tile1, diff - 1 - Delta, ter, to_be_placed, wc, tc, mc);
+	place_terrain(tile1, diff - 1 - Delta, pterrain,
+		      to_be_placed, wc, tc, mc);
     }
   } cardinal_adjc_iterate_end;
 }
@@ -353,15 +445,14 @@ static void make_plain(struct tile *ptile, int *to_be_placed )
 {
   /* in cold place we get tundra instead */
   if (tmap_is(ptile, TT_FROZEN)) {
-    map_set_terrain(ptile, T_ARCTIC); 
+    tile_set_terrain(ptile,
+		     pick_terrain(MG_FROZEN, MG_LAST, MG_MOUNTAINOUS));
   } else if (tmap_is(ptile, TT_COLD)) {
-    map_set_terrain(ptile, T_TUNDRA); 
+    tile_set_terrain(ptile,
+		     pick_terrain(MG_COLD, MG_LAST, MG_MOUNTAINOUS)); 
   } else {
-    if (myrand(100) > 50) {
-      map_set_terrain(ptile, T_GRASSLAND);
-    } else {
-      map_set_terrain(ptile, T_PLAINS);
-    }
+    tile_set_terrain(ptile,
+		     pick_terrain(MG_TEMPERATE, MG_GREEN, MG_MOUNTAINOUS));
   }
   map_set_placed(ptile);
   (*to_be_placed)--;
@@ -434,15 +525,20 @@ static void make_terrains(void)
   /* the placement loop */
   do {
     
-    PLACE_ONE_TYPE(forests_count , plains_count, T_FOREST,
+    PLACE_ONE_TYPE(forests_count , plains_count,
+		   pick_terrain(MG_FOLIAGE, MG_TEMPERATE, MG_LAST),
 		   WC_ALL, TT_NFROZEN, MC_NONE, 60);
-    PLACE_ONE_TYPE(jungles_count, forests_count , T_JUNGLE,
+    PLACE_ONE_TYPE(jungles_count, forests_count,
+		   pick_terrain(MG_FOLIAGE, MG_TROPICAL, MG_COLD),
 		   WC_ALL, TT_TROPICAL, MC_NONE, 50);
-    PLACE_ONE_TYPE(swamps_count, forests_count , T_SWAMP,
+    PLACE_ONE_TYPE(swamps_count, forests_count,
+		   pick_terrain(MG_FOLIAGE, MG_WET, MG_TROPICAL),
 		   WC_NDRY, TT_HOT, MC_LOW, 50);
-    PLACE_ONE_TYPE(deserts_count, alt_deserts_count , T_DESERT,
+    PLACE_ONE_TYPE(deserts_count, alt_deserts_count,
+		   pick_terrain(MG_DRY, MG_TROPICAL, MG_COLD),
 		   WC_DRY, TT_NFROZEN, MC_NLOW, 80);
-    PLACE_ONE_TYPE(alt_deserts_count, plains_count , T_DESERT,
+    PLACE_ONE_TYPE(alt_deserts_count, plains_count,
+		   pick_terrain(MG_DRY, MG_TROPICAL, MG_COLD),
 		   WC_ALL, TT_NFROZEN, MC_NLOW, 40);
  
   /* make the plains and tundras */
@@ -493,8 +589,7 @@ static int river_test_rivergrid(struct tile *ptile)
 *********************************************************************/
 static int river_test_highlands(struct tile *ptile)
 {
-  return (((map_get_terrain(ptile) == T_HILLS) ? 1 : 0) +
-	  ((map_get_terrain(ptile) == T_MOUNTAINS) ? 2 : 0));
+  return ptile->terrain->property[MG_MOUNTAINOUS];
 }
 
 /*********************************************************************
@@ -518,8 +613,13 @@ static int river_test_adjacent_river(struct tile *ptile)
 *********************************************************************/
 static int river_test_adjacent_highlands(struct tile *ptile)
 {
-  return (count_terrain_near_tile(ptile, TRUE, TRUE, T_HILLS)
-	  + 2 * count_terrain_near_tile(ptile, TRUE, TRUE, T_MOUNTAINS));
+  int sum = 0;
+
+  adjc_iterate(ptile, ptile2) {
+    sum += ptile2->terrain->property[MG_MOUNTAINOUS];
+  } adjc_iterate_end;
+
+  return sum;
 }
 
 /*********************************************************************
@@ -527,7 +627,7 @@ static int river_test_adjacent_highlands(struct tile *ptile)
 *********************************************************************/
 static int river_test_swamp(struct tile *ptile)
 {
-  return (map_get_terrain(ptile) != T_SWAMP) ? 1 : 0;
+  return FC_INFINITY - ptile->terrain->property[MG_WET];
 }
 
 /*********************************************************************
@@ -535,7 +635,13 @@ static int river_test_swamp(struct tile *ptile)
 *********************************************************************/
 static int river_test_adjacent_swamp(struct tile *ptile)
 {
-  return 100 - count_terrain_near_tile(ptile, TRUE, TRUE, T_SWAMP);
+  int sum = 0;
+
+  adjc_iterate(ptile, ptile2) {
+    sum += ptile2->terrain->property[MG_WET];
+  } adjc_iterate_end;
+
+  return FC_INFINITY - sum;
 }
 
 /*********************************************************************
@@ -691,7 +797,7 @@ static bool make_river(struct tile *ptile)
     /* We arbitrarily make rivers end at the poles. */
     if (count_special_near_tile(ptile, TRUE, TRUE, S_RIVER) > 0
 	|| count_ocean_near_tile(ptile, TRUE, TRUE) > 0
-        || (map_get_terrain(ptile) == T_ARCTIC 
+        || (ptile->terrain->property[MG_FROZEN] > 0
 	    && map_colatitude(ptile) < 0.8 * COLD_LEVEL)) { 
 
       freelog(LOG_DEBUG,
@@ -718,6 +824,7 @@ static bool make_river(struct tile *ptile)
       cardinal_adjc_dir_iterate(ptile, tile1, dir) {
 	if (rd_direction_is_valid[dir]) {
 	  rd_comparison_val[dir] = (test_funcs[func_num].func) (tile1);
+	  assert(rd_comparison_val[dir] >= 0);
 	  if (best_val == -1) {
 	    best_val = rd_comparison_val[dir];
 	  } else {
@@ -802,8 +909,6 @@ static void make_rivers(void)
   /* The number of river tiles that have been set. */
   int current_riverlength = 0;
 
-  int i; /* Loop variable. */
-
   /* Counts the number of iterations (should increase with 1 during
      every iteration of the main loop in this function).
      Is needed to stop a potentially infinite loop. */
@@ -812,7 +917,7 @@ static void make_rivers(void)
   create_placed_map(); /* needed bu rand_map_characteristic */
   set_all_ocean_tiles_placed();
 
-  river_map = fc_malloc(sizeof(int) * MAX_MAP_INDEX);
+  river_map = fc_malloc(sizeof(*river_map) * MAP_INDEX_SIZE);
 
   /* The main loop in this function. */
   while (current_riverlength < desirable_riverlength
@@ -827,10 +932,10 @@ static void make_rivers(void)
      */
     if (
 	/* Don't start a river on ocean. */
-	!is_ocean(map_get_terrain(ptile))
+	!is_ocean(tile_get_terrain(ptile))
 
 	/* Don't start a river on river. */
-	&& !map_has_special(ptile, S_RIVER)
+	&& !tile_has_special(ptile, S_RIVER)
 
 	/* Don't start a river on a tile is surrounded by > 1 river +
 	   ocean tile. */
@@ -840,34 +945,27 @@ static void make_rivers(void)
 	/* Don't start a river on a tile that is surrounded by hills or
 	   mountains unless it is hard to find somewhere else to start
 	   it. */
-	&& (count_terrain_near_tile(ptile, TRUE, TRUE, T_HILLS)
-	    + count_terrain_near_tile(ptile, TRUE, TRUE, T_MOUNTAINS) < 90
+	&& (count_terrain_property_near_tile(ptile, TRUE, TRUE,
+					     MG_MOUNTAINOUS) < 90
 	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 5)
 
 	/* Don't start a river on hills unless it is hard to find
 	   somewhere else to start it. */
-	&& (map_get_terrain(ptile) != T_HILLS
+	&& (ptile->terrain->property[MG_MOUNTAINOUS] == 0
 	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 6)
 
-	/* Don't start a river on mountains unless it is hard to find
-	   somewhere else to start it. */
-	&& (map_get_terrain(ptile) != T_MOUNTAINS
-	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 7)
-
 	/* Don't start a river on arctic unless it is hard to find
 	   somewhere else to start it. */
-	&& (map_get_terrain(ptile) != T_ARCTIC
+	&& (ptile->terrain->property[MG_FROZEN] == 0
 	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 8)
 
 	/* Don't start a river on desert unless it is hard to find
 	   somewhere else to start it. */
-	&& (map_get_terrain(ptile) != T_DESERT
+	&& (ptile->terrain->property[MG_DRY] == 0
 	    || iteration_counter >= RIVERS_MAXTRIES / 10 * 9)) {
 
       /* Reset river_map before making a new river. */
-      for (i = 0; i < map.xsize * map.ysize; i++) {
-	river_map[i] = 0;
-      }
+      memset(river_map, 0, MAP_INDEX_SIZE * sizeof(*river_map));
 
       freelog(LOG_DEBUG,
 	      "Found a suitable starting tile for a river at (%d, %d)."
@@ -878,14 +976,16 @@ static void make_rivers(void)
       if (make_river(ptile)) {
 	whole_map_iterate(tile1) {
 	  if (TEST_BIT(rmap(tile1), RS_RIVER)) {
-	    Terrain_type_id t = map_get_terrain(tile1);
+	    struct terrain *pterrain = tile_get_terrain(tile1);
 
-	    if (!terrain_has_flag(t, TER_CAN_HAVE_RIVER)) {
+	    if (!terrain_has_flag(pterrain, TER_CAN_HAVE_RIVER)) {
 	      /* We have to change the terrain to put a river here. */
-	      t = get_flag_terrain(TER_CAN_HAVE_RIVER);
-	      map_set_terrain(tile1, t);
+	      /* FIXME: pick_terrain_by_flag may return NULL
+	       * if there is no match. */
+	      pterrain = pick_terrain_by_flag(TER_CAN_HAVE_RIVER);
+	      tile_set_terrain(tile1, pterrain);
 	    }
-	    map_set_special(tile1, S_RIVER);
+	    tile_set_special(tile1, S_RIVER);
 	    current_riverlength++;
 	    map_set_placed(tile1);
 	    freelog(LOG_DEBUG, "Applied a river to (%d, %d).",
@@ -922,9 +1022,11 @@ static void make_land(void)
   hmap_shore_level = (hmap_max_level * (100 - map.landpercent)) / 100;
   ini_hmap_low_level();
   whole_map_iterate(ptile) {
-    map_set_terrain(ptile, T_UNKNOWN); /* set as oceans count is used */
+    tile_set_terrain(ptile, T_UNKNOWN); /* set as oceans count is used */
     if (hmap(ptile) < hmap_shore_level) {
-      map_set_terrain(ptile, T_OCEAN);
+      int depth = (hmap_shore_level - hmap(ptile)) * 100 / hmap_shore_level;
+
+      tile_set_terrain(ptile, pick_ocean(depth));
     }
   } whole_map_iterate_end;
   if (HAS_POLES) {
@@ -951,16 +1053,16 @@ static void make_land(void)
 **************************************************************************/
 static bool is_tiny_island(struct tile *ptile) 
 {
-  Terrain_type_id t = map_get_terrain(ptile);
+  struct terrain *pterrain = tile_get_terrain(ptile);
 
-  if (is_ocean(t) || t == T_ARCTIC) {
+  if (is_ocean(pterrain) || pterrain->property[MG_FROZEN] > 0) {
     /* The arctic check is needed for iso-maps: the poles may not have
      * any cardinally adjacent land tiles, but that's okay. */
     return FALSE;
   }
 
   cardinal_adjc_iterate(ptile, tile1) {
-    if (!is_ocean(map_get_terrain(tile1))) {
+    if (!is_ocean(tile_get_terrain(tile1))) {
       return FALSE;
     }
   } cardinal_adjc_iterate_end;
@@ -975,9 +1077,9 @@ static void remove_tiny_islands(void)
 {
   whole_map_iterate(ptile) {
     if (is_tiny_island(ptile)) {
-      map_set_terrain(ptile, T_OCEAN);
-      map_clear_special(ptile, S_RIVER);
-      map_set_continent(ptile, 0);
+      tile_set_terrain(ptile, pick_ocean(0));
+      tile_clear_special(ptile, S_RIVER);
+      tile_set_continent(ptile, 0);
     }
   } whole_map_iterate_end;
 }
@@ -992,24 +1094,24 @@ static void print_mapgen_map(void)
   int terrain_count[T_COUNT];
   int total = 0;
 
-  terrain_type_iterate(t) {
-    terrain_count[t] = 0;
+  terrain_type_iterate(pterrain) {
+    terrain_count[pterrain->index] = 0;
   } terrain_type_iterate_end;
 
   whole_map_iterate(ptile) {
-    Terrain_type_id t = map_get_terrain(ptile);
+    struct terrain *pterrain = tile_get_terrain(ptile);
 
-    assert(t >= 0 && t < T_COUNT);
-    terrain_count[t]++;
-    if (!is_ocean(t)) {
+    terrain_count[pterrain->index]++;
+    if (!is_ocean(pterrain)) {
       total++;
     }
   } whole_map_iterate_end;
 
-  terrain_type_iterate(t) {
+  terrain_type_iterate(pterrain) {
     freelog(loglevel, "%20s : %4d %d%%  ",
-	    get_terrain_name(t), terrain_count[t],
-	    (terrain_count[t] * 100 + 50) / total);
+	    terrain_rule_name(pterrain),
+	    terrain_count[pterrain->index],
+	    (terrain_count[pterrain->index] * 100 + 50) / total);
   } terrain_type_iterate_end;
 }
 
@@ -1031,8 +1133,12 @@ void map_fractal_generate(bool autosize)
   /* save the current random state: */
   RANDOM_STATE rstate = get_myrand_state();
  
-  if (map.seed==0)
+  if (map.seed == 0) {
+    /* Create a "random" map seed.  Note the call to myrand() which will
+     * depend on the game seed. */
     map.seed = (myrand(MAX_UINT32) ^ time(NULL)) & (MAX_UINT32 >> 1);
+    freelog(LOG_DEBUG, "Setting map.seed:%d", map.seed);
+  }
 
   mysrand(map.seed);
 
@@ -1063,12 +1169,12 @@ void map_fractal_generate(bool autosize)
     if (map.generator == 2) {
       make_pseudofractal1_hmap(1 + ((map.startpos == 0
 				     || map.startpos == 3)
-				    ? 0 : game.nplayers));
+				    ? 0 : game.info.nplayers));
     }
 
     if (map.generator == 1) {
-      make_random_hmap(MAX(1, 1 + SQSIZE 
-			   - (map.startpos ? game.nplayers / 4 : 0)));
+      make_random_hmap(MAX(1, 1 + get_sqsize() 
+			   - (map.startpos ? game.info.nplayers / 4 : 0)));
     }
 
     /* if hmap only generator make anything else */
@@ -1087,8 +1193,8 @@ void map_fractal_generate(bool autosize)
   }
   
   /* some scenarios already provide specials */
-  if (!map.have_specials) {
-    add_specials(map.riches);
+  if (!map.have_resources) {
+    add_resources(map.riches);
   }
 
   if (!map.have_huts) {
@@ -1152,6 +1258,7 @@ void map_fractal_generate(bool autosize)
   }
 
   assign_continent_numbers(FALSE);
+  game_map_init();
 
   print_mapgen_map();
 }
@@ -1191,7 +1298,7 @@ static void adjust_terrain_param(void)
 static bool near_safe_tiles(struct tile *ptile)
 {
   square_iterate(ptile, 1, tile1) {
-    if (!terrain_has_flag(map_get_terrain(tile1), TER_UNSAFE_COAST)) {
+    if (!terrain_has_flag(tile_get_terrain(tile1), TER_UNSAFE_COAST)) {
       return TRUE;
     }	
   } square_iterate_end;
@@ -1215,11 +1322,11 @@ static void make_huts(int number)
     /* Add a hut.  But not on a polar area, on an ocean, or too close to
      * another hut. */
     if ((ptile = rand_map_pos_characteristic(WC_ALL, TT_NFROZEN, MC_NONE))) {
-      if (is_ocean(map_get_terrain(ptile))) {
+      if (is_ocean(tile_get_terrain(ptile))) {
 	map_set_placed(ptile); /* not good for a hut */
       } else {
 	number--;
-	map_set_special(ptile, S_HUT);
+	tile_set_special(ptile, S_HUT);
 	set_placed_near_pos(ptile, 3);
       }
     }
@@ -1228,14 +1335,13 @@ static void make_huts(int number)
 }
 
 /****************************************************************************
-  Return TRUE iff there's a special (i.e., SPECIAL_1 or SPECIAL_2) within
-  1 tile of the given map position.
+  Return TRUE iff there's a resource within one tile of the given map
+  position.
 ****************************************************************************/
-static bool is_special_close(struct tile *ptile)
+static bool is_resource_close(const struct tile *ptile)
 {
   square_iterate(ptile, 1, tile1) {
-    if (map_has_special(tile1, S_SPECIAL_1)
-	|| map_has_special(tile1, S_SPECIAL_2)) {
+    if (tile1->resource) {
       return TRUE;
     }
   } square_iterate_end;
@@ -1246,35 +1352,31 @@ static bool is_special_close(struct tile *ptile)
 /****************************************************************************
   Add specials to the map with given probability (out of 1000).
 ****************************************************************************/
-static void add_specials(int prob)
+static void add_resources(int prob)
 {
-  Terrain_type_id ttype;
-
   whole_map_iterate(ptile)  {
-    ttype = map_get_terrain(ptile);
-    if (!is_ocean(ttype)
-	&& !is_special_close(ptile) 
-	&& myrand(1000) < prob) {
-      if (get_tile_type(ttype)->special_1_name[0] != '\0'
-	  && (get_tile_type(ttype)->special_2_name[0] == '\0'
-	      || (myrand(100) < 50))) {
-	map_set_special(ptile, S_SPECIAL_1);
-      } else if (get_tile_type(ttype)->special_2_name[0] != '\0') {
-	map_set_special(ptile, S_SPECIAL_2);
-      }
-    } else if (is_ocean(ttype) && near_safe_tiles(ptile) 
-	       && myrand(1000) < prob && !is_special_close(ptile)) {
-      if (get_tile_type(ttype)->special_1_name[0] != '\0'
-	  && (get_tile_type(ttype)->special_2_name[0] == '\0'
-	      || (myrand(100) < 50))) {
-        map_set_special(ptile, S_SPECIAL_1);
-      } else if (get_tile_type(ttype)->special_2_name[0] != '\0') {
-	map_set_special(ptile, S_SPECIAL_2);
+    const struct terrain *pterrain = tile_get_terrain(ptile);
+
+    if (is_resource_close (ptile) || myrand (1000) > prob) {
+      continue;
+    }
+    if (!is_ocean(pterrain) || near_safe_tiles (ptile)) {
+      int i = 0;
+      struct resource **r;
+
+      for (r = pterrain->resources; *r; r++) {
+	/* This is a standard way to get a random element from the
+	 * pterrain->resources list, without computing its length in
+	 * advance. Note that if *(pterrain->resources) == NULL, then
+	 * this loop is a no-op. */
+	if (!myrand (++i)) {
+	  tile_set_resource(ptile, *r);
+	}
       }
     }
   } whole_map_iterate_end;
   
-  map.have_specials = TRUE;
+  map.have_resources = TRUE;
 }
 
 /**************************************************************************
@@ -1311,10 +1413,10 @@ static struct tile *get_random_map_position_from_state(
 static void fill_island(int coast, long int *bucket,
 			int warm0_weight, int warm1_weight,
 			int cold0_weight, int cold1_weight,
-			Terrain_type_id warm0,
-			Terrain_type_id warm1,
-			Terrain_type_id cold0,
-			Terrain_type_id cold1,
+			struct terrain *warm0,
+			struct terrain *warm1,
+			struct terrain *cold0,
+			struct terrain *cold1,
 			const struct gen234_state *const pstate)
 {
   int i, k, capac;
@@ -1338,7 +1440,7 @@ static void fill_island(int coast, long int *bucket,
   while (i > 0 && (failsafe--) > 0) {
     struct tile *ptile =  get_random_map_position_from_state(pstate);
 
-    if (map_get_continent(ptile) == pstate->isleindex &&
+    if (tile_get_continent(ptile) == pstate->isleindex &&
 	not_placed(ptile)) {
 
       /* the first condition helps make terrain more contiguous,
@@ -1352,12 +1454,12 @@ static void fill_island(int coast, long int *bucket,
 	     )
 	   &&( !is_cardinally_adj_to_ocean(ptile) || myrand(100) < coast )) {
 	if (map_colatitude(ptile) < COLD_LEVEL) {
-	  map_set_terrain(ptile, (myrand(cold0_weight
+	  tile_set_terrain(ptile, (myrand(cold0_weight
 					+ cold1_weight) < cold0_weight) 
 			  ? cold0 : cold1);
 	  map_set_placed(ptile);
 	} else {
-	  map_set_terrain(ptile, (myrand(warm0_weight
+	  tile_set_terrain(ptile, (myrand(warm0_weight
 					+ warm1_weight) < warm0_weight) 
 			  ? warm0 : warm1);
 	  map_set_placed(ptile);
@@ -1393,7 +1495,7 @@ static void fill_island_rivers(int coast, long int *bucket,
 
   while (i > 0 && (failsafe--) > 0) {
     struct tile *ptile = get_random_map_position_from_state(pstate);
-    if (map_get_continent(ptile) == pstate->isleindex
+    if (tile_get_continent(ptile) == pstate->isleindex
 	&& not_placed(ptile)) {
 
       /* the first condition helps make terrain more contiguous,
@@ -1405,7 +1507,7 @@ static void fill_island_rivers(int coast, long int *bucket,
 	if (is_water_adjacent_to_tile(ptile)
 	    && count_ocean_near_tile(ptile, FALSE, TRUE) < 50
             && count_special_near_tile(ptile, FALSE, TRUE, S_RIVER) < 35) {
-	  map_set_special(ptile, S_RIVER);
+	  tile_set_special(ptile, S_RIVER);
 	  i--;
 	}
       }
@@ -1421,7 +1523,8 @@ static bool is_near_land(struct tile *ptile)
 {
   /* Note this function may sometimes be called on land tiles. */
   adjc_iterate(ptile, tile1) {
-    if (!is_ocean(map_get_terrain(tile1))) {
+    if (tile1->terrain != T_UNKNOWN
+	&& !is_ocean(tile_get_terrain(tile1))) {
       return TRUE;
     }
   } adjc_iterate_end;
@@ -1485,10 +1588,10 @@ static bool place_island(struct gen234_state *pstate)
 	  return i != 0;
 	}
 
-        map_set_terrain(tile1, T_UNKNOWN);
+        tile_set_terrain(tile1, T_UNKNOWN);
 	map_unset_placed(tile1);
 
-	map_set_continent(tile1, pstate->isleindex);
+	tile_set_continent(tile1, pstate->isleindex);
         i++;
       }
     }
@@ -1526,7 +1629,7 @@ static bool create_island(int islemass, struct gen234_state *pstate)
   bool j;
   struct tile *ptile = native_pos_to_tile(map.xsize / 2, map.ysize / 2);
 
-  memset(height_map, '\0', sizeof(int) * map.xsize * map.ysize);
+  memset(height_map, '\0', MAP_INDEX_SIZE * sizeof(*height_map));
   hmap(native_pos_to_tile(map.xsize / 2, map.ysize / 2)) = 1;
   pstate->n = ptile->nat_y - 1;
   pstate->w = ptile->nat_x - 1;
@@ -1624,11 +1727,6 @@ static bool make_island(int islemass, int starters,
     /* makes the islands this big */
     islemass = islemass - balance;
 
-    /* don't create continents without a number */
-    if (pstate->isleindex >= MAP_NCONT) {
-      return FALSE;
-    }
-
     if (islemass > lastplaced + 1 + lastplaced / 50) {
       /* don't create big isles we can't place */
       islemass = lastplaced + 1 + lastplaced / 50;
@@ -1678,22 +1776,34 @@ static bool make_island(int islemass, int starters,
     mountbuck += mountain_pct * i;
     fill_island(20, &mountbuck,
 		3, 1, 3,1,
-		T_HILLS, T_MOUNTAINS, T_HILLS, T_MOUNTAINS,
+		pick_terrain(MG_MOUNTAINOUS, MG_GREEN, MG_LAST),
+		pick_terrain(MG_MOUNTAINOUS, MG_LAST, MG_GREEN),
+		pick_terrain(MG_MOUNTAINOUS, MG_GREEN, MG_LAST),
+		pick_terrain(MG_MOUNTAINOUS, MG_LAST, MG_GREEN),
 		pstate);
     desertbuck += desert_pct * i;
     fill_island(40, &desertbuck,
 		1, 1, 1, 1,
-		T_DESERT, T_DESERT, T_DESERT, T_TUNDRA,
+		pick_terrain(MG_DRY, MG_TROPICAL, MG_LAST),
+		pick_terrain(MG_DRY, MG_TROPICAL, MG_LAST),
+		pick_terrain(MG_DRY, MG_TROPICAL, MG_LAST),
+		pick_terrain(MG_COLD, MG_LAST, MG_LAST),
 		pstate);
     forestbuck += forest_pct * i;
     fill_island(60, &forestbuck,
 		forest_pct, swamp_pct, forest_pct, swamp_pct,
-		T_FOREST, T_JUNGLE, T_FOREST, T_TUNDRA,
+		pick_terrain(MG_FOLIAGE, MG_TEMPERATE, MG_LAST),
+		pick_terrain(MG_FOLIAGE, MG_TROPICAL, MG_COLD),
+		pick_terrain(MG_FOLIAGE, MG_TEMPERATE, MG_LAST),
+		pick_terrain(MG_FOLIAGE, MG_TROPICAL, MG_COLD),
 		pstate);
     swampbuck += swamp_pct * i;
     fill_island(80, &swampbuck,
 		1, 1, 1, 1,
-		T_SWAMP, T_SWAMP, T_SWAMP, T_SWAMP,
+		pick_terrain(MG_WET, MG_LAST, MG_FOLIAGE),
+		pick_terrain(MG_WET, MG_LAST, MG_FOLIAGE),
+		pick_terrain(MG_WET, MG_LAST, MG_FOLIAGE),
+		pick_terrain(MG_WET, MG_LAST, MG_FOLIAGE),
 		pstate);
 
     pstate->isleindex++;
@@ -1707,16 +1817,16 @@ static bool make_island(int islemass, int starters,
 **************************************************************************/
 static void initworld(struct gen234_state *pstate)
 {
-  height_map = fc_malloc(sizeof(int) * map.ysize * map.xsize);
+  height_map = fc_malloc(MAP_INDEX_SIZE * sizeof(*height_map));
   create_placed_map(); /* land tiles which aren't placed yet */
   create_tmap(FALSE);
   
   whole_map_iterate(ptile) {
-    map_set_terrain(ptile, T_OCEAN);
-    map_set_continent(ptile, 0);
+    tile_set_terrain(ptile, pick_ocean(100));
+    tile_set_continent(ptile, 0);
     map_set_placed(ptile); /* not a land tile */
-    map_clear_all_specials(ptile);
-    map_set_owner(ptile, NULL);
+    tile_clear_all_specials(ptile);
+    tile_set_owner(ptile, NULL);
   } whole_map_iterate_end;
   
   if (HAS_POLES) {
@@ -1760,7 +1870,7 @@ static void mapgenerator2(void)
 
   pstate->totalmass = ((map.ysize - 6 - spares) * map.landpercent 
                        * (map.xsize - spares)) / 100;
-  totalweight = 100 * game.nplayers;
+  totalweight = 100 * game.info.nplayers;
 
   assert(!placed_map_is_initialized());
 
@@ -1775,7 +1885,7 @@ static void mapgenerator2(void)
     initworld(pstate);
     
     /* Create one big island for each player. */
-    for (i = game.nplayers; i > 0; i--) {
+    for (i = game.info.nplayers; i > 0; i--) {
       if (!make_island(bigfrac * pstate->totalmass / totalweight,
                       1, pstate, 95)) {
 	/* we couldn't make an island at least 95% as big as we wanted,
@@ -1811,10 +1921,10 @@ static void mapgenerator2(void)
 
   /* Now place smaller islands, but don't worry if they're small,
    * or even non-existent. One medium and one small per player. */
-  for (i = game.nplayers; i > 0; i--) {
+  for (i = game.info.nplayers; i > 0; i--) {
     make_island(midfrac * pstate->totalmass / totalweight, 0, pstate, DMSIS);
   }
-  for (i = game.nplayers; i > 0; i--) {
+  for (i = game.info.nplayers; i > 0; i--) {
     make_island(smallfrac * pstate->totalmass / totalweight, 0, pstate, DMSIS);
   }
 
@@ -1851,11 +1961,11 @@ static void mapgenerator3(void)
       ((map.ysize - 6 - spares) * map.landpercent * (map.xsize - spares)) /
       100;
 
-  bigislands= game.nplayers;
+  bigislands= game.info.nplayers;
 
   landmass = (map.xsize * (map.ysize - 6) * map.landpercent)/100;
   /* subtracting the arctics */
-  if (landmass > 3 * map.ysize + game.nplayers * 3){
+  if (landmass > 3 * map.ysize + game.info.nplayers * 3){
     landmass -= 3 * map.ysize;
   }
 
@@ -1864,7 +1974,7 @@ static void mapgenerator3(void)
   if (islandmass < 4 * maxmassdiv6) {
     islandmass = (landmass)/(2 * bigislands);
   }
-  if (islandmass < 3 * maxmassdiv6 && game.nplayers * 2 < landmass) {
+  if (islandmass < 3 * maxmassdiv6 && game.info.nplayers * 2 < landmass) {
     islandmass= (landmass)/(bigislands);
   }
 
@@ -1898,19 +2008,18 @@ static void mapgenerator3(void)
     islandmass= 2;
   }
 
-  while (pstate->isleindex <= MAP_NCONT - 20 && checkmass > islandmass &&
-         ++j < 1500) {
-      if (j < 1000) {
-	size = myrand((islandmass+1)/2+1)+islandmass/2;
-      } else {
-	size = myrand((islandmass+1)/2+1);
-      }
-      if (size < 2) {
-        size=2;
-      }
+  while (checkmass > islandmass && ++j < 1500) {
+    if (j < 1000) {
+      size = myrand((islandmass+1)/2+1)+islandmass/2;
+    } else {
+      size = myrand((islandmass+1)/2+1);
+    }
+    if (size < 2) {
+      size=2;
+    }
 
-      make_island(size, (pstate->isleindex - 2 <= game.nplayers) ? 1 : 0,
-		  pstate, DMSIS);
+    make_island(size, (pstate->isleindex - 2 <= game.info.nplayers) ? 1 : 0,
+		pstate, DMSIS);
   }
 
   make_plains();  
@@ -1940,7 +2049,7 @@ static void mapgenerator4(void)
 
   /* no islands with mass >> sqr(min(xsize,ysize)) */
 
-  if (game.nplayers < 2 || map.landpercent > 80) {
+  if (game.info.nplayers < 2 || map.landpercent > 80) {
     map.startpos = 1;
     return;
   }
@@ -1960,12 +2069,12 @@ static void mapgenerator4(void)
       100;
 
   /*!PS: The weights NEED to sum up to totalweight (dammit) */
-  totalweight = (30 + bigweight) * game.nplayers;
+  totalweight = (30 + bigweight) * game.info.nplayers;
 
   initworld(pstate);
 
-  i = game.nplayers / 2;
-  if ((game.nplayers % 2) == 1) {
+  i = game.info.nplayers / 2;
+  if ((game.info.nplayers % 2) == 1) {
     make_island(bigweight * 3 * pstate->totalmass / totalweight, 3, 
 		pstate, DMSIS);
   } else {
@@ -1975,10 +2084,10 @@ static void mapgenerator4(void)
     make_island(bigweight * 2 * pstate->totalmass / totalweight, 2,
 		pstate, DMSIS);
   }
-  for (i = game.nplayers; i > 0; i--) {
+  for (i = game.info.nplayers; i > 0; i--) {
     make_island(20 * pstate->totalmass / totalweight, 0, pstate, DMSIS);
   }
-  for (i = game.nplayers; i > 0; i--) {
+  for (i = game.info.nplayers; i > 0; i--) {
     make_island(10 * pstate->totalmass / totalweight, 0, pstate, DMSIS);
   }
   make_plains();  
