@@ -842,6 +842,7 @@ static void package_player_common(struct player *plr,
   packet->is_male=plr->is_male;
   packet->team = plr->team ? plr->team->index : -1;
   packet->is_ready = plr->is_ready;
+  packet->was_created = plr->was_created;
   if (city_styles != NULL) {
     packet->city_style = city_style_of_player(plr);
   } else {
@@ -1203,9 +1204,9 @@ void maybe_make_contact(struct tile *ptile, struct player *pplayer)
     if (pcity) {
       make_contact(pplayer, city_owner(pcity), ptile);
     }
-    unit_list_iterate(tile1->units, punit) {
+    unit_list_iterate_safe(tile1->units, punit) {
       make_contact(pplayer, unit_owner(punit), ptile);
-    } unit_list_iterate_end;
+    } unit_list_iterate_safe_end;
   } square_iterate_end;
 }
 
@@ -1652,6 +1653,13 @@ void civil_war(struct player *pplayer)
             "players"), player_name(pplayer));
     return;
   }
+  if (normal_player_count() >= game.playable_nations) {
+    /* No nation for additional player */
+    freelog(LOG_NORMAL,
+            _("Could not throw %s into civil war - no available nations"),
+            nation_plural_for_player(pplayer));
+    return;
+  }
 
   cplayer = split_player(pplayer);
 
@@ -1755,3 +1763,19 @@ void handle_player_phase_done(struct player *pplayer,
 
   send_player_info(pplayer, NULL);
 }
+
+/**************************************************************************
+  Return the number of barbarian players.
+**************************************************************************/
+int barbarian_count(void)
+{
+  return game.info.nbarbarians;
+}
+
+/**************************************************************************
+  Return the number of non-barbarian players.
+**************************************************************************/
+int normal_player_count(void)
+{
+  return game.info.nplayers - game.info.nbarbarians;
+}
