@@ -19,17 +19,20 @@
 #include <stdio.h>
 #include <string.h>
 
-#include "events.h"
 #include "fciconv.h"
 #include "fcintl.h"
-#include "game.h"
-#include "government.h"
 #include "log.h"
 #include "mem.h"
-#include "packets.h"
-#include "player.h"
 #include "rand.h"
 #include "support.h"
+
+#include "events.h"
+#include "game.h"
+#include "government.h"
+#include "packets.h"
+#include "player.h"
+#include "specialist.h"
+#include "unitlist.h"
 #include "version.h"
 
 #include "citytools.h"
@@ -94,9 +97,12 @@ static const char *area_to_text(int value);
 static const char *percent_to_text(int value);
 static const char *production_to_text(int value);
 static const char *economics_to_text(int value);
+static const char *science_to_text(int value);
 static const char *mil_service_to_text(int value);
 static const char *pollution_to_text(int value);
 
+#define GOOD_PLAYER(p) ((p)->is_alive && !is_barbarian(p))
+
 /*
  * Describes a row.
  */
@@ -110,7 +116,7 @@ static struct dem_row {
   {'N', N_("Population"),       get_population,  population_to_text,  TRUE },
   {'A', N_("Land Area"),        get_landarea,    area_to_text,        TRUE },
   {'S', N_("Settled Area"),     get_settledarea, area_to_text,        TRUE },
-  {'R', N_("Research Speed"),   get_research,    percent_to_text,     TRUE },
+  {'R', N_("Research Speed"),   get_research,    science_to_text,     TRUE },
   {'L', N_("Literacy"),         get_literacy,    percent_to_text,     TRUE },
   {'P', N_("Production"),       get_production,  production_to_text,  TRUE },
   {'E', N_("Economics"),        get_economics,   economics_to_text,   TRUE },
@@ -118,24 +124,17 @@ static struct dem_row {
   {'O', N_("Pollution"),        get_pollution,   pollution_to_text,   FALSE }
 };
 
+/* Demographics columns. */
 enum dem_flag {
   DEM_COL_QUANTITY,
   DEM_COL_RANK,
-  DEM_COL_BEST
+  DEM_COL_BEST,
+  DEM_COL_LAST
 };
-
-/*
- * Describes a column.
- */
-static struct dem_col
-{
+BV_DEFINE(bv_cols, DEM_COL_LAST);
+static struct dem_col {
   char key;
-  enum dem_flag flag;
-} coltable[] = {
-    { 'q', DEM_COL_QUANTITY },
-    { 'r', DEM_COL_RANK },
-    { 'b', DEM_COL_BEST }
-};
+} coltable[] = {{'q'}, {'r'}, {'b'}}; /* Corresponds to dem_flag enum */
 
 /**************************************************************************
 ...
@@ -163,16 +162,17 @@ static void historian_generic(enum historian_type which_news)
   int i, j = 0, rank = 0;
   char buffer[4096];
   char title[1024];
-  struct player_score_entry size[game.nplayers];
+  struct player_score_entry size[player_count()];
 
   players_iterate(pplayer) {
-    if (pplayer->is_alive && !is_barbarian(pplayer)) {
+    if (GOOD_PLAYER(pplayer)) {
       switch(which_news) {
       case HISTORIAN_RICHEST:
 	size[j].value = pplayer->economic.gold;
 	break;
       case HISTORIAN_ADVANCED:
-	size[j].value = (pplayer->score.techs + pplayer->future_tech);
+	size[j].value
+	  = pplayer->score.techs + get_player_research(pplayer)->future_tech;
 	break;
       case HISTORIAN_MILITARY:
 	size[j].value = pplayer->score.units;
@@ -188,22 +188,24 @@ static void historian_generic(enum historian_type which_news)
       }
       size[j].player = pplayer;
       j++;
-    } /* else the player is dead or barbarian */
+    } /* else the player is dead or barbarian or observer */
   } players_iterate_end;
 
-  qsort(size, j, sizeof(struct player_score_entry), secompare);
+  qsort(size, j, sizeof(size[0]), secompare);
   buffer[0] = '\0';
   for (i = 0; i < j; i++) {
     if (i == 0 || size[i].value < size[i - 1].value) {
       rank = i;
     }
     cat_snprintf(buffer, sizeof(buffer),
-		 _("%2d: The %s %s\n"), rank + 1, _(greatness[rank]),
-		 get_nation_name_plural(size[i].player->nation));
+		 _("%2d: The %s %s\n"),
+		 rank + 1,
+		 _(greatness[rank]),
+		 nation_plural_for_player(size[i].player));
   }
   my_snprintf(title, sizeof(title), _(historian_message[which_news]),
     _(historian_name[myrand(ARRAY_SIZE(historian_name))]));
-  page_conn_etype(&game.game_connections, _("Historian Publishes!"),
+  page_conn_etype(game.est_connections, _("Historian Publishes!"),
 		  title, buffer, E_BROADCAST_REPORT);
 }
 
@@ -215,7 +217,7 @@ static int nr_wonders(struct city *pcity)
   int result = 0;
 
   built_impr_iterate(pcity, i) {
-    if (is_wonder(i)) {
+    if (is_great_wonder(i)) {
       result++;
     }
   } built_impr_iterate_end;
@@ -247,8 +249,7 @@ void report_top_five_cities(struct conn_list *dest)
       if (value_of_pcity > size[NUM_BEST_CITIES - 1].value) {
 	size[NUM_BEST_CITIES - 1].value = value_of_pcity;
 	size[NUM_BEST_CITIES - 1].city = pcity;
-	qsort(size, NUM_BEST_CITIES, sizeof(struct player_score_entry),
-	      secompare);
+	qsort(size, NUM_BEST_CITIES, sizeof(size[0]), secompare);
       }
     } city_list_iterate_end;
   } players_iterate_end;
@@ -267,8 +268,9 @@ void report_top_five_cities(struct conn_list *dest)
 
     cat_snprintf(buffer, sizeof(buffer),
 		 _("%2d: The %s City of %s of size %d, "), i + 1,
-		 get_nation_name(city_owner(size[i].city)->nation),
-		 size[i].city->name, size[i].city->size);
+		 nation_adjective_for_player(city_owner(size[i].city)),
+		 city_name(size[i].city),
+		 size[i].city->size);
 
     wonders = nr_wonders(size[i].city);
     if (wonders == 0) {
@@ -293,29 +295,31 @@ void report_wonders_of_the_world(struct conn_list *dest)
   buffer[0] = '\0';
 
   impr_type_iterate(i) {
-    if (is_wonder(i)) {
-      struct city *pcity = find_city_wonder(i);
+    if (is_great_wonder(i)) {
+      struct city *pcity = find_city_from_great_wonder(i);
 
       if (pcity) {
 	cat_snprintf(buffer, sizeof(buffer), _("%s in %s (%s)\n"),
-		     get_impr_name_ex(pcity, i), pcity->name,
-		     get_nation_name(city_owner(pcity)->nation));
-      } else if(game.global_wonders[i] != 0) {
+		     get_impr_name_ex(pcity, i),
+		     city_name(pcity),
+		     nation_adjective_for_player(city_owner(pcity)));
+      } else if (great_wonder_was_built(i)) {
 	cat_snprintf(buffer, sizeof(buffer), _("%s has been DESTROYED\n"),
-		     get_improvement_type(i)->name);
+		     improvement_name_translation(i));
       }
     }
   } impr_type_iterate_end;
 
   impr_type_iterate(i) {
-    if (is_wonder(i)) {
+    if (is_great_wonder(i)) {
       players_iterate(pplayer) {
 	city_list_iterate(pplayer->cities, pcity) {
-	  if (pcity->currently_building == i && !pcity->is_building_unit) {
+	  if (pcity->production.value == i && !pcity->production.is_unit) {
 	    cat_snprintf(buffer, sizeof(buffer),
 			 _("(building %s in %s (%s))\n"),
-			 get_improvement_type(i)->name, pcity->name,
-			 get_nation_name(pplayer->nation));
+			 improvement_name_translation(i),
+			 city_name(pcity),
+			 nation_adjective_for_player(pplayer));
 	  }
 	} city_list_iterate_end;
       } players_iterate_end;
@@ -351,7 +355,7 @@ static int get_settledarea(struct player *pplayer)
 
 static int get_research(struct player *pplayer)
 {
-  return (pplayer->score.techout * 100) / total_bulbs_required(pplayer);
+  return pplayer->score.techout;
 }
 
 static int get_literacy(struct player *pplayer)
@@ -417,7 +421,7 @@ static int get_settlers(struct player *pplayer)
 
   /* count up settlers */
   unit_list_iterate(pplayer->units, punit) {
-    if (unit_flag(punit, F_CITIES)) {
+    if (unit_has_type_flag(punit, F_CITIES)) {
       result++;
     }
   } unit_list_iterate_end;
@@ -493,24 +497,20 @@ static int get_unhappypop(struct player *pplayer)
   return pplayer->score.unhappy;
 }
 
-static int get_taxmen(struct player *pplayer)
+static int get_specialists(struct player *pplayer)
 {
-  return pplayer->score.taxmen;
-}
+  int count = 0;
 
-static int get_scientists(struct player *pplayer)
-{
-  return pplayer->score.scientists;
-}
+  specialist_type_iterate(sp) {
+    count += pplayer->score.specialists[sp];
+  } specialist_type_iterate_end;
 
-static int get_elvis(struct player *pplayer)
-{
-  return pplayer->score.elvis;
+  return count;
 }
 
 static int get_gov(struct player *pplayer)
 {
-  return pplayer->government;
+  return government_of_player(pplayer)->index;
 }
 
 static int get_corruption(struct player *pplayer)
@@ -518,7 +518,7 @@ static int get_corruption(struct player *pplayer)
   int result = 0;
 
   city_list_iterate(pplayer->cities, pcity) {
-    result += pcity->corruption;
+    result += pcity->waste[O_TRADE];
   } city_list_iterate_end;
 
   return result;
@@ -562,6 +562,11 @@ static const char *economics_to_text(int value)
   return value_units(value, _(" M goods"));
 }
 
+static const char *science_to_text(int value)
+{
+  return value_units(value, _(" bulbs"));
+}
+
 static const char *mil_service_to_text(int value)
 {
   return value_units(value, PL_(" month", " months", value));
@@ -600,26 +605,26 @@ static const char *number_to_ordinal_string(int num)
 **************************************************************************/
 static void dem_line_item(char *outptr, size_t out_size,
 			  struct player *pplayer, struct dem_row *prow,
-			  int selcols)
+			  bv_cols selcols)
 {
-  if (TEST_BIT(selcols, DEM_COL_QUANTITY)) {
+  if (BV_ISSET(selcols, DEM_COL_QUANTITY)) {
     const char *text = prow->to_text(prow->get_value(pplayer));
 
     cat_snprintf(outptr, out_size, " %s", text);
     cat_snprintf(outptr, out_size, "%*s",
-		 18 - get_internal_string_length(text), "");
+                 18 - (int) get_internal_string_length(text), "");
   }
 
-  if (TEST_BIT(selcols, DEM_COL_RANK)) {
+  if (BV_ISSET(selcols, DEM_COL_RANK)) {
     int basis = prow->get_value(pplayer);
     int place = 1;
 
     players_iterate(other) {
-      if (other->is_alive && !is_barbarian(other) &&
-	  ((prow->greater_values_are_better
-	    && prow->get_value(other) > basis)
-	   || (!prow->greater_values_are_better
-	       && prow->get_value(other) < basis))) {
+      if (GOOD_PLAYER(other)
+	  && ((prow->greater_values_are_better
+	       && prow->get_value(other) > basis)
+	      || (!prow->greater_values_are_better
+	          && prow->get_value(other) < basis))) {
 	place++;
       }
     } players_iterate_end;
@@ -627,12 +632,12 @@ static void dem_line_item(char *outptr, size_t out_size,
     cat_snprintf(outptr, out_size, " %6s", number_to_ordinal_string(place));
   }
 
-  if (TEST_BIT(selcols, DEM_COL_BEST)) {
+  if (BV_ISSET(selcols, DEM_COL_BEST)) {
     struct player *best_player = pplayer;
     int best_value = prow->get_value(pplayer);
 
     players_iterate(other) {
-      if (other->is_alive && !is_barbarian(other)) {
+      if (GOOD_PLAYER(other)) {
 	int value = prow->get_value(other);
 
 	if ((prow->greater_values_are_better && value > best_value)
@@ -645,7 +650,7 @@ static void dem_line_item(char *outptr, size_t out_size,
 
     if(player_has_embassy(pplayer, best_player) && (pplayer != best_player)) {
       cat_snprintf(outptr, out_size, "   %s: %s",
-		   get_nation_name_plural(best_player->nation),
+		   nation_plural_for_player(best_player),
 		   prow->to_text(prow->get_value(best_player)));
     }
   }
@@ -669,7 +674,7 @@ bool is_valid_demography(const char *demography, const char **error_string)
     int j;
 
     /* See if the character is a valid column label. */
-    for (j = 0; j < ARRAY_SIZE(coltable); j++) {
+    for (j = 0; j < DEM_COL_LAST; j++) {
       if (demography[i] == coltable[j].key) {
 	found = TRUE;
 	break;
@@ -712,12 +717,15 @@ void report_demographics(struct connection *pconn)
   char buffer[4096];
   unsigned int i;
   bool anyrows;
-  int selcols;
+  bv_cols selcols;
+  int numcols = 0;
 
-  selcols = 0;
-  for (i = 0; i < ARRAY_SIZE(coltable); i++) {
+  BV_CLR_ALL(selcols);
+  assert(ARRAY_SIZE(coltable) == DEM_COL_LAST);
+  for (i = 0; i < DEM_COL_LAST; i++) {
     if (strchr(game.demography, coltable[i].key)) {
-      selcols |= (1u << coltable[i].flag);
+      BV_SET(selcols, i);
+      numcols++;
     }
   }
 
@@ -729,15 +737,16 @@ void report_demographics(struct connection *pconn)
     }
   }
 
-  if (!pplayer || !pplayer->is_alive || !anyrows || selcols == 0) {
-    page_conn(&pconn->self, _("Demographics Report:"),
+  if (!pplayer || !pplayer->is_alive || !anyrows || numcols == 0) {
+    page_conn(pconn->self, _("Demographics Report:"),
 	      _("Sorry, the Demographics report is unavailable."), "");
     return;
   }
 
-  my_snprintf(civbuf, sizeof(civbuf), _("The %s of the %s"),
-	      get_government_name(pplayer->government),
-	      get_nation_name_plural(pplayer->nation));
+  my_snprintf(civbuf, sizeof(civbuf), _("The %s of the %s (%s)"),
+	      government_name_for_player(pplayer),
+	      nation_plural_for_player(pplayer),
+	      textyear(game.info.year));
 
   buffer[0] = '\0';
   for (i = 0; i < ARRAY_SIZE(rowtable); i++) {
@@ -746,13 +755,13 @@ void report_demographics(struct connection *pconn)
 
       cat_snprintf(buffer, sizeof(buffer), "%s", name);
       cat_snprintf(buffer, sizeof(buffer), "%*s",
-		   18 - get_internal_string_length(name), "");
+                   18 - (int) get_internal_string_length(name), "");
       dem_line_item(buffer, sizeof(buffer), pplayer, &rowtable[i], selcols);
       sz_strlcat(buffer, "\n");
     }
   }
 
-  page_conn(&pconn->self, _("Demographics Report:"), civbuf, buffer);
+  page_conn(pconn->self, _("Demographics Report:"), civbuf, buffer);
 }
 
 /**************************************************************************
@@ -856,7 +865,7 @@ static bool scan_score_log(FILE * fp, int *last_turn, char *id,
     return FALSE;
   }
 
-  if (*last_turn + 1 != game.turn) {
+  if (*last_turn + 1 != game.info.turn) {
     freelog(LOG_ERROR, "Scorelog doesn't match savegame!");
     return FALSE;
   }
@@ -909,9 +918,7 @@ static void log_civ_score(void)
     {"happypop",        get_happypop},
     {"contentpop",      get_contentpop},
     {"unhappypop",      get_unhappypop},
-    {"taxmen",          get_taxmen},
-    {"scientists",      get_scientists},
-    {"elvis",           get_elvis},
+    {"specialists",     get_specialists},
     {"gov",             get_gov},
     {"corruption",      get_corruption} /* new 1.11.5 tags end here */
   };
@@ -934,7 +941,7 @@ static void log_civ_score(void)
   }
 
   if (!fp) {
-    if (game.year == GAME_START_YEAR) {
+    if (game.info.year == GAME_START_YEAR) {
       oper = SL_CREATE;
     } else {
       fp = fopen(logname, "r");
@@ -962,7 +969,7 @@ static void log_civ_score(void)
       fprintf(fp, 
 	      "\n"
 	      "# For a specification of the format of this see doc/README.scorelog or \n"
-	      "# <http://www.freeciv.org/lxr/source/doc/README.scorelog?v=cvs>.\n"
+	      "# <http://svn.gna.org/viewcvs/freeciv/trunk/doc/README.scorelog?view=auto>.\n"
 	      "\n");
 
       fprintf(fp, "id %s\n", game.id);
@@ -983,37 +990,38 @@ static void log_civ_score(void)
     }
   }
 
-#define GOOD_PLAYER(p) ((p)->is_alive)
-
-  if (game.turn > last_turn) {
-    fprintf(fp, "turn %d %d %s\n", game.turn, game.year, textyear(game.year));
-    last_turn = game.turn;
+  if (game.info.turn > last_turn) {
+    fprintf(fp, "turn %d %d %s\n", game.info.turn, game.info.year, 
+            textyear(game.info.year));
+    last_turn = game.info.turn;
   }
 
   for (i = 0; i < ARRAY_SIZE(player_names); i++) {
-    if (strlen(player_names[i]) > 0 && !GOOD_PLAYER(get_player(i))) {
-      fprintf(fp, "delplayer %d %d\n", game.turn - 1, i);
+    if (strlen(player_names[i]) > 0 && !GOOD_PLAYER(player_by_number(i))) {
+      fprintf(fp, "delplayer %d %d\n", game.info.turn - 1, i);
       player_names[i][0] = '\0';
     }
   }
 
   players_iterate(pplayer) {
     if (GOOD_PLAYER(pplayer)
-	&& strlen(player_names[pplayer->player_no]) == 0) {
-      fprintf(fp, "addplayer %d %d %s\n", game.turn, pplayer->player_no,
-	      pplayer->name);
-      mystrlcpy(player_name_ptrs[pplayer->player_no], pplayer->name,
+	&& strlen(player_names[player_index(pplayer)]) == 0) {
+      fprintf(fp, "addplayer %d %d %s\n", game.info.turn,
+	      player_number(pplayer),
+	      player_name(pplayer));
+      mystrlcpy(player_name_ptrs[player_index(pplayer)], player_name(pplayer),
 		MAX_LEN_NAME);
     }
   } players_iterate_end;
 
   players_iterate(pplayer) {
     if (GOOD_PLAYER(pplayer)
-	&& strcmp(player_names[pplayer->player_no], pplayer->name) != 0) {
-      fprintf(fp, "delplayer %d %d\n", game.turn - 1, pplayer->player_no);
-      fprintf(fp, "addplayer %d %d %s\n", game.turn, pplayer->player_no,
-	      pplayer->name);
-      mystrlcpy(player_names[pplayer->player_no], pplayer->name,
+	&& strcmp(player_names[player_index(pplayer)], player_name(pplayer)) != 0) {
+      fprintf(fp, "delplayer %d %d\n", game.info.turn - 1, player_number(pplayer));
+      fprintf(fp, "addplayer %d %d %s\n", game.info.turn,
+	      player_number(pplayer),
+	      player_name(pplayer));
+      mystrlcpy(player_names[player_index(pplayer)], player_name(pplayer),
 		MAX_LEN_NAME);
     }
   } players_iterate_end;
@@ -1024,8 +1032,8 @@ static void log_civ_score(void)
 	continue;
       }
 
-      fprintf(fp, "data %d %d %d %d\n", game.turn, i, pplayer->player_no,
-	      score_tags[i].get_value(pplayer));
+      fprintf(fp, "data %d %d %d %d\n", game.info.turn, i,
+	      player_number(pplayer), score_tags[i].get_value(pplayer));
     } players_iterate_end;
   }
 
@@ -1043,72 +1051,27 @@ log_civ_score_disable:
   disabled = TRUE;
 }
 
-#undef GOOD_PLAYER
-
 /**************************************************************************
   ...
 **************************************************************************/
 void make_history_report(void)
 {
-  static enum historian_type report = HISTORIAN_FIRST;
-  static int time_to_report=20;
-
   if (game.scorelog) {
     log_civ_score();
   }
 
-  if (game.nplayers == 1) {
+  if (player_count() == 1) {
     return;
   }
 
-  time_to_report--;
-
-  if (time_to_report > 0) {
+  if (game.scoreturn > game.info.turn) {
     return;
   }
 
-  time_to_report=myrand(20) + 20;
+  game.scoreturn = game.info.turn + GAME_DEFAULT_SCORETURN
+                 + myrand(GAME_DEFAULT_SCORETURN);
 
-  historian_generic(report);
-  
-  report++;
-  if (report > HISTORIAN_LAST) {
-    report = HISTORIAN_FIRST;
-  }
-}
-
-/**************************************************************************
- Inform clients about player scores during a game.
-**************************************************************************/
-void report_progress_scores(void)
-{
-  int i, j = 0;
-  char buffer[4096];
-  struct player_score_entry size[game.nplayers];
-
-  players_iterate(pplayer) {
-    if (!is_barbarian(pplayer)) {
-      size[j].value = get_civ_score(pplayer);
-      size[j].player = pplayer;
-      j++;
-    }
-  } players_iterate_end;
-
-  qsort(size, j, sizeof(struct player_score_entry), secompare);
-  buffer[0] = '\0';
-
-  for (i = 0; i < j; i++) {
-    cat_snprintf(buffer, sizeof(buffer),
-		 PL_("%2d: The %s %s scored %d point\n",
-		     "%2d: The %s %s scored %d points\n",
-		     size[i].value),
-		 i + 1, _(greatness[i]),
-		 get_nation_name_plural(size[i].player->nation),
-		 size[i].value);
-  }
-  page_conn(&game.game_connections,
-	    _("Progress Scores:"),
-	    _("The Greatest Civilizations in the world."), buffer);
+  historian_generic(game.scoreturn % HISTORIAN_LAST);
 }
 
 /**************************************************************************
@@ -1117,22 +1080,22 @@ void report_progress_scores(void)
 void report_final_scores(void)
 {
   int i, j = 0;
-  struct player_score_entry size[game.nplayers];
+  struct player_score_entry size[player_count()];
   struct packet_endgame_report packet;
 
   players_iterate(pplayer) {
-    if (!is_barbarian(pplayer)) {
-      size[j].value = get_civ_score(pplayer);
+    if (GOOD_PLAYER(pplayer)) {
+      size[j].value = pplayer->score.game;
       size[j].player = pplayer;
       j++;
     }
   } players_iterate_end;
 
-  qsort(size, j, sizeof(struct player_score_entry), secompare);
+  qsort(size, j, sizeof(size[0]), secompare);
 
   packet.nscores = j;
   for (i = 0; i < j; i++) {
-    packet.id[i] = size[i].player->player_no;
+    packet.id[i] = player_number(size[i].player);
     packet.score[i] = size[i].value;
     packet.pop[i] = get_pop(size[i].player) * 1000; 
     packet.bnp[i] = get_economics(size[i].player); 
@@ -1148,7 +1111,7 @@ void report_final_scores(void)
     packet.spaceship[i] = get_spaceship(size[i].player); 
   }  
 
-  lsend_packet_endgame_report(&game.game_connections, &packet);
+  lsend_packet_endgame_report(game.est_connections, &packet);
 }	
 
 /**************************************************************************
